/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.WorkItemClient");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.ProjectArea");
dojo.require("com.ibm.rqm.planning.web.client.internal.QueryClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemUtils");


(function() {

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var getCurrentProjectArea	= com.ibm.rqm.planning.web.client.internal.ProjectArea.getCurrentProjectArea;
var ServiceRequest			= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient       = com.ibm.team.repository.web.transport.TeamServerClient;
var WorkItemUtils          = com.ibm.rqm.planning.web.client.internal.WorkItemUtils;
var QueryClient				= com.ibm.rqm.planning.web.client.internal.QueryClient;

function validateWorkItemParams(appArgs, method) {
	
	// validate that we have any request parameters
	if (!appArgs) {
		throw new Error("WorkItemClient#" + method + ": missing request parameters.");
	}
	
	// validate that we have only one of 'id', 'itemId', or 'newWorkItem'
	var count= 0;
	var setStr = "";
	dojo.forEach(["id", "itemId", "newWorkItem"],
		function(x) {
			if (appArgs[x]) { count++; setStr= setStr + " " + x; }
		});
	if (count > 1) {
		throw new Error("WorkItemClient#" + method +
			": may only set one of 'id', 'itemId', or 'newWorkItem' (saw: [" +
			setStr + "]");
	}

}

/**
 * Provides helper methods to fetch workitem-related data from the team server.
 */
com.ibm.rqm.planning.web.client.internal.WorkItemClient= {

	// service URI
	SERVICE_URI: "com.ibm.team.workitem.common.internal.rest.IWorkItemRestService",
	
	// get methods
	GET_VALUES_METHOD: "getValues",
	GET_CONTRIBUTORS_METHOD: "getContributors",

	/*
	 * Invokes the work item REST service specified by 'method' with optional
	 * request parameters specified by 'appArgs'.  Calls back to the functions
	 * defined by 'responseHandler'.
	 */
	_invokeWorkItemService: function(responseHandler, method, appArgs) {
		
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length === 2) {
			var serviceRequest=
					new ServiceRequest(WorkItemClient.SERVICE_URI, method);
		} else {
			serviceRequest=
					new ServiceRequest(WorkItemClient.SERVICE_URI, method, appArgs);
		}
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	getWorkItemEditableProperties: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getWorkItemEditableProperties");
		if (appArgs.newWorkItem) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		WorkItemClient._invokeWorkItemService(responseHandler, "getWorkItemEditableProperties", appArgs);
	},
	
	getAllValues: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getAllValues");
		if (appArgs.newWorkItem) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		WorkItemClient._invokeWorkItemService(responseHandler, "getAllValues", appArgs);
	},
	
	/**
	 * Returns a set of owners; dependent on project area, category, and target.
	 */
	getOwners: function(responseHandler, appArgs) {
		appArgs.attributeName= "owner";
		appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		WorkItemClient._invokeWorkItemService(responseHandler, "getValues", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#getWorkItemDTO.
	 */
	getWorkItemDTO: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getWorkItemDTO");
		if (appArgs.newWorkItem) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		var actualHandler = this.getInternalFixupHandler(responseHandler);
		this._invokeWorkItemService(actualHandler, "getWorkItemDTO2", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#getWorkItemSummaryPage.
	 */
	getWorkItemSummaryPage: function(responseHandler, appArgs) {
		this._invokeWorkItemService(responseHandler, "getWorkItemSummaryPage", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#postWorkItem.
	 */
	postWorkItem: function(responseHandler, appArgs) {
		// in the case of a new work item, add the project area item id
		if (!appArgs.stateId) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		this._invokeWorkItemService(responseHandler, "postWorkItem", appArgs);
	},
	
	getResolutions: function(responseHandler, appArgs) {
		if (!(appArgs.status || appArgs.actionId)) {
			throw new Error("WorkItemClient#getResolutions: must set either appArgs.status or appArgs.actionId (none set)");
		}
		if (appArgs.status && appArgs.actionId) {
			throw new Error("WorkItemClient#getResolutions: must set either appArgs.status or appArgs.actionId (both set)");
		}
		appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		appArgs.attributeName= "resolution";
		this._invokeWorkItemService(responseHandler, WorkItemClient.GET_VALUES_METHOD, appArgs);
	},
	
	getContributors: function(responseHandler, appArgs) {
		this._invokeWorkItemService(responseHandler, WorkItemClient.GET_CONTRIBUTORS_METHOD, appArgs);
	},
	
   getInternalFixupHandler: function(responseHandler){
      var internalHandler = {
         success: function(workItemDTO){
            if (workItemDTO != null) {
               var fixedUpWI = WorkItemUtils.fixupWorkItemDTO(workItemDTO)
               workItemDTO = fixedUpWI;                                                        
            }
            responseHandler.doCallback(workItemDTO);

         },
         failure: function(error){
            responseHandler.doErrorCallback(error);
         }
      };
      var handler = new ServiceResponseHandler(internalHandler, "success", "failure");
      return handler;
   }	
};

// must be defined after com.ibm.team.workitem.web.client.internal.WorkItemClient
var WorkItemClient			= com.ibm.rqm.planning.web.client.internal.WorkItemClient;

})();
