/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog.Attribute");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._AttributeBase");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Pointer");
dojo.declare("com.ibm.asq.common.web.client.internal._catalog.Attribute", com.ibm.asq.common.web.client.internal._catalog._AttributeBase, 
{
    VALIDATION_OK: 0,
    VALIDAIION_INVALID_CHARACTER: 10,
    VALIDATION_INVALID_SEQUENCE: 20,
    VALIDATION_DUPLICATE_OPERATORS: 30,
    VALIDATION_DISJOINTED: 40,
    VALIDATION_INCOMPLETE: 50,
    VALIDATION_EXTRA_COMPONENTS: 60,
    
    constructor: function(document, parent, instanceRoot, templateObj, path){
        if (instanceRoot != null) {
            this.instanceObj = instanceRoot;
        }
        if (path == null) {
            this.path = templateObj.name;
        }
        else {
            this.path = path + "." + templateObj.name;
        }
        if(this.templateObj.type == "ConfigurationPartType"){
            this.partCache = new Array();
            if(this.instanceObj != null) {
                this._buildPartCache();
            }
        }
    },
    // Get the type of the attributes value.  Returns one of 
    // {'StringType', 'NumericType', 'PointerType', 'TimeStampType',
    // 'ConfigurationPartType'}
    getType: function(){
        return this.templateObj.type;
    }, 
    // If the type of the attribute is 'ConfigurationPartType', 
    // this method can be called to get the specific name of 
    // the part.  Then getTemplate(partName) can be called to
    // get the detailed information about part names and 
    // attributes.
    getPartTypeName: function(){
        if(this.templateObj.type != "ConfigurationPartType") {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getPartTypeName called on a non ConfigurationPartType!");
		    throw new Error("getPartTypeName called on a non ConfigurationPartType!");
        }
        return this.templateObj.AssetType.type;
    }, 
	_getTemplate: function(){
	        if(this.templateObj.type != "ConfigurationPartType") {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getTemplate called on a non ConfigurationPartType!");
            throw new Error("must only be called on a config part attribute");
        }
		var cat = new com.ibm.asq.common.web.client.internal.Catalog();
        var templateName = this.templateObj.AssetType.name;
        return cat.getTemplate(templateName);
	},
    // If the type of the attribute is 'ConfigurationPartType', 
    // this method can be called to get all the names of 
    // all parts in the family. 
    getAllPartTypeNames: function(){
        return this._getTemplate().getPartNames();
    }, 
    // If the type of the attribute is 'ConfigurationPartType', 
    // this method can be called to get the root part for the attribute
	// type from the type's template. 
    getRootPart: function(){
        return this._getTemplate().getRoot();
    }, 
    // If the type of the attribute is 'PointerType', 
    // this method can be called to get the pointer 
    // that is associated with this attribute.  If the attribute
    // is an array, this method will return an array of Pointers.
    getPointer: function(){
        if(this.templateObj.type != "PointerType") {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getPointer called on a non PointerType!");
            throw new Error("getPointer called on a non PointerType");
        }
        var ds = null;
        var uuid = null;
        if (this.instanceObj) {
            var val = this.instanceObj.value;
            if (val != null) {
                if(this.isArray()) {
                    var arr = val.DisplayValue;
                    var pointerArray = new Array();
                    for(var i=0; i<arr.length; i++) {
                        ds = val.DisplayValue.displayString;
                        uuid = val.DisplayValue.uuid;
                        pointerArray.push( new com.ibm.asq.common.web.client.internal._catalog.Pointer(ds, uuid));
                    }
                    return pointerArray;
                } else {
                    ds = val.DisplayValue.displayString;
                    uuid = val.DisplayValue.uuid;
                }
            }
        }
        return new com.ibm.asq.common.web.client.internal._catalog.Pointer(ds, uuid);
    }, 
    // Mark the attribute as ignored in a search config
    setIgnored: function(ignore){
        if(this.document instanceof com.ibm.asq.common.web.client.internal._catalog.Asset){
            throw new Error("Can't set ignore flag in an Asset.")
        }
        this.instanceObj.ignore = ignore;
    },
    // Get if the attribute is ignored in a search config
    isIgnored: function(){
        var ignore = this.instanceObj.ignore;
        if(ignore == null){
            ignore = false;
        }
        return ignore;
    },
    // Determine if this attribute can contain an array of 
    // selected values.  If this method is true, the 
    // getArrayValues(), addArrayValue(), getArrayValue(),
    // setArrayValue() and removeArrayValue() methods become
    // valid. If this method is false, the getValue() and 
    // setValue() methods become valid. 
    isArray: function(){
        var cardinality = this._getCardinality();
        if (cardinality == "ZERO_OR_MORE" || cardinality == "ONE_OR_MORE") {
            return true;
        }
        else {
            return false;
        }
    },
    // Get the name of the attribute
    getName: function(){
        return this.templateObj.name;
    },
    // Determine if this attribute is an enumeration. 
    // If this method is true, the getEnumValues
    // method becomes valid.
    isEnum: function(){
        return this.parent._isEnum(this.templateObj.name);
    },
    // Get the values of the enum for this attribute.  This 
    // method is only valid for Enum types.
    getEnumValues: function(){
        return this.parent._getEnumValues(this.templateObj.name);
    },
    // This method can be called to validate an attribute value
    // prior to attempting to set the value in the attribute.
    // This method is really only implemented for numeric types as
    // that type can have a complex search string.  For all other
    // types this method returns VALIDATION_OK (0).  For numeric types a code is returned.
    //    VALIDATION_OK - The value is valid
    //    VALIDAIION_INVALID_CHARACTER - The value contains an invalid character.
    //    VALIDATION_INVALID_SEQUENCE - The value contains segments that are out
    //         of sequence, such an operator followed by an operator.
    //    VALIDATION_DUPLICATE_OPERATORS - The value contains two operators with
    //         the same type.
    //    VALIDATION_DISJOINTED - The search string will never contain results
    //         do to a disjointed pattern (such as "<1 >10").
    //    VALIDATION_INCOMPLETE - Incomplete Sequence.
    //    VALIDATION_EXTRA_COMPONENTS - Too many search components.
    validate: function(value){
        if(value != null && this.templateObj.type == "NumericType") {
            try {
                var arr = this._parseSearchString(value);
                return this._validateSearchArray(arr);
                } 
            catch(e) {
                return this.VALIDAIION_INVALID_CHARACTER;
            }
        } else {
            return this.VALIDATION_OK;
        }
    },    
    _getValue: function(returnNullObjects){
        if(this.instanceObj == null || this.instanceObj.value == null) {
            return null;
        }
        if(returnNullObjects && this.instanceObj.value["null"] == "") {
            return {"null":""};
        }
        if(this.isArray()) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValue called on an array type: " + this.templateObj.name);
            throw new Error("getValue called on an array type: " + this.templateObj.name);
        }
        var type = this.templateObj.type;
        if(type == "StringType") {
            return this.instanceObj.value.string;
        } else if(type == "NumericType") {
            var arr = this.instanceObj.value;
            // arr may not be an array (single value case)
            if(!(arr instanceof Array)){
                return arr["int"];
            }
            var str = "";
            for(var i = 0; i < arr.length; i++) {
                if(str.length > 0){
                    str = str + " ";
                }
                var op = arr[i].operator;
                if(op != null){
                    str = str + op;
                }
                str = str + arr[i]["int"];
            }
            return str;
        } else if(type == "TimeStampType") {
            return this.instanceObj.value.timestamp;
        } else if(type == "PointerType") {
            return this.getPointer();
        } else if(type == "ConfigurationPartType") {
            if(this.partCache.length == 0){
                return null;
            }
            else{
                return(this.partCache[0]);
            }
        }
    },
    // Get the value of the attribute at the given array
    // index.  This method is only valid if the attribute
    // is an array.
    getArrayValue: function(index){
        if(this.instanceObj == null) {
            return null;
        }
        if(!(this.isArray())) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValues called on a non array type: " + this.templateObj.name);
            throw new Error("getValues called on a non array type: " + this.templateObj.name);
        }
         // return the array of values.  This may not be
         // an array type in the obj!
        // Get the type
        var type = this.templateObj.type;
        if(type == "StringType") {
            return this.instanceObj.value.string[index];
        } else if(type == "NumericType") {
            // Note: we don't currently have arrays of numbers
            // This code won't work with arrays of numbers with 
            //operators.
            return this.instanceObj.value["int"][index];
        } else if(type == "TimeStampType") {
            return this.instanceObj.value.timestamp[index];
        } else if(type == "PointerType") {
            return this.getPointer()[index];
        } else if(type == "ConfigurationPartType") {
            return this.partCache[index];
        }
    },
    // Set the value of the attribute at the given array
    // index.  If this method is called with a value of null,
    // the part at the corresponding index will be removed.
    // This method is only valid if the attribute
    // is an array.
    setArrayValue: function(index, value){
        return this._setArrayValue(index, value, true);
    },
    _setArrayValue: function(index, value, setSwap){
        if(value == null) {
            return this.removeArrayValue(index);
        }
        if(this.instanceObj == null) {
            return null;
        }
        if(!(this.isArray())) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValues called on a non array type: " + this.templateObj.name);
            throw new Error("getValues called on a non array type: " + this.templateObj.name);
        }
        // return the array of values.  This may not be
        // an array type in the obj!
        // Get the type
        var type = this.templateObj.type;
        if(type == "StringType") {
            this.instanceObj.value.string[index] = value;
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        } else if(type == "NumericType") {
            // Note: we don't currently have arrays of numbers
            // This code won't work with arrays of numbers with 
            //operators.
            this.instanceObj.value["int"][index] = value;
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        } else if(type == "TimeStampType") {
            this.instanceObj.value.timestamp[index] = value;
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        } else if(type == "PointerType") {
            this.getPointer()[index] = value;
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        } else if(type == "ConfigurationPartType") {
            // Get the part from the cache
            var part = this.partCache[index];
            // Check if new and old are the same
            if(part != null){
                if (part.getName() == value) {
                    // Nothing to do
                    return;
                }
            }
            // get the tag to trasfer to the new part
            var tag = part.tag;
            // Get the part if it had been swapped
            var newPart = null;
            if (setSwap) {
                newPart = this.document._getSwappedPart(this.getName(), value);
                // Save the current part in case it's swapped in later
                this.document._setSwappedPart(this.getName(), part.getName(), part);
            }
            part.instanceRoot.remove = true;
            // Remove the object from the AssetInstance array
            var removed = false;
            var aiArr = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
            for(var i=0;i<aiArr.length && !removed;i++){
                if(aiArr[i].remove == true){
                    var newArr = this._util._removeFromArray(aiArr, i);
                    this.instanceObj.value.AssetInstance = newArr;
                    var ai = this.instanceObj.value.AssetInstance;
                    if(ai.length == 0){
                        delete this.instanceObj.value.AssetInstance;
                    }
                    removed = true;
                }
            }
            // Check if we have this part saved
            if (newPart == null) {
                // Create a json object for the new attribute value
                var templatePart = value;
                if (dojo.isString(templatePart)) {
                    // Get the correct part from the template
                    var template = this._getTemplate();
                    templatePart = template.getPart(value);
                    if (templatePart == null) {
                        console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Part not found in " + this.templateObj.AssetType.type + " template!");
                        throw new Error("Part not found in Template!");
                    }
                }
                // Create a Part for the partCache
                newPart = templatePart._createInstancePart(this.document, null, this.parent.path);
            
                // Set the values from those in the old part
                newPart._setValuesFromPart(part);
            } else {
                // remove the swapped part
                if (setSwap) {
                    this.document._clearSwappedPart();
                }
            }
            newPart.tag = tag;
            this.partCache[index] = newPart;           
            
            // Add the new json to the unresolved
            var val = this.instanceObj.value;
            if (val.AssetInstance == null) {
                val.AssetInstance = new Array();
            }
            
            aiArr = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
            aiArr.push(newPart.instanceRoot);
        }
    },
    // Remove the value of the attribute at the given array
    // index.  This method is only valid if the attribute
    // is an array.  This will re-index all of the values
    // in the array.
    removeArrayValue: function(index){
        if(this.instanceObj == null) {
            return null;
        }
        if(!(this.isArray())) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValues called on a non array type: " + this.templateObj.name);
            throw new Error("getValues called on a non array type: " + this.templateObj.name);
        }
        // return the array of values.  This may not be
        // an array type in the obj!
        // Get the type
        var type = this.templateObj.type;
        if(type == "StringType") {
            var newArr = this._util._removeFromArray(this.instanceObj.value.string, index);
            this.instanceObj.value.string = newArr;
            this.document._clearSwappedPart();
        } else if(type == "NumericType") {
            // Note: we don't currently have arrays of numbers
            // This code won't work with arrays of numbers with 
            //operators.
            var newArr = this._util._removeFromArray(this.instanceObj.value["int"], index);
            this.instanceObj.value["int"] = newArr;
            this.document._clearSwappedPart();
        } else if(type == "TimeStampType") {
            var newArr = this._util._removeFromArray(this.instanceObj.value.timestamp, index);
            this.instanceObj.value.timestamp = newArr;
            this.document._clearSwappedPart();
        } else if(type == "PointerType") {
            var newArr = this._util._removeFromArray(this.instanceObj.value.DisplayValue, index);
            this.instanceObj.value.DisplayValue = newArr;
            this.document._clearSwappedPart();
        } else if(type == "ConfigurationPartType") {
            // Get the part from the cache
            var part = this.partCache[index];
            // Save the part in case it's swapped in later
            this.document._setSwappedPart(this.getName(), part.getName(), part);

            // Remove the object from the AssetInstance array
            part.instanceRoot.remove = true;
            var removed = false;
            var aiArr = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
            for(var i=0;i<aiArr.length && !removed;i++){
                if(aiArr[i].remove == true){
                    var newArr = this._util._removeFromArray(aiArr, index);
                    this.instanceObj.value.AssetInstance = newArr;
                    var ai = this.instanceObj.value.AssetInstance;
                    if(ai.length == 0){
                        delete this.instanceObj.value.AssetInstance;
                    }
                    removed = true;
                }
            }
            // Remove the part from the cache
            this.partCache = this._util._removeFromArray(this.partCache, index);           
        }
    },
    // Get the current values of the attribute.  This method 
    // is only valid if the attribute is an array. This will 
    // always be null for Attributes in Template Parts, but may 
    // be defined for the Attributes in Parts contained in 
    // Configurations and Assets.
    getArrayValues: function(){
        if(this.instanceObj == null) {
            return new Array();
        }
        if(!(this.isArray())) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> getValues called on a non array type: " + this.templateObj.name);
            throw new Error("getValues called on a non array type: " + this.templateObj.name);
        }
        // return the array of values.  This may not be
        // an array type in the obj!
        // Get the type
        var type = this.templateObj.type;
        if(type == "StringType") {
            var arr =  this.instanceObj.value.string;
            if(arr == null){
                arr = new Array();
            }
        } else if(type == "NumericType") {
            var arr =  this.instanceObj.value["int"];
            if(arr == null){
                arr = new Array();
            }
        } else if(type == "TimeStampType") {
            var arr =  this.instanceObj.value.timestamp;
            if(arr == null){
                arr = new Array();
            }
        } else if(type == "PointerType") {
            var arr =  this.getPointer();
            if(arr == null){
                arr = new Array();
            }
        } else if(type == "ConfigurationPartType") {
            return(this.partCache);
        }
    },
    // Set the value of an attribute.  This method is only 
    // valid if the attribute is not an array. It is an error 
    // to attempt to set the attribute's value to an object of 
    // the incorrect type.  If there is an existing value 
    // present, it will be replaced by the new value.
    setValue: function(object){
        return this._setValue(object, true);
    },
    _setValue: function(object, setSwap){
        if(this.isArray()) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> setValue called on an array type: " + this.templateObj.name);
            throw new Error("setValue called on an array type: " + this.templateObj.name);
        }
        if(object instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
            return this._setValuePart(object, setSwap);
        }
        if(object instanceof com.ibm.asq.common.web.client.internal._catalog.Pointer){
            return this._setValuePointer(object, setSwap);
        }
        return this._setValueRaw(object, setSwap);
    },
    // set the value of an attribute of type part
    _setValuePart: function(part, setSwap){
        // Save the old part in case it's swapped in later
        if (this.partCache.length != 0) {
            var oldPart = this.partCache[0];
            if (setSwap) {
                this.document_setSwappedPart(this.getName(), oldPart.getName(), oldPart);
            }
        }
        this.partCache = [part];           
        if (this.instanceObj == null) {
            // create the json obj for this attribute
            var obj = this._createPartObj(part, false);
            
            // set the obj in the document
            this.parent.instanceObj.Attributes.Attribute.push(obj);
            this.instanceObj = obj;
            
        }
        else {
            this.instanceObj.value = {
                AssetInstance: part.instanceRoot
            };
        }
    },
    // set the value of an attribute of type pointer
    _setValuePointer: function(pointer, setSwap){
        if (setSwap) {
            this.document._clearSwappedPart();
        }
        if (this.instanceObj == null) {
            // create the json obj for this attribute
            var obj = this._createPointerObj(pointer, false);
            
            // set the obj in the document
            this.parent.instanceObj.Attributes.Attribute.push(obj);
            this.instanceObj = obj;
            
        }
        else {
            this.instanceObj.value = this._createPointerValObj(pointer);
        }
    },
    // set the value of an attribute of type string, number or date
    _setValueRaw: function(object, setSwap){
        // Check if we can delete that attribute
        if(object == null) {
            if (this.instanceObj != null) {
                if (this.parent._isResolved()) {
                    this.instanceObj.value = { "null": "" }
                }
                else {
                    this.instanceObj.value = null;
                }
            }
            if(this.templateObj.type == "ConfigurationPartType"){
                this.partCache = [];
            }
            return;
        }
        // Get the type
        var type = this.templateObj.type;
        var obj = null;
            
        // Get a formatted obj
        if (type == "ConfigurationPartType") {
            // Get the existing part if there is one
            var oldPart = this.getValue();
            // Check if new and old are the same
            if(oldPart != null){
                if (oldPart.getName() == object) {
                    // Nothing to do
                    return;
                }
            }
            // Get the part if it is available from the swap
            if (setSwap) {
                var newPart = this.document._getSwappedPart(this.getName(), object);
                // Save the old part in case it's swapped in later
                if (oldPart != null) {
                    this.document._setSwappedPart(this.getName(), oldPart.getName(), oldPart);
                }
            }
            
            // Check if we have this part saved
            if (newPart == null) {
                var part = object;
                if (dojo.isString(part)) {
                    // Get the correct part from the template
                    var template = this._getTemplate();
                    part = template.getPart(object);
                    if (part == null) {
                        console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Part " + object + " not found in " + this.templateObj.AssetType.type + " template!");
                        throw new Error("Part not found in Template!");
                    }
                }
                // Create a Part for the partCache
                newPart = part._createInstancePart(this.document, null, this.parent.path);
                
                // Set the values from those in the old part
                if (oldPart != null) {
                    newPart._setValuesFromPart(oldPart);
                }
            }
            this.partCache= [newPart];           
            obj = this._createPartObj(newPart, false);
            
        } else if (type == "StringType") {
            obj = this._createObj("StringType", "string", object, false);
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        } else if (type == "NumericType") {
            obj = this._createNumericObj(object, false);
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        } else if (type == "TimeStampType") {
            obj = this._createObj("TimeStampType", "timestamp", object, false);
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        } else if (type == "PointerType") {
            obj = this._createPointerObj(object, false);
            if (setSwap) {
                this.document._clearSwappedPart();
            }
        }
        // set the obj in the document
        if(this.instanceObj == null) {
            this.parent.instanceObj.Attributes.Attribute.push(obj);
            this.instanceObj = obj;
        } else {
            this.instanceObj.value = obj.value;
        }
    },
    // Add a value to the selection of values for the attribute. 
    // This method is only valid if the attribute is an array. 
    addArrayValue: function(object){
        return this._addArrayValue(object, true);
    },
    _addArrayValue: function(object, setSwap){
        if(setSwap){
            this.document._clearSwappedPart();
        }
        if(!(this.isArray())) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> addValue called on a non array type!");
            throw new Error("addValue called on a non array type");
        }
        if(object instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
            return this._addArrayValuePart(object);
        }
        if(object instanceof com.ibm.asq.common.web.client.internal._catalog.Pointer){
            return this._addArrayValuePointer(object);
        }
        return this._addArrayValueRaw(object);
    },
    // Add a part to the selection of values for the attribute. 
    _addArrayValuePart: function(part){
		for(var j=0;j<this.partCache.length;j++){
			//dont add duplicates
			if(this.partCache[j].tag == part.tag){
				return;
			}
		}
       	this.partCache.push(part);           
        if(this.instanceObj == null) {
            // create the json obj for this attribute
            var obj = this._createPartObj(part, true);
                
            // set the obj in the document
            this.parent.instanceObj.Attributes.Attribute.push(obj);
            this.instanceObj = obj;

        } else {
            if(this.instanceObj.value == null || this.instanceObj.value["null"] == ""){
                this.instanceObj.value = "";
            }
            var valArray = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
            valArray.push(part.instanceRoot);
        }
    },
    // Add a pointer to the selection of values for the attribute. 
    // This method is only valid if the attribute is an array. 
    _addArrayValuePointer: function(pointer){
        if(this.instanceObj == null) {
            // create the json obj for this attribute
            var obj = this._createPointerObj(pointer, true);

            // set the obj in the document
            this.parent.instanceObj.Attributes.Attribute.push(obj);
            this.instanceObj = obj;
        } else {
            if(this.instanceObj.value == null || this.instanceObj.value["null"] == ""){
                this.instanceObj.value = "";
            }
            var valArray = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
            var val = this._createPointerValObj(pointer);
            valArray.push(val);
        }
    },
    // Add a value to the selection of values for the attribute. 
    // This method is only valid if the attribute is an array. 
    _addArrayValueRaw: function(object){
        // Validate the type
        var type = this.templateObj.type;

        if(this.instanceObj == null) {
            var obj = null;
            
            // Get a formatted obj
            if (type == "ConfigurationPartType") {
                var part = object;
                if(dojo.isString(part)) {
                    // Get the correct part from the template
                    var cat = new com.ibm.asq.common.web.client.internal.Catalog();
                    var template = cat.getTemplate(this.templateObj.AssetType.type);
                    part = template.getPart(object);
                    if(part == null) {
                        console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Part " + object + " not found in " + this.templateObj.AssetType.type + " template!");
                        throw new Error("Part not found in Template!");
                    }
                }
                // Create a Part for the partCache
                var newPart = part._createInstancePart(this.document, null, this.parent.path);
                this.partCache.push(newPart);           
                obj = this._createPartObj(newPart, true);
                
            } else if (type == "StringType") {
                obj = this._createObj("StringType", "string", object, true);
            } else if (type == "NumericType") {
                // Note: we don't currently have arrays of numbers
                // This code won't work with arrays of numbers with 
                //operators.
                obj = this._createObj("NumericType", "int", object, true);
            } else if (type == "TimeStampType") {
                obj = this._createObj("TimeStampType", "timestamp", object, true);
            } else if (type == "PointerType") {
                obj = this._createPointerObj(object, true);
            }
            // set the obj in the document
            this.parent.instanceObj.Attributes.Attribute.push(obj);
            this.instanceObj = obj;
        } else {
            if(this.instanceObj.value == null || this.instanceObj.value["null"] == ""){
                this.instanceObj.value = "";
            }
            if (type == "ConfigurationPartType") {
                var valArray = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
                var part = object;
                if(dojo.isString(part)) {
                    // Get the correct part from the template
                    var cat = new com.ibm.asq.common.web.client.internal.Catalog();
                    var template = cat.getTemplate(this.templateObj.AssetType.type);
                    part = template.getPart(object);
                    if(part == null) {
                        console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Part " + object + " not found in " + this.templateObj.AssetType.type + " template!");
                        throw new Error("Part not found in Template!");
                    }
                }
                // Create a Part for the partCache
                var newPart = part._createInstancePart(this.document, null, this.parent.path);
                this.partCache.push(newPart);           
                valArray.push(newPart.instanceRoot);

            } else if (type == "StringType") {
                var valArray = this._util._getArrayObject(this.instanceObj, "value", "string");
                valArray.push(object);
            } else if (type == "NumericType") {
                var valArray = this._util._getArrayObject(this.instanceObj, "value", "int");
                valArray.push(object);
            } else if (type == "TimeStampType") {
                var valArray = this._util._getArrayObject(this.instanceObj, "value", "timestamp");
                valArray.push(object);
            } else if (type == "PointerType") {
                var valArray = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
                var val = this._createPointerValObj(object);
                valArray.push(val);
            }
        }
    },
    // Determine if the attribute is required.`
    isRequired: function(){
		//return false;
        var cardinality = this._getCardinality();
        if (cardinality == "ONE_OR_MORE" || cardinality == "ONE") {
            return true;
        }
        else {
            return false;
        }
    },
    _getCardinality: function(){
        return this.templateObj.cardinality;
    },
    _extractOperators: function(val){
        var value = new Array();
        if(val != null) {
            if(!dojo.isString(val)){
                return [{ "int": val }];
            }
            var arr = this._parseSearchString(val);
            if(this._validateSearchArray(arr) != 0){
                 throw new Error("Invalid Search String")
            }
            if (arr.length == 1) {
                value = {};
                value["int"] = arr[0];
            }
            else {
                for (var i = 0; i < arr.length; i = i + 2) {
                    value.push({
                        operator: arr[i],
                        "int": arr[i + 1]
                    });
                }
            }
        }
        return value;
    },
    _validateSearchArray: function(arr) {
        // Array must be of size 1,2 or 4.  If size 1, we can only have a
        // number.  If size 2 there must be an operator followed by a number.
        // If size 4 it must be operator-number-operator-number, the operators
        // must be different directions (ie greater than or less than) and 
        // the values must make sence with the given operators.
        if(arr.length > 1 && this.document instanceof com.ibm.asq.common.web.client.internal._catalog.Asset){
            return this.VALIDATION_INVALID_SEQUENCE;
        }
        if(arr.length == 0) {
            return this.VALIDATION_OK;
        }
        else if(arr.length == 1) {
            if (isNaN(arr[0])) {
                return this.VALIDATION_INCOMPLETE;
            }
        }
        else if (arr.length == 2) {
            if (!(this._validateGEOp(arr[0]) || (this._validateLEOp(arr[0])))) {
                return this.VALIDATION_INVALID_SEQUENCE;
            }
            if (isNaN(arr[1])) {
                return this.VALIDATION_INVALID_SEQUENCE;
            }
        }
        else if (arr.length == 3) {
            return this.VALIDATION_INCOMPLETE;
        }
        else 
            if (arr.length == 4) {
                // validate the numbers
                if (isNaN(arr[1]) || isNaN(arr[3])) {
                    return this.VALIDATION_INVALID_SEQUENCE;
                }
                // validate the operators
                if (this._validateGEOp(arr[0])) {
                    if (this._validateLEOp(arr[2])) {
                        // the first number should be less than the second
                        if (parseFloat(arr[1]) >= parseFloat(arr[3])) {
                            return this.VALIDATION_DISJOINTED;
                        }
                    }
                    else {
                        return this.VALIDATION_DUPLICATE_OPERATORS;
                    }
                }
                else 
                    if (this._validateLEOp(arr[0])) {
                        if (this._validateGEOp(arr[2])) {
                            // the first number should be greater than the second
                            if (parseFloat(arr[1]) <= parseFloat(arr[3])) {
                                return this.VALIDATION_DISJOINTED;
                            }
                        }
                        else {
                            return this.VALIDATION_DUPLICATE_OPERATORS;
                        }
                    }
                    else {
                        return this.VALIDATION_DUPLICATE_OPERATORS;
                    }
            }
            else {
                return this.VALIDATION_EXTRA_COMPONENTS;
            }
        return this.VALIDATION_OK;
    },
    _validateGEOp: function(op){
        return(op == ">" || op == ">=");
    },
    _validateLEOp: function(op){
        return(op == "<" || op == "<=");
    },
    _parseSearchString: function(str) {
        str = str.replace(/\s+/g, "");
        var arr = new Array();
        if(str.length == 0){
            return arr;
        }
        var curr = "";
        var gettingDigit = false;
        for(var i = 0; i < str.length; i++){
            var ch = str.charAt(i);
            if(gettingDigit && !isNaN(ch)){
                curr = curr + ch;
            } 
            else if(gettingDigit && isNaN(ch)){
                if(curr != ""){
                    arr.push(curr);
                }
                curr = ch;
                gettingDigit = false;
            } 
            else if(!gettingDigit && !isNaN(ch)){
                if(curr != ""){
                    if(!(this._validateGEOp(curr) || this._validateLEOp(curr))){
                        throw new Error("Invalid character");
                    }
                    arr.push(curr);
                }
                curr = ch;
                gettingDigit = true;
            } 
            else if(!gettingDigit && isNaN(ch)){
                curr = curr + ch;
            }
            else {
                throw new Error("Invalid character");
            }
        }
        if(isNaN(curr) && (!(this._validateGEOp(curr) || this._validateLEOp(curr)))){
            throw new Error("Invalid character");
        }
        arr.push(curr);
        return arr;
        
    },
    _createObj: function(typeName, internalName, _value, isArray){
        var val = null;
        if(_value == null) {
            val = {'null':""};
        } else {
            if (isArray) {
                val = {};
                val[internalName] = [_value];
            }
            else {
                val = {};
                val[internalName] = _value;
            }
        }
        return { 
            name: this.templateObj.name,
            type: typeName, 
            value: val };
    },
    _createNumericObj: function(_value, isArray){
        var val = null;
        if(_value == null) {
            val = {'null':""};
        } else {
            if (isArray) {
                // Note: we don't currently have arrays of numbers
                // This code won't work with arrays of numbers with 
                //operators.
                val = {};
                val["int"] = [_value];
            }
            else {
                val = this._extractOperators(_value);           
            }
        }
        return { 
            name: this.templateObj.name,
            type: "NumericType", 
            value: val };
    },
    _createPointerObj: function(_value, isArray){
        var val = null;
        if(_value == null) {
            val = {'null':""};
        } else {
            var valObj = this._createPointerValObj(_value);
            if(isArray){
                val = [valObj];
            } else {
                val = valObj;
            }
        }
        return {
            type: "PointerType", 
            pointerType: this.templateObj.service,
            name: this.templateObj.name, 
            value: val };
    },
    _createPointerValObj: function(pointer){
        return ({
            DisplayValue: {
                displayString: pointer.ds,
                uuid: pointer.uuid
            }
        });
    },
    _createPartObj: function(_part, isArray){
        var val = null;
        if(_part == null) {
            val = {'null':""};
        } else {
            if(isArray){
                val = { AssetInstance: [_part.instanceRoot] };
            } else {
                val = { AssetInstance: _part.instanceRoot };
            }
        }
        return {
            name: this.templateObj.name,
            type: "ConfigurationPartType",
            value: val };
    },
    _buildPartCache: function(){
        if (this.isArray()) {
            // Need to get both resolved and unresolved
            
            // Get the correct template
            var template = this._getTemplate();
            
            var value = this.instanceObj.value;
            if (value == null) {
                return;
            }
            
            // Add the AssetInstance
            if (value.AssetInstance != null) {
                var aiArr = this._util._getArrayObject(this.instanceObj, "value", "AssetInstance");
                for (var i = 0; i < aiArr.length; i++) {
                    if (aiArr[i] != null) {
                        // Get the correct part from the template
                        var at = aiArr[i].AssetType;
                        var part = template.getPart(at.name);
                        var instancePart = part._createInstancePart(this.document, aiArr[i], this.parent.path);
                        this.partCache.push(instancePart);
                    }
                }
            }
        }
        else {
            // Construct a part object whose parent
            // is in the templateMap.
            
            // Get the correct part from the template
            var template = this._getTemplate();
            var instanceObj = null;
            var value = this.instanceObj.value;
            if(value == null){
                return;
            }
            if (this.instanceObj.value.AssetInstance != null) {
                instanceObj = this.instanceObj.value.AssetInstance;
            }
            else {
                return null;
            }
            var at = instanceObj.AssetType;
            part = template.getPart(at.name);
            var instancePart = part._createInstancePart(this.document, instanceObj, this.parent.path);
            this.partCache.push(instancePart);
        }

    }
}   
)
})();
