/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*
*
*	This class returns general administrative info.  It extends the lmContainer object.
*
*
*/

(function() {

	function addChildren(teams, teamAreas){
		if (teams && teams.length) {
			for (var j = 0; j < teams.length; ++j) {
				var team = teams[j];
				teamAreas.push({
					'name': team.name,
					'itemId': team.itemId
				});
				if (team.children){
					addChildren(team.children, teamAreas);
				}
			}
		}
		
	}

	var _version = null;
	dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
	dojo.require("com.ibm.rational.test.lm.web.client.internal.lmContainer");
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
	var process = com.ibm.team.process.web.client.internal.ProcessClient;
	dojo.provide("com.ibm.rational.test.lm.web.client.internal.admin");


	dojo.declare("com.ibm.rational.test.lm.web.client.internal.admin", 
	   com.ibm.rational.test.lm.web.client.internal.lmContainer, 
		{
			constructor: function()
			{
		       	this.setServiceInterface('com.ibm.rational.test.lm.common.IAdminRestService');
			},
		
			getVersion : function(callerResponseHandler) {
				if (_version != null) {
					callerResponseHandler._contextObject[callerResponseHandler._callbackName](_version);
					return;
				}
				
	       		var myResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(
					{
						_success: function ( version ) {
							_version = version;
							callerResponseHandler._contextObject[callerResponseHandler._callbackName](version);
						},
					
						_failure: function (errObj) {
							callerResponseHandler._contextObject[callerResponseHandler._errorCallbackName](errObj);
						}
					},
					"_success",
					"_failure"
				);
				
	        	this._invokeService(	myResponseHandler, 
        								"getVersion");
			},
			
			/*
			 * This is called with:
			 *     - projectAreaItemId: the UUID for the project area
			 *     - parentType: the name of the parent type under which the new type is to be created
			 *     - newType: the name of the type to be created
			 *     
			 * All fields are required and the newType MUST be unique within the catalog.
			 * 
			 * Returns true if the type was created.  Returns false if it was not created (because it
			 * duplicates an existing name only).
			 * 
			 * Upon success, callers must reload all catalog templates that they may have cached before
			 * the new type will be available to them.
			 * 
			 * This method performs the following process-enabled operation and action:
			 *     - operation ID: com.ibm.rqm.labresourceschema.save
			 *     - action ID: create
			 */
			createNewCatalogType : function(callerResponseHandler, projectAreaItemId, parentType, newType) {
				this._invokeService(callerResponseHandler,
				                    "postNewCatalogType",
									{ 
									    "projectAreaItemId" : projectAreaItemId,
										"parentType" : parentType,
										"newType" : newType
								    });
			},
			
			/*
			 * This is called with the item IDs of lab resources.  It returns an 
			 * array each of whose entries is the resource's item ID and a set
			 * of flags describing the operations that the current user is allowed
			 * to perform on the resource.  It makes NO promises that the operations
			 * will actually be possible but merely answers the question "does the
			 * current user have permission to perform the given action on this
			 * resource?".  Fields in each element of the returned array:
			 * 
			 * - itemId
			 * - canAddToBf
			 * - canAutomate
			 * - canDiscover
			 * - canEdit
			 * - canReserve
			 */
			getResourcePermissions : function(callerResponseHandler, itemIds) {
				this._invokeService(callerResponseHandler,
				                    "postResourcePermissions",
									{"uuid" : dojo.isArray(itemIds) ? itemIds : [itemIds]});
			},
			
			/*
			 * This returns an object with the following fields:
			 * 
			 *    - globalCpuSoftLimit (long): the currently set soft limit on the number of lab resource-hosted
			 *                                 CPUs that may be stored in the repository
			 *    - numGlobalCpus (long): the current count of such CPUs                          
			 */
			getSystemOptions : function(callerResponseHandler) {
                this._invokeService(callerResponseHandler, "getSystemOptions"); 
			},
			
			/*
			 * This is called with the UUID for the project area.  The startup of our Application
			 * stores this info in com.ibm.team.process.web.projectArea (name and itemId for the project).
			 * returns an array of team area objects. Each object contains two values, the name and
			 * the itemId (the UUID). You can then call getTeamAreaByUUID for details of each team area.
			 */
			getTeamAreasFromProject: function(callerResponseHandler, projectAreaUuid){
	       		var myResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(
					{
						_success: function ( projectAreas ) {
							var teamAreas = [];
							if (projectAreas && projectAreas.length) {
								for (var i = 0; i < projectAreas.length; ++i) {
									var project = projectAreas[i];
									if (project.itemId == projectAreaUuid) {
										addChildren(project.children,teamAreas);
									}
								}
							}
							callerResponseHandler._contextObject[callerResponseHandler._callbackName](teamAreas);
						},
					
						_failure: function (errObj) {
							callerResponseHandler._contextObject[callerResponseHandler._errorCallbackName](errObj);
						}
					},
					"_success",
					"_failure"
				);
				process.getProcessAreaHierarchies(myResponseHandler);
			},
			/*
			 * returns a com.ibm.team.process.internal.common.rest:TeamAreaDTO
			 * This contains:
			 * 		description
			 * 		developmentLine (object)
			 * 		members (array of objects)
			 * 		name
			 * 		processDTO (object)
			 * 		processHierarchy (an object tree with this team area at the root, containing all nested team areas)
			 * 		summary
			 * 
			 * Each member has:
			 * 		email
			 * 		name
			 * 		processRoles (array containing description and id for each role)
			 * 		userid
			 * 		uuid
			 * 
			 * ProcessDTO contains an array of all roles (including description and id for each)
			 * 
			 */
			getTeamAreaByUUID: function(callerResponseHandler, uuid){
				process.getTeamAreaByUUID({'uuid': uuid},callerResponseHandler);
			},
			
			getTeamAreasForUser: function(callerResponseHandler, projectAreaUuid, userId){
	       		var myResponseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(
					{
						_success: function ( projectAreas ) {
							var teamAreas = [];
							if (projectAreas && projectAreas.length) {
								for (var i = 0; i < projectAreas.length; ++i) {
									var project = projectAreas[i];
									if (project.itemId == projectAreaUuid) {
										addChildren(project.children,teamAreas);
									}
								}
							}
							callerResponseHandler._contextObject[callerResponseHandler._callbackName](teamAreas);
						},
					
						_failure: function (errObj) {
							callerResponseHandler._contextObject[callerResponseHandler._errorCallbackName](errObj);
						}
					},
					"_success",
					"_failure"
				);
				process.getProcessAreasForUser({'userId': userId},myResponseHandler);
			},
			
			getServerTime: function(serviceResponseHandler){
				this._invokeService(serviceResponseHandler, "getServerTime");
			}
			

		}
	);
})();
