/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Author: Paul Beaulieu

dojo.require("dojo.io.iframe");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.client.internal.lmContainer");
dojo.provide("com.ibm.rational.test.lm.web.client.internal.request");


dojo.declare("com.ibm.rational.test.lm.web.client.internal._requestReservation", com.ibm.rational.test.lm.web.client.internal.lmContainer, 
        
        {
            constructor: function(){
                this.setServiceInterface('com.ibm.rational.test.lm.common.IReservationRestService');
            },
        
            //
            getReservationsFromRequest: function(responseHandler, requestId)
            {
                if (!requestId)
                {
                    throw new Error("must specify requestId");
                }
                var args = {
                    "uuid": requestId
                }
                return this._invokeService( responseHandler, 
                                            "getReservationsFromRequest", 
                                            args );
            }
        }   
);

// Summary:
//    Internal object to normalize the request response with catalog information.
//    Convert catalog configuration string to objects for client consumption.
//
dojo.declare("request.callbackObject", null,
{
    responseHandler: null,
    constructor: function(handler)
    {
        this.responseHandler = handler;
    },
    successFunc: function(response) {
       if(response.foundMachines) {
           var machines = dojo.fromJson(response.foundMachines);
           response.foundMachines = machines;
        }
        if(response.reservations) {
            for( var i = 0; i < response.reservations.length; i++ )
            {
                var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
                var config = catalog.createAssetFromJson(response.reservations[i].configuration);
                response.reservations[ i ].configuration = config;
            }
        }
        if(this.responseHandler)
            this.responseHandler._contextObject[this.responseHandler._callbackName](response);
    },
    errorFunc: function(errorType, errorMessage) {
        if(this.responseHandler)
            this.responseHandler._contextObject[this.responseHandler._errorCallbackName](errorType, errorMessage);
    }
});

dojo.declare("com.ibm.rational.test.lm.web.client.internal.request", com.ibm.rational.test.lm.web.client.internal.lmContainer, 
    {
        constructor: function(){
            this.requestReservation = new com.ibm.rational.test.lm.web.client.internal._requestReservation();
            this.setServiceInterface('com.ibm.rational.test.lm.common.IRequestRestService');
            console.debug("initialized serviceInterface: "+this.serviceInterface);
        },
        //
        _getHost : function(url) {
            var h = url.match(/^[^\/]+\/\/[^\/]+\//);
            return h;
        },
            
        _getProjectAreaArgs : function() {
            var args = { "projectAreaItemId" : com.ibm.team.process.web.projectArea.itemId };
            
            return args;
        },
                     
       /**
        *
        */
        update: function(responseHandler, attributes) {
            var callbackObj = new request.callbackObject(responseHandler);
            
            var myHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(callbackObj, "successFunc", "errorFunc");
            return this._invokeService(myHandler, "postRequest", attributes);
        },

        /**
         * This API gets a request filled out in a minimal fashion.  
         */
         getAttributeValues: function(responseHandler) {
            // Calling getByUuid without providing a uuid at all returns
            // a minimally-filled-out request.  
            return this.getByUuid(responseHandler);             
         },
         
        /**
         *
         */
         getViewIdsForCurrentUser: function(responseHandler) {
            return this._invokeService(responseHandler, "getViewIds", this._getProjectAreaArgs());
         },
         
        /**
         *
         */
         getViewData: function(responseHandler, viewId) {
		 	var args = this._getProjectAreaArgs();
			
			args["viewId"] = viewId;
            return this._invokeService(responseHandler, "getViewData", args);
         },

         /**
          *
          */
         getReservations: function(responseHandler, requestId) {
            return this.requestReservation.getReservationsFromRequest(responseHandler, requestId );
         },
         
        /*
        * This function returns the details about a given entity by its ID.  It overrides the base class
        * implementation so that the reserved machines can be turned into proper configs.
        */
        getByUuid: function(responseHandler, uuid) {
            var callbackObj = new request.callbackObject(responseHandler);
            var myHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(callbackObj, "successFunc", "errorFunc");
            var args = this._getProjectAreaArgs();
            
            if (uuid) {
                args["uuid"] = uuid;
            }
            
            return this._invokeService(myHandler, "getByUuid", args);
        },

        /*
        * This function returns the details about a list of requests by their IDs.
        * The uuids parameter should be an array of uuids.
        */
        getByUuids: function(responseHandler, uuids) {
            var callbackObj = new request.callbackObject(responseHandler);
            var myHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(callbackObj, "successFunc", "errorFunc");
            var args = this._getProjectAreaArgs();
            
            if (uuids) {
                args["uuid"] = uuids;
            }
            
            return this._invokeService(myHandler, "postByUuids", args);
        }
    }
);
