/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
(function() {

var Root             = "com.ibm.rqm.planning.home.actionDispatcher";
var BaseActionHref   = "#action=" + Root;

// Summary:
//    Utility object to compose URIs for asset page's actions.
//
com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil = {
	
	// The page's root ID.
	PageRoot: Root,
	
	// Page's action keys.
	//
    AssetImportKey:     "&subAction=importAsset",
	AssetSearchKey:     "&subAction=searchAsset",
    AssetNewKey:        "&subAction=newAsset",
    AssetViewDetailKey: "&subAction=viewAsset",
    GroupsManageKey:    "&subAction=manageGroups",
    GroupNewKey:        "&subAction=newGroup",
    GroupViewDetailKey: "&subAction=viewGroup",
    InventoryKey:       "&subAction=manageInventory",
    DeployViewKey:      "&subAction=viewDeployments",
	CustomzeCatalogKey: "&subAction=customizeCatalog",

	getImportAssetURI: function()
	{
        return BaseActionHref + this.AssetImportKey;		
	},
	
    // Summary:
    //     Creates the URI to forward to the request query view. Optionally, 
    //     caller can specify whether to refresh the current query result, and/or
    //     removing a cached view in page.
    //     
    //
    getSearchAssetURI: function(assetType, removeViewId)
    {
        var base = BaseActionHref + this.AssetSearchKey + "&type=" + assetType;
        var uri = base; 
        
        if(removeViewId) {
            uri += ("&removeViewId=" + removeViewId);
        }
        return uri;
    },
	
    // Summary:
    //     Creates the URI to forward to the machine search view.  The client can specify args to pass to the view
    //     
    //	MFV  Should probably consolidate this method and getSearchAssetURI.  There is no reason to have two methods
	//	other than the fact that we are hours from Beta (05/01) and I don't want to change all the places that use getSearchAssetURI
    getSearchAssetURIWithArgs: function(args)
    {
		var moreArgs = "";
		if(args) 
		{
			for(var argName in args) 
			{
				moreArgs += ("&" + argName + "=" + args[argName]); 
			}
		}

        var uri = BaseActionHref + this.AssetSearchKey + moreArgs;

        return uri;
    },
	
     
    // Summary:
    //    Creates the URI to forward to the new request view.  
    //        
    getNewAssetURI: function(assetType)
    {
        var timestamp = dojo.date.locale.format(new Date(), {selector:"time", timePattern:"hh:mm:ss"});
         return BaseActionHref + this.AssetNewKey + "&id=" + timestamp + "&type=" + assetType;
    },

    // Summary:
    //    Creates the URI to forward to the edit request view for the specified request id.  Optionally,
	//    client can pass in additional argument name/value pairs in args: {"arg-name": "arg-value",..}
    //        
    getViewAssetDetailURI: function(assetType, id, args) 
    {
        var base = BaseActionHref +  this.AssetViewDetailKey + "&id=" + id + "&type=" + assetType;
        var uri = base; 
        
        if(args) {
            for(var arg in args) {
                uri += ("&" + arg + "=" + args[arg]);
            }
        }
        return uri;
    },
    
    getManageGroupsURI: function(refresh, statusMessage, statusState)
	{
        var uri = BaseActionHref + this.GroupsManageKey;
		
        if (refresh) {
            uri += ("&refresh=true");
        }
		
        if (statusMessage) {
            uri += ("&initMessage=" + statusMessage);
        }
		
        if (statusState) {
            uri += ("&initStatus=" + statusState);
        }
		
		return uri;
    },

    getNewGroupURI: function()
    {
        var timestamp = dojo.date.locale.format(new Date(), {selector:"time", timePattern:"hh:mm:ss"});
        return BaseActionHref + this.GroupNewKey + "&id=" + timestamp;
    },

    getGroupDetailURI: function(id, removeViewId, statusMessage, statusState) 
    {
        var base = BaseActionHref +  this.GroupViewDetailKey + "&id=" + id;
        var uri = base; 
        
        if (removeViewId) {
            uri += ("&removeViewId=" + removeViewId);
        }
		
        if (statusMessage) {
            uri += ("&initMessage=" + statusMessage);
        }
		
        if (statusState) {
            uri += ("&initStatus=" + statusState);
        }
		
        return uri;
    },
    
    getInventoryURI: function()
    {
        return BaseActionHref + this.InventoryKey;
    },

    getDeployViewURI: function()
    {
        var uri = BaseActionHref + this.DeployViewKey;
        return uri;
    },

    getCustomizeCatalogURI: function()
    {
        return BaseActionHref + this.CustomzeCatalogKey;
    }
       
};
})();
