/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.pages.ConfigurationsPage");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
//xojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.PrimaryActionButton");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.PageLayout");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.PageUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationEditor");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.QueryConfigurationsView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil");


(function() 
{
var RtlmUiUtil 		= com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;	
var PlatformUI      = net.jazz.ajax.ui.PlatformUI;
var ActionRegistry  = null; // bind to ActionRegistry later
var LOADING         = RtlmUiUtil.getMessage("cfgMsgRetrievingConfigurationsWorking");
var QUERY_VIEW_ID   = "configurations_query_view";
var UriUtil 		= com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil;

function getWorkbench() 
{
    return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.pages.ConfigurationsPage",
    com.ibm.asq.common.web.ui.internal.view.layout.PageLayout,
{   
    hideSearchSection: true, // we don't have search in the left action panel yet

    // Parent main page that holds this configurations sub-page.  All registered
    // action must have the parent main page bring up this sub-page before
    // performing the action.
    parentMainPage: null,

	postCreate: function() 
	{
		this.inherited("postCreate", arguments);
		this.viewCache = {};
        
		var param = 
		{
			id: "newConfiguration",
			iconClass: "newReservationIcon",
			actionLabel: RtlmUiUtil.getMessage("newLabel"),
			itemLabel: RtlmUiUtil.getMessage("cfgCfgLabel")
		};
        var newConfigurationAction = new com.ibm.asq.common.web.ui.internal.view.layout.PrimaryActionButton(param);
        newConfigurationAction.onClick = dojo.hitch(this, "handleNewAction");
        this.setPrimaryAction(newConfigurationAction.domNode);
		
		var moreActionsButton = new com.ibm.asq.common.web.ui.internal.view.layout.SecondaryActionsDropdownButton();
        moreActionsButton.setLabel(RtlmUiUtil.getMessage("moreActionsButtonLabel"));
        moreActionsButton.addAction(dojo.hitch(this, "queryConfigurationsView", {refresh: true}), "Show Query");
        this.setSecondaryActions(moreActionsButton.domNode);
        
        var viewActions = new com.ibm.rational.test.lm.web.ui.internal.pages.PageNavigator({fixedSelection:"Configurations"});
        this._leftNavigator.getViewsSection().appendChild(viewActions.domNode);

        ActionRegistry = getWorkbench().getActionRegistry();
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.ConfigurationsPage.queryConfigurationsView", this, "queryConfigurationsView");
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.ConfigurationsPage.newConfigurationView", this, "newConfigurationView");      
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.ConfigurationsPage.editConfigurationView", this, "editConfigurationView");      

        this.addSampleRelatedInfo();
    },

    // Summary:
    //     Handle registered action for bringing the request query view into the page.
    //     The action may also includes directives to refresh the query results, and/or 
    //     to remove a cached request form view in the page.
    //
    queryConfigurationsView: function(args)
    {
        if (this.parentMainPage) 
		{
            this.parentMainPage.showConfigurationsSubPage();
        }
    
        var queryView = this.getCachedView(QUERY_VIEW_ID);
        
        // no request query view created yet, create one and display it.
        if(!queryView) 
        {
            queryView = new com.ibm.rational.test.lm.web.ui.internal.views.configurations.QueryConfigurationsView();
        }
        this.setPageView(QUERY_VIEW_ID, queryView);
        
        // carry out the directives.
        //
        if(args.removeViewId != null && this.getCachedView(args.removeViewId)) 
        {
            this.removeCachedView(args.removeViewId);
        }
        // refresh the query results.
        if(args.refresh)
            queryView.runSelectedQuery();
    },

    // Summary:
    //     Handle registered action for bringing the new configuration editor view.
    //
    newConfigurationView: function(args)
    {
        if (this.parentMainPage) {
            this.parentMainPage.showConfigurationsSubPage();
        }
    
        var configEditor = this.getCachedView(args.id);
        
        // no request query view created yet, create one and display it.
        if(!configEditor) 
        {
			var editorArgs = {
				isNew: true,
				configId: args.id,
				assetType: args.type
			}
            configEditor = new com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationEditor(editorArgs);
        }
        this.setPageView(args.id, configEditor);
    },
	
	// Summary:
    //     Handle registered action for editing and existing config in the configuration editor view.
    //
    editConfigurationView: function(args)
    {
        if (this.parentMainPage) {
            this.parentMainPage.showConfigurationsSubPage();
        }
    
        var configEditor = this.getCachedView(args.id);
        
        // no request query view created yet, create one and display it.
        if(!configEditor) 
        {
			var editorArgs = {
				isNew: false,
				configId: args.id,
				assetType: args.type
			}
            configEditor = new com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationEditor(editorArgs);
        }
        this.setPageView(args.id, configEditor);
    },
    
    // Summary:
    //     Handles the "New Configuration" button click event in the page's action panel. Generates
    //     a new configuration URI and this is forward the action "back" into this page's new configuration action handler.
    //
    handleNewAction: function()
    {
		window.location.href = UriUtil.getNewConfigurationURI();
    }

});

})();
