/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.pages.LabManagement");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");

dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.MachinePage");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.RequestPage");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.ReservationsPage");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.ConfigurationsPage");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.InventoryPage");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.PageUtil");

(function() {

var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var InitializationData      = com.ibm.team.repository.web.client.init.InitializationData;
var RtlmInitializer         = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer;
var PageUtil                = com.ibm.rational.test.lm.web.ui.internal.pages.PageUtil;

function getWorkbench() {
    return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.pages.LabManagement",
    [dijit._Widget, dijit._Templated],
    {
        templateString: '<div id="${id}" dojoAttachPoint="domNode">',
        
        currentPage: null,
        assetsPage: null,
        requestsPage: null,
        reservationsPage: null,
        configurationsPage: null,
		inventoryPage: null,
        
        assetsPageInited: false,
        requestsPageInited: false,
        reservationsPageInited: false,
        configurationsPageInited: false,
		inventoryPageInited: false,
    
        postCreate: function() {
            com.ibm.rational.test.lm.web.ui.internal.pages.LabManagement.superclass.postCreate.apply(this, arguments);

            this._initDataCompleted = InitializationData.get("com.ibm.rational.test.lm.web.catalogInitialized");

            ActionRegistry = getWorkbench().getActionRegistry();
            ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.LabManagement.showViewPage", this, "showViewPage");

            // create the sub-pages
            this.assetsPage = new com.ibm.rational.test.lm.web.ui.internal.pages.MachinePage({label:"Resources",parentMainPage:this});
            this.requestsPage = new com.ibm.rational.test.lm.web.ui.internal.pages.RequestPage({label:"Requests",parentMainPage:this});
            this.reservationsPage = new com.ibm.rational.test.lm.web.ui.internal.pages.ReservationsPage({label:"Reservations",parentMainPage:this});
            this.configurationsPage = new com.ibm.rational.test.lm.web.ui.internal.pages.ConfigurationsPage({label:"Configurations",parentMainPage:this});
            this.inventoryPage = new com.ibm.rational.test.lm.web.ui.internal.pages.InventoryPage({label:"Inventory",parentMainPage:this});
			
			// keep pages created in a global page cache so it can be referenced
			PageUtil.PageCache[PageUtil.AssetsPageId] = this.assetsPage;
			PageUtil.PageCache[PageUtil.RequestsPageId] = this.requestsPage;
			PageUtil.PageCache[PageUtil.ReservationsPageId] = this.reservationsPage;
			PageUtil.PageCache[PageUtil.ConfigurationsPageId] = this.configurationsPage;
			PageUtil.PageCache[PageUtil.InventoryPageId] = this.inventoryPage;

            // Data initialization is async so we need to initialize in the very first time default
            // action is invoked which is right after this instance is created.
        },

        // Catalog can be already initialized from the application outside of this page.
        _initDataCompleted: null,
        
        // Default action is intercepted if catalog data initialization is required.  The
        // args from the default action is cached for the success handler to re-invoke
        // the default action.
        _initDataArgs: {},
        
        // Handler on success from initializing the catalog data.
        //
        _initCatalogSuccess: function(response) {
            console.debug("initialized catalog successfully!");

            // The method _initCatalogSuccess() is invoked from the data initializer which is
            // from the showViewPage() method so invoke it here after data is initialized.
            this.showViewPage(this._initDataArgs);
        },

        // Handler on failure from initializing the catalog data.
        //
        _initFailure: function(errorObject) {
            // put up the error message on init failure
            var errorPrompt = document.createElement('div');
            var errorMessage = document.createElement('div');
            errorPrompt.innerHTML = "<p><p><h2>Failed to initialize the Lab Management</h2><p>";
            errorMessage.innerHTML = errorObject['errorMessage'];
            this.domNode.appendChild(errorPrompt);
            this.domNode.appendChild(errorMessage);
            console.debug("initialization failed!");
        },
        
        _initRtlm: function(successCallback) {
            this._initDataCompleted = true;
            var responseHandler = new ServiceResponseHandler(this, successCallback, "_initFailure");
            RtlmInitializer.initializeData(responseHandler);
        },
        
        // Default action for bringing up the page.  Based on the "page" parameter
        // in the args, the corresponding page and view is displayed.  This method
        // is invoked from user click on the "Lab Management" tab or on any one of
        // the page view navigator.  Registered actions of the sub-pages do not use
        // this method to bring up the sub-page.
        //
        showViewPage: function(args) {
            // intercept the action if catalog data is not initialized yet
            if (!this._initDataCompleted) {
                // initialize the catalog data the very first time this method is invoked
                this._initDataArgs = args;
                this._initRtlm("_initCatalogSuccess");
                return;
            }
            
            if (this.currentPage) {
                this.domNode.removeChild(this.currentPage.domNode);
            }
            
            if (args) {
                switch (args.page) {
                    case PageUtil.RequestsPageId:
                        this.currentPage = this.requestsPage;
                        this.domNode.appendChild(this.currentPage.domNode);
                        // invoke the default action of the Requests page
                        // if it has not been initialized yet
                        if (!this.requestsPageInited) {
                            this.currentPage.queryRequestView(args);
                            this.requestsPageInited = true;
                        }
                        break;
                    case PageUtil.ReservationsPageId:
                        this.currentPage = this.reservationsPage;
                        this.domNode.appendChild(this.currentPage.domNode);
                        // invoke the default action of the Reservations page
                        // if it has not been initialized yet
                        if (!this.reservationsPageInited) {
                            this.currentPage.queryReservationsView(args);
                            this.reservationsPageInited = true;
                        }
                        break;
                    case PageUtil.AssetsPageId:
                        this.currentPage = this.assetsPage;
                        this.domNode.appendChild(this.currentPage.domNode);
                        // invoke the default action of the Assets page
                        // if it has not been initialized yet
                        if (!this.assetsPageInited) {
                            if (!args.type) {
                                args.type = "Machine";
                            }
                            this.currentPage.findAssetsView(args);
                            this.assetsPageInited = true;
                        }
                        break;
                    case PageUtil.ConfigurationsPageId:
                        this.currentPage = this.configurationsPage;
                        this.domNode.appendChild(this.currentPage.domNode);
                        // invoke the default action of the Reservations page
                        // if it has not been initialized yet
                        if (!this.configurationsPageInited) {
                            this.currentPage.queryConfigurationsView(args);
                            this.configurationsPageInited = true;
                        }
                        break;
                    case PageUtil.InventoryPageId:
                        this.currentPage = this.inventoryPage;
                        this.domNode.appendChild(this.currentPage.domNode);
                        // invoke the default action of the Reservations page
                        // if it has not been initialized yet
                        if (!this.inventoryPageInited) {
                            this.currentPage.queryInventoryView(args);
                            this.inventoryPageInited = true;
                        }
                        break;
                    default:
                        // clicking on the "Lab Management" tab don't have the
                        // "page" attribute in the args
                        this.showDefaultSubPage();
                }
            } else {
                // default action never get called with no args, show default anyway
                this.showDefaultSubPage();
            }
        },
        
        // Clicking on the "Lab Management" tab does not have any "page" parameter
        // in the args.  If there is a sub-page is already set, bring it up.
        // Otherwise, bring up the default assets sub-page.
        //
        showDefaultSubPage: function() {
            if (!this._initDataCompleted) {
                this._initRtlm("showDefaultSubPage");
                return;
            }
            if (this.currentPage) {
                // bring up whatever the current page is
                this.domNode.appendChild(this.currentPage.domNode);
            } else {
                // default to assets search page when there is no current page
                this.showAssetsSubPage();
				if(this.currentPage)
                    this.currentPage.findAssetsView({type: "Machine"});
            }
        },
        
        // Registered actions in the Requests page instance must bring up the
        // Requests sub-page before performing the action.
        //
        showRequestsSubPage: function() {
            if (!this._initDataCompleted) {
                this._initRtlm("showRequestsSubPage");
                return;
            }
            if (this.currentPage != this.requestsPage) {
                if (this.currentPage)
                    this.domNode.removeChild(this.currentPage.domNode);
                this.currentPage = this.requestsPage;
                this.requestsPageInited = true;
                this.domNode.appendChild(this.currentPage.domNode);
            }
        },
        
        // Registered actions in the Reservations page instance must bring up the
        // Reservations sub-page before performing the action.
        //
        showReservationsSubPage: function() {
            if (!this._initDataCompleted) {
                this._initRtlm("showReservationsSubPage");
                return;
            }
            if (this.currentPage != this.reservationsPage) {
                if (this.currentPage)
                    this.domNode.removeChild(this.currentPage.domNode);
                this.currentPage = this.reservationsPage;
                this.reservationsPageInited = true;
                this.domNode.appendChild(this.currentPage.domNode);
            }
        },

        // Registered actions in the Assets page instance must bring up the
        // Assets sub-page before performing the action.
        //
        showAssetsSubPage: function() {
            if (!this._initDataCompleted) {
                this._initRtlm("showAssetsSubPage");
                return;
            }
            if (this.currentPage != this.assetsPage) {
                if (this.currentPage)
                    this.domNode.removeChild(this.currentPage.domNode);
                this.currentPage = this.assetsPage;
                this.assetsPageInited = true;
                this.domNode.appendChild(this.currentPage.domNode);
            }
        },
		
		// Registered actions in the Configurations page instance must bring up the
        // Configurations sub-page before performing the action.
        //
        showConfigurationsSubPage: function() {
            if (!this._initDataCompleted) {
                this._initRtlm("showConfigurationsSubPage");
                return;
            }
            if (this.currentPage != this.configurationsPage) {
                if (this.currentPage)
                    this.domNode.removeChild(this.currentPage.domNode);
                this.currentPage = this.configurationsPage;
                this.configurationsPageInited = true;
                this.domNode.appendChild(this.currentPage.domNode);
            }
        },
		
		// Registered actions in the Configurations page instance must bring up the
        // Configurations sub-page before performing the action.
        //
        showInventorySubPage: function() {
            if (!this._initDataCompleted) {
                this._initRtlm("showInventorySubPage");
                return;
            }
            if (this.currentPage != this.inventoryPage) {
                if (this.currentPage)
                    this.domNode.removeChild(this.currentPage.domNode);
                this.currentPage = this.inventoryPage;
                this.inventoryPageInited = true;
                this.domNode.appendChild(this.currentPage.domNode);
            }
        }
    }
);

})();
