/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.pages.RequestPage");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

//xojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.PrimaryActionButton");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.PageLayout");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.PageUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestFormView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestView");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");

(function() {

var RtlmUiUtil 		= com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var checkNotNull    = net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty   = net.jazz.ajax.toolbox.lang.checkNotEmpty;

var PlatformUI      = net.jazz.ajax.ui.PlatformUI;
var ActionRegistry  = null; // bind to ActionRegistry later
var LOADING         = RtlmUiUtil.getMessage("reqMsgRetrievingRequestsWorking");
var UriUtil         = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil;
var QUERY_VIEW_ID   = "request_query_view";

function getWorkbench() {
    return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

// Summary:
//    Defines the page for RTLM requests. 
//

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.pages.RequestPage",
    com.ibm.asq.common.web.ui.internal.view.layout.PageLayout,
{   
    hideSearchSection: true, // we don't have search in the left action panel yet
    
    // Parent main page that holds this requests sub-page.  All registered
    // action must have the parent main page bring up this sub-page before
    // performing the action.
    parentMainPage: null,
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.viewCache = {};
        
        var param = {
            id: "newRequest",
            iconClass: "newRequestIcon",
            actionLabel: RtlmUiUtil.getMessage("newLabel"),
            itemLabel: RtlmUiUtil.getMessage("reqLabel_request")
        };
        var newRequestAction = new com.ibm.asq.common.web.ui.internal.view.layout.PrimaryActionButton(param);
        newRequestAction.onClick = dojo.hitch(this, "handleNewAction");
        this.setPrimaryAction(newRequestAction.domNode);
        
        var moreActionsButton = new com.ibm.asq.common.web.ui.internal.view.layout.SecondaryActionsDropdownButton();
        moreActionsButton.setLabel(RtlmUiUtil.getMessage("moreActionsButtonLabel"));
        moreActionsButton.addAction(dojo.hitch(this, "handleShowQueryAction"), "Show Query");
        this.setSecondaryActions(moreActionsButton.domNode);
            
        
        var viewActions = new com.ibm.rational.test.lm.web.ui.internal.pages.PageNavigator({fixedSelection:"Requests"});
        this._leftNavigator.getViewsSection().appendChild(viewActions.domNode);

        ActionRegistry = getWorkbench().getActionRegistry();
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.RequestPage.queryRequestView", this, "queryRequestView");
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.RequestPage.newRequestView", this, "newRequestView");      
        ActionRegistry.registerAction("com.ibm.rational.test.lm.web.ui.internal.pages.RequestPage.editRequestView", this, "editRequestView");      

        this.addSampleRelatedInfo();
    },

    // Summary:
    //     Handle registered action for bringing the request query view into the page.
    //     The action may also includes directives to refresh the query results, and/or 
    //     to remove a cached request form view in the page.
    //
    queryRequestView: function(args)
    {
        if (this.parentMainPage) {
            this.parentMainPage.showRequestsSubPage();
        }
        var queryView = this.getCachedView(QUERY_VIEW_ID);
        
        // no request query view created yet, create one and display it.
        if(!queryView) {
            queryView = new com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestView();
        }
        this.setPageView(QUERY_VIEW_ID, queryView);
        
        // carry out the directives.
        //
        if(args.removeViewId != null && this.getCachedView(args.removeViewId)) {
            this.removeCachedView(args.removeViewId);
        }
        // refresh the query results.
        if(args.refresh)
            queryView.runSelectedQuery();
    },
    
    // Summary:
    //     Handle registered action for bringing a new request form into the page.
    //
    newRequestView: function(args)
    {
        if (this.parentMainPage) {
            this.parentMainPage.showRequestsSubPage();
        }
        return this._showRequestView(args, true);
    },

    // Summary:
    //     Handle registered action for display a request form for a request id.
    //
    editRequestView: function(args)
    {
        if (this.parentMainPage) {
            this.parentMainPage.showRequestsSubPage();
        }
        this._showRequestView(args, false);
    },
    
    // Summary:
    //    Internal method to display the request from view in the page. If the particular request
    //    form view is already cached in the page (via its id), we'll just reuse it. 
    //    NOTE: This works the same for new request. A new request is generated with a timestamp ID.
    //    This id is then used to cache the new request's view. This way if the user "back button"
    //    from the new request page, a forward button with bring back the exact new request form. 
    //
    _showRequestView: function(args, newRequestForm)
    {
        var requestView = this.getCachedView(args.id);
        
        // the view is not in our cache, create a new one.
        if(!requestView) {    
            var formArg = {requestId: args.id, isNew: newRequestForm, doneLoadingCallback: args.doneLoadingCallback};
            requestView = new com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestFormView(formArg);
        }
        this.setPageView(args.id, requestView);
		
		return requestView;
    },
    
    // Summary:
    //     Handles the "new request" button click event in the page's action panel. Generates
    //     a new request URI and this is forward the action "back" into this page's new request action handler.
    //
    handleNewAction: function()
    {
        window.location.href = UriUtil.getNewRequestURI();
    },
    
    handleShowQueryAction: function()
    {
        window.location.href = UriUtil.getQueryRequestURI();
    },
    
    getController: function() {
        return this;
    },

    getSafeUrlActions: function() {
        return [];
    },
    
    init: function() {
    }
});

})();
