/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function(){
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
    var CategoryClient = com.ibm.rqm.planning.web.client.internal.CategoryClient;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
   	var SelectUtility = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
		
	var UNASSIGNED = Constants.getMessage("labelUnassigned");
    var UNKNOWN = Constants.getMessage("labelUnknown");
	//var DIALOG_STYLE = "position: absolute; top: 15%; left: 15%; width: 600px; height: 500px; z-index: 999";
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoriesEditor",
	[dijit._Widget, dijit._Templated],
	{
        //		
	    // Provides dynamic selection of Categories 
	    //
	    templateString: '<div class="com-ibm-rqm-categories-editor">' +
		                '  <div dojoAttachPoint="_loadingNode" class="hidden">${labelLoading}</div>' +
                        '  <div class="action-option-bar">' +
                        '      <div class="actions" dojoAttachPoint="_actionsNode"></div>' +
						'      <div class="clear"></div>' +
						'  </div>' +
						'  <div dojoAttachPoint="_categoryTypesNode" class="category-types"></div>' +
						'  <div class="options">' +
						'      <input name="show-archived-checkbox" id="show-archived-checkbox" type="checkbox" dojoAttachPoint="_showArchivedCheckbox" dojoAttachEvent="onclick: _onArchivedClick"></input>' +
						'      <label for="show-archived-checkbox" class="" dojoAttachPoint="_showArchivedLabel"></label>' +
						'  </div>' +	                
		                '</div>',
		
        dialogWidth: "650px",
		nonDialogWidth: "425px",   //Smaller to fit in 1024x768

		constructor: function() {
			this.labelLoading = Constants.getMessage("labelLoading");
			this.isDialog = true;
			this.artifactType = null;
            this.categoryTypes = [];
			this.onClose = null;  //optional callback fired on editor close
			this._editors = [];  //CategoryTypeEditor cache
			this._showArchived = false;			
			this._permissions = {};
			this._actionButtons = {};
			this._contributors = [];			
		},		
	    
	    postCreate: function() {
			//ensure there is an artifact type specified
			if (!this.artifactType) {
                throw new Error("Parameter 'artifactType' is REQUIRED!");
            }
			
			//show the 'archived' option
			this._showArchivedLabel.innerHTML = Constants.getMessage("labelShowArchivedCategories");	
            if (!this.isDialog) {
				this.domNode.style.width = this.nonDialogWidth;
			}
			//determine the current user's permissions
            var operations = [];
            operations.push({
                id: PermissionsClient.OP_CATEGORY_TYPE, 
                actions: PermissionsClient.DEFAULT_ACTIONS
            });
            operations.push({
                id: PermissionsClient.OP_CATEGORY, 
                actions: PermissionsClient.DEFAULT_ACTIONS
            });
			
			var responseHandler = new ServiceResponseHandler(this, "_preinitPermissions", "_error");
            PermissionsClient.checkAllPermissions(responseHandler, operations);				
	    },
		
		_preinitPermissions: function(permissions) {
			this._permissions = permissions;			
	        EditorUtil.getContributors(this, "_preinitContributors", "_error");  			
		},
		
        _preinitContributors: function(contributors) {
            this._contributors = contributors;
            this.initialize();          
        },		

        initialize: function(permissions) {			
			this._showActions();
            this._showArchivedCheckbox.checked = this._showArchived;
			dojox.data.dom.removeChildren(this._categoryTypesNode);
			dojo.removeClass(this._loadingNode, "hidden");
			
			//retrieve the categoryTypes for display/edit			
			var self = this;
			var handlerObj = {
				success: function(categoryTypes) {
                    dojo.addClass(self._loadingNode, "hidden");
					self.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
					self.showCategoryTypes();
					//if this is being shown as a dialog, init the dialog
		            if (self.isDialog && !self._dialog) {
		                self.show();
		            }
				},
				
				failure: function(e) {
					self._error(e);
				}
			}
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            CategoryClient.getCategoryTypes(responseHandler, this.artifactType, this._showArchived);
        },
		
        show: function() {		
            if (!this._dialog) {           
                var dialogArgs = {                    
					primaryTitle: Constants.getMessage("labelManageCategories", this._getTypeDisplayName()),
					isClosable: true,
					onClose: dojo.hitch(this, "hide"),
					showActions: false,
					_truncate: function() {}  //disable the Dialog's '_truncate'
                };                
                this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
				if (this.artifactType == Constants.MODEL_TYPE_CASE) {
	                var cshelp = new CSHelp({contextID: "rqm_manage_tc_categories.html"});
	                cshelp.setButtonLabel(Constants.getMessage("labelManageCategoriesCSHelp"));
	                this._dialog.setSecondaryTitleNode(cshelp.domNode);
				} else if (this.artifactType == Constants.MODEL_TYPE_PLAN){
    	            var cshelp = new CSHelp({contextID: "rqm_manage_tp_categories.html"});
    	            cshelp.setButtonLabel(Constants.getMessage("labelManageCategoriesCSHelp"));
	                this._dialog.setSecondaryTitleNode(cshelp.domNode);
				}
                this._dialog.setWidth(this.dialogWidth);
                this._dialog.contentContainer.appendChild(this.domNode);                     
            }
            this._dialog.open();    						
		},		
		
		hide: function() {
			//fire onClose callback, if specified
			if (dojo.isFunction(this.onClose)) {
				this.onClose(this.categoryTypes);
			}
            if (dojo.isFunction(this.destroy)) {
                this.destroy();
            }			
			return;
		},
	
		showCategoryTypes: function() {
			this._editors = [];
			dojox.data.dom.removeChildren(this._categoryTypesNode);			
			for (var i = 0; i < this.categoryTypes.length; i++) {
				var editor = new  com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryTypeEditor({
					parent: this,
					index: this._editors.length,
					categoryType: this.categoryTypes[i]
				});
				this._editors.push(editor);   //cache the editor
				this._categoryTypesNode.appendChild(editor.domNode);    //show the editor
			}
		},
		
        addCategoryType: function(newCategoryType) {
            this.categoryTypes.push(newCategoryType);
        },		
		
		_getTypeDisplayName: function() {
            if (this.artifactType == Constants.MODEL_TYPE_PLAN) {
                return Constants.TYPE_PLAN;
            } else if (this.artifactType == Constants.MODEL_TYPE_CASE) {
                return Constants.TYPE_CASE;
            }
			return "";
        },
		
        _showActions: function() {
            dojox.data.dom.removeChildren(this._actionsNode);
            var actions = this._getActions();
            for (var i = 0; i < (actions ? actions.length : 0); i++) {
                this._addAction(actions[i]);   
            }            
        },  		
		
        _getActions: function() {
            var actions = []; 
            if (this.hasCategoryTypePermission(PermissionsClient.ACTION_CREATE)) {
                actions.push({
					id: "new", //$NON-NLS$
                    label: Constants.getMessage("labelAddCategoryType"),
                    actionHandler: dojo.hitch(this, "_handleAddNew"),
                    enabledIcon: ImageCache.ADD_ICON_ENABLED,
                    disabledIcon: ImageCache.ADD_ICON_DISABLED
                });
            }
            return actions; 
        },      
        
        _addAction: function(action) {                
            var hover = action.disabledIcon && action.enabledIcon;
            var button = document.createElement("img");
            dojo.addClass(button, "action-button");
            button.src = hover ? action.disabledIcon : action.enabledIcon;
            button.title = action.label;
            button.onclick = function(event){
				dojo.stopEvent(event);
				action.actionHandler(event);
			}      
            if (hover) {
                button.onmouseover = function() { button.src = action.enabledIcon; }
                button.onmouseout = function() { button.src = action.disabledIcon; }
            }			
			
			/*
			var hover = action.disabledIconClass && action.enabledIconClass;
            var args = {
                label: action.label,
                onClick: action.actionHandler,
                iconClass: action.iconClass,
                showLabel: false
            };
            var button = new com.ibm.asq.common.web.ui.internal.widgets.Button(args); 			
            if (hover) {
				button.setDisabled(true);
                button.domNode.onmouseover = function() { button.setDisabled(false); }
                button.domNode.onmouseout = function() { button.setDisabled(true); }
            }	
            */
            this._actionsNode.appendChild(button);					
        },
					
        //////////////////////////////////////////////////////////////////////////////////////////
        ///                                        API                                         ///
        //////////////////////////////////////////////////////////////////////////////////////////		
		isShowArchived: function() {
			return this._showArchived;
		},    
		
        hasPermission: function(operationId, actionId) {
            var permission = this._permissions[operationId];
            return (permission && permission[actionId] && permission[actionId] == true);         
        },		
		
		hasCategoryTypePermission: function(actionId) {
			return this.hasPermission(PermissionsClient.OP_CATEGORY_TYPE, actionId);
		},

        hasCategoryPermission: function(actionId) {
            return this.hasPermission(PermissionsClient.OP_CATEGORY, actionId);
        },
		
        getCategoryType: function(name) {
            for (var i = 0; i < (this._editors ? this._editors.length : 0); i++) {
                var categoryType = this._editors[i].getCategoryType();
                if (categoryType.name.toUpperCase() == name.toUpperCase()) {
                    return categoryType;
                }            
            }
            return null;
        },		
		
		getArtifactType: function() {
			return this.artifactType;
		},	
		
		getContributors: function() {
			return this._contributors;
		},
		
        removeEditor: function(editor) {
            if (editor) {
                this._categoryTypesNode.removeChild(editor.domNode);
                this._editors.splice(editor.index, 1);
                editor.destroy();
            }
        },		
		
		_onArchivedClick: function(event) {						
			//reinit the view, changing the archive option			
			if (this._showArchived != this._showArchivedCheckbox.checked) {
				this._showArchived = this._showArchivedCheckbox.checked;
				this.initialize();
			}
		},
		
        _handleAddNew: function() {
			//protect against concurrent adds
			if (this._isAdding) {
				return;
			} else {
				this._isAdding = true;
			}

            var newNum = 1;
            var newName = Constants.getMessage("labelNewCategoryType");            
            while (this.getCategoryType(newName) != null) {
                newName = Constants.getMessage("labelNewCategoryType") + " (" + newNum + ")"; 
				newNum++;
            }
            var editor = new  com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryTypeEditor({
				parent: this,
				index: this._editors.length,
                categoryTypeName: newName
            });			
            this._editors.push(editor);   //cache the editor
            this._categoryTypesNode.appendChild(editor.domNode);    //show the editor
            editor.setInitialFocus();
			
            //this._actionButtons["new"].setDisabled(false);	      
			this._isAdding = false;
        },
		
		_capFirst: function(str) {
			var value = str + "";
			return value.charAt(0).toUpperCase() + value.substring(1).toLowerCase()
		},
		
        _sortValues: function(val1, val2) {
            if (val1.name < val2.name) {
				return -1;
			} else if (val1.name > val2.name) {
				return 1;
			}
			return 0;
        },		
		
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
        }
	});  
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryTypeEditor", [dijit._Widget, dijit._Templated],
    {                   
        templateString: '<table dojoAttachPoint="_categoryTypeEditorNode" class="category-type-editor-table">' +
		                '    <tr class="category-type-row" dojoAttachEvent="onclick:_handleRowClick">' + 
		                '        <th dojoAttachPoint="_nameNode" class="content-column"></th>' +
						'        <th dojoAttachPoint="_ownerNode" class="content-column">${labelOwner}</th>' +
						'        <th dojoAttachPoint="_actionsNode" class="actions-column"></th>' +
		                '    </tr>' +  
						'    <tbody dojoAttachPoint="_categoryTypesNode">' +
		                '        <!-- individual category editor rows will be appended here -->' +
						'    </tbody>' +
						'</table>',	
						
	    labelOwner: Constants.getMessage("labelOwner"),
		
		constructor: function() {
			this.parent = null;
			this.categoryType = null;
			this.editable = false;
			this.inEditMode = false;
			this._editors = [];  //CategoryEditor array; index is rowNum , element is editor instance
		},
		
		postCreate: function() {
			if (!this.categoryType) {
				if (!this.categoryTypeName) {
					throw new Error("Parameter 'categoryTypeName' REQUIRED for new CategoryTypes!");
				}
				this.inEditMode = true;
				this.categoryType = {
					newCategoryType: true,
					name: this.categoryTypeName	
				}
			}
			this.editable = this.hasCategoryTypePermission(PermissionsClient.ACTION_EDIT);
			this._initialize();
		},

        _initialize: function() {
            this._showName();
            this._showCategories();
			if (!this.isNew()) {
				this._showActions();
			}
		},
		
        hasCategoryTypePermission: function(actionId) {
            return this.parent.hasCategoryTypePermission(actionId);
        },

        hasCategoryPermission: function(actionId) {
            return this.parent.hasCategoryPermission(actionId);
        },		
			
        getCategoryType: function() {
            return this.categoryType;
        },		
		
		getCategories: function() {
			if (this.categoryType && this.categoryType.categories) {
				return this.categoryType.categories;
			}
			return null;
		},
		
        addCategory: function(newCategory) {
            if (this.categoryType) {
                if (!this.categoryType.categories) {
                    this.categoryType.categories = [];
                }
                this.categoryType.categories.push(newCategory);
            }           
        },		
		
		getContributors: function() {
			return this.parent.getContributors();
		},
		
        isShowArchived: function() {
            return this.parent.isShowArchived();
        },	
		
		setInitialFocus: function() {
			if (this.inEditMode && this._name) {
				this._name.focus();
			}
		},
		
        isNew: function() {
            return (this.categoryType.newCategoryType == true);
        },		
		
		_showName: function() {		
            //TODO move the elements generated here to the template, as seen in CategoryEditor 
            dojox.data.dom.removeChildren(this._nameNode);
            delete this._name;
            if (this.editable && this.inEditMode) {
				var inputLabel = document.createElement("label");
				inputLabel.setAttribute("for", "category-name-input");
                this._name = document.createElement("input");
                this._name.type = "text";
				this._name.name = "category-name-input";
				this._name.id = "category-name-input";
                this._name.size = 50;
                this._name.maxLength = 50;
                this._name.value = this.categoryType.name;
                this._name.onchange = dojo.hitch(this, "_doEdit");
                //this._name.onblur = dojo.hitch(this, "_doEdit");
                this._name.onkeypress = dojo.hitch(this, "_doEditKeypress");
				inputLabel.appendChild(this._name);
                this._nameNode.appendChild(inputLabel);
            } else {
                this._name = document.createElement("span");
                this._name.innerHTML = this.categoryType.name;
				if (this.editable) {
					dojo.addClass(this._nameNode, "editable");
					//this._name.onclick = dojo.hitch(this, "_toggleEditMode");
				} else {
					dojo.removeClass(this._nameNode, "editable");
				}
                if (this.categoryType.archived) {
                    dojo.addClass(this._name, "archived");
                }
                this._nameNode.appendChild(this._name);             
            }
		},
		
        _showActions: function() {
            dojox.data.dom.removeChildren(this._actionsNode);
            var actions = this._getActions();
            for (var i = 0; i < (actions ? actions.length : 0); i++) {
                this._addAction(actions[i]);   
            }            
        },		
		
        _showCategories: function() {
			if (!this.categoryType ||!this.categoryType.categories) {
				return;
			}
			//initialize the category editors and display them
			this._editors = [];
			dojox.data.dom.removeChildren(this._categoryTypesNode);			
			this.categoryType.categories.sort(this._sortCategories);
            for (var i = 0; i < this.categoryType.categories.length; i++) {
				//create an editor for the CategoryType
				var category = this.categoryType.categories[i];
                if (category.archived == false || this.isShowArchived()) {
				    this._appendCategory(category);	
				}				
			}			
        },

        _appendCategory: function(category) {
            var editor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryEditor({
                parent: this,
				index: this._editors.length,
                category: category
            });
			this._editors.push(editor); //cache the editor
            this._categoryTypesNode.appendChild(editor.domNode);			
		},
		
		removeEditor: function(editor) {
			var index = editor.index; //this._getCategoryIndex();
			this._editors.splice(index, 1);
			this._categoryTypesNode.deleteRow(index);
		},	
		
        _sortCategories: function(val1, val2) {
            if (val1.name < val2.name) {
                return -1;
            } else if (val1.name > val2.name) {
                return 1;
            }
            return 0;
        },  	
		
        _getActions: function() {
            var actions = []; 
            if (this.hasCategoryTypePermission(PermissionsClient.ACTION_ARCHIVE)) {
	            if (!this.getCategoryType().archived) {
	                actions.push({
	                    label: Constants.getMessage("labelArchiveCategoryType"),
	                    actionHandler: dojo.hitch(this, "_doArchive"),
	                    enabledIcon: ImageCache.ARCHIVE_ICON_ENABLED,
	                    disabledIcon: ImageCache.ARCHIVE_ICON_DISABLED,
	                    disabledOnReadOnly: true
	                });
	            } else {
	                actions.push({
	                    label: Constants.getMessage("labelUnarchiveCategoryType"),
	                    actionHandler: dojo.hitch(this, "_doUnarchive"),
	                    enabledIcon: ImageCache.RESTORE_ICON_ENABLED,
	                    disabledIcon: ImageCache.RESTORE_ICON_DISABLED,
	                    disabledOnReadOnly: true
	                });             
	            }
            }			
			if (this.hasCategoryPermission(PermissionsClient.ACTION_CREATE)) {
				actions.push({
					label: Constants.getMessage("labelAddCategory", this.getCategoryType().name),
					actionHandler: dojo.hitch(this, "_handleAddNew"),
					enabledIcon: ImageCache.ADD_ICON_ENABLED,
					disabledIcon: ImageCache.ADD_ICON_DISABLED,
					disabledOnReadOnly: true
				});
			}
            return actions; 
        },		
		
        _addAction: function(action) {			
            var hover = action.disabledIcon && action.enabledIcon;
            var button = document.createElement("img");
            dojo.addClass(button, "action-button");
            button.src = hover ? action.disabledIcon : action.enabledIcon;
            button.title = action.label;
            button.onclick = function(event) {
				dojo.stopEvent(event);
				action.actionHandler();
			}      
			if (hover) {
                button.onmouseover = function() { button.src = action.enabledIcon; }
                button.onmouseout = function() { button.src = action.disabledIcon; }
            }
            this._actionsNode.appendChild(button);  			     
        },		
		
		_handleAddNew: function() {
            //protect against concurrent adds
            if (this._isAdding) {
                return;
            } else {
                this._isAdding = true;
            }
			
            var newNum = 1;
			var newName = Constants.getMessage("labelNewCategory", this.getCategoryType().name);			
			while (this.getCategory(newName) != null) {
				newName = Constants.getMessage("labelNewCategory", this.getCategoryType().name) + " (" + newNum + ")";
				newNum++;
			}
            var editor = new com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryEditor({
                parent: this,
				index: this._editors.length,
                categoryName: newName
            });
            this._editors.push(editor); //cache the editor
            this._categoryTypesNode.appendChild(editor.domNode);
			editor.setInitialFocus();
			
			this._isAdding = false;
		},
		
		getCategory: function(name) {
			for (var i = 0; i < (this._editors ? this._editors.length : 0); i++) {
                var category = this._editors[i].getCategory();
				if (category.name.toUpperCase() == name.toUpperCase()) {
					return category;
				}			 
			}
			return null;
		},	
		
        _handleRowClick: function(event) {
            dojo.stopEvent(event);
            if (!this.inEditMode) {
                this._toggleEditMode();
            }
        },		
		
        _toggleEditMode: function() {
            this.inEditMode = !this.inEditMode;
            this._showName();
			this.setInitialFocus();
        },
        
        _doEditKeypress: function(e) {            
            if (e.keyCode == dojo.keys.ENTER) {
                //perform the edit on ENTER pressed 
                this._doEdit();
            } else if (e.keyCode == dojo.keys.ESCAPE) {
                //on ESCAPE - cancel out if existing
                if (!this.categoryType.newCategoryType) {
                    this._toggleEditMode();
                } else {
                    //'un-create' if new 
	                this.parent.removeEditor(this);
                }              
				dojo.stopEvent(e); //prevent Dialog from closing 
            }
        },
        
        _doEdit: function() {
            if (this._name.value != null && dojo.trim(this._name.value).length > 0) {
                if (!this._validate()) {
                    //revert if invalid for edit
                    this._name.value = this.categoryType.name;
                    //this._toggleEditMode();
                    return;
                }
                this.categoryType.name = this._name.value;
                this._save();               
            } else {
                //don't allow empty name - reset name
                this._name.value = this.categoryType.name;
                this._name.focus();				
			}
        },
		
        _validate: function() {
			/*
            if (!this.categoryType.newCategoryType && this._name.value == this.categoryType.name) {
                return false;
            }
            */
			//validate the name
			var existing = this.parent.getCategoryType(this._name.value);
            if (this._name.value != this.categoryType.name && this._name.value.toUpperCase() == this.categoryType.name.toUpperCase()) {
				//allow renaming of same category type with change to case only
				return true; 
			} else if (existing && existing.itemId && existing.itemId != this.categoryType.itemId) {
                //disallow naming/renaming of category type to already existing category type
                alert(Constants.getMessage("msgCategoryTypeExists", this._name.value));
                return false;               
            }
            return true;
        },
        
        _showSaving: function() {
            dojox.data.dom.removeChildren(this._nameNode);
            this._nameNode.innerHTML = "Saving...";
        },
        
        _doArchive: function(event) {
			this.categoryType.archived = true;
            this._save();
        },
        
        _doUnarchive: function() {
            this.categoryType.archived = false;
            this._save();
        },
				
        _save: function() {
            this._showSaving();
            var postObj = {
                newCategoryType: this.categoryType.newCategoryType ? true : false, 
				itemType: this.parent.getArtifactType(),             
                name: this.categoryType.name,
				archive: this.categoryType.archived ? true : false             
            }
            if (this.categoryType.itemId) {
                postObj.itemId = this.categoryType.itemId;
            }
            var srh = new ServiceResponseHandler(this, "_saved", "_error");
            CategoryClient.postCategoryType(srh, postObj);                      
        },      
        
        _saved: function(categoryType) {
            if (this.isNew()) {
                this.parent.addCategoryType(categoryType);
            }			
            this.categoryType = categoryType;
            this.inEditMode = false;
            this._initialize();
            //hide this editor if its been archived
            if (categoryType.archived && !this.isShowArchived()) {
                dojo.addClass(this.domNode, "hidden");
            } else {
                dojo.removeClass(this.domNode, "hidden");
            }	
			
            var type = null;
            
			//set catched TC CategoryTypeSummaries dirty		
			if(this.parent.getArtifactType() == Constants.MODEL_TYPE_CASE) {
                type = Constants.TYPE_TC_CATEGORY;
			}
            else if(this.parent.getArtifactType() == Constants.MODEL_TYPE_PLAN) {
                type = Constants.TYPE_TP_CATEGORY;
            }
            if(type) {
                dojo.publish(GlobalEvents.ArtifactsUpdated, [ type,  this.parent.categoryTypes] );
            }
        },
				
        _error: function(e) {
            console.error(this.declaredClass + ": Error: " + e.message);
        }			       
    });    
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.CategoryEditor", [dijit._Widget, dijit._Templated],
    {                   
        templateString: '    <tr dojoAttachPoint="_categoryEditorNode" class="category-row" dojoAttachEvent="onclick:_handleRowClick">' +                        
                        '        <td dojoAttachPoint="_nameNode" class="content-column">' +
                        '            <span dojoAttachPoint="_nameText"></span>' +
                        '            <input dojoAttachPoint="_name" dojoAttachEvent="onchange:_doEdit,onkeypress:_doEditKeypress" type="text" size="${nameInputSize}" maxlength="${nameInputMax}" class="hidden"></input>' +
                        '        </td>' +												
						'        <td dojoAttachPoint="_ownerNode" class="content-column">' +
						'            <label for="ownerSelect" dojoAttachPoint="_ownerText"></label>' +
						'            <select id="ownerSelect" name="ownerSelect" dojoAttachPoint="_owner" dojoAttachEvent="onclick:_clickOwner, onchange:_doEditOwner" class="owner-select hidden"><option value="">${labelLoading}</option></select>' +
						'        </td>' +
                        '        <td dojoAttachPoint="_actionsCell" class="actions-column">' +
						'            <div dojoAttachPoint="_actionsNode" class="hidden"></div>' +
						'        </td>' +						
                        '    </tr>',
						
		labelLoading: Constants.getMessage("labelLoading"),	
		nameInputSize: 25,			
        nameInputMax: 50,
		
        constructor: function() {
			this.parent = null;
            this.category = null;
			this.editable = false;
			this.inEditMode = false;
			this._isSaving = false;
			this.editOwnerFinish = false;
			this.clickingOwner = false;
        },
        
        postCreate: function() {
            if (!this.parent) {
                throw new Error("Parameter 'parent' REQUIRED!");
            }
            if (!this.category) {
                if (!this.categoryName) {
                    throw new Error("Parameter 'categoryName' REQUIRED for new Categories!");
                }
				this.inEditMode = true;
                this.category = {
                    newCategory: true,
                    name: this.categoryName 
                }
            }
			this.editable = this.parent.hasCategoryPermission(PermissionsClient.ACTION_EDIT);
			this._initialize();
        },
		
        getCategory: function() {
            return this.category;
        },
		
		isNew: function() {
			return (this.category.newCategory == true);
		},
		
        setInitialFocus: function() {
            if (this.inEditMode && this._name) {
                this._name.focus();
            }
        },		

        _initialize: function() {
            this._showInfo();
            this._showActions();
            this._setupHover();
		},
				
        _showInfo: function() {
			//if in edit mode
            if (this.editable && this.inEditMode) {
				//name
                this._name.value = this.category.name;                
                dojo.removeClass(this._name, "hidden");
                dojo.addClass(this._nameText, "hidden");
				
				//owner
                dojo.removeClass(this._owner, "hidden");
                dojo.addClass(this._ownerText, "hidden");
                dojox.data.dom.removeChildren(this._owner);
                var contributors = this.parent.getContributors();
                for (var i = 0; i < contributors.length; i++) {
					var o = new Option(contributors[i].name, contributors[i].itemId);
					var selected = null;
					if (this.category && this.category.itemId && this.category.itemId == contributors[i].itemId) {
						selected = contributors[i];
						o.selected = true;						
					} else if (selected == null && contributors[i].name == UNASSIGNED) {
						o.selected = true;
					}
					SelectUtility.add( this._owner, o );
				}
				
				//toggle row's editable style
                dojo.removeClass(this._categoryEditorNode, "editable");
								
            } else {
				//name
//                this._nameText.innerHTML = this.category.name;
				dojox.data.dom.removeChildren(this._nameText);
				var text= document.createTextNode(this.category.name);
				this._nameText.appendChild(text);
				
                dojo.removeClass(this._categoryEditorNode, "editable");
                dojo.removeClass(this._nameText, "hidden");
                dojo.addClass(this._name, "hidden");
                if (this.category.archived) {
                    dojo.addClass(this._nameText, "archived");
                }
				else //Not archived.  Remove the style if applied
                   dojo.removeClass(this._nameText, "archived");
 				
				//owner
				if (this.category.ownerItemId) {
					var owner = this._getContributor(this.category.ownerItemId);
					this._ownerText.innerHTML = owner ? owner.name : UNKNOWN;
				} else {
					this._ownerText.innerHTML = UNASSIGNED;
				}                
                dojo.removeClass(this._ownerText, "hidden");
                dojo.addClass(this._owner, "hidden");
				
                //toggle row's editable style
                dojo.addClass(this._categoryEditorNode, "editable");				 				  
            }
			this.clickingOwner = false;
			this.editOwnerFinish = false;
        },
		
		_getContributor: function(itemIdOrName) {
            var contributors = this.parent.getContributors();
            for (var i = 0; i < (contributors ? contributors.length : 0); i++) {
                if (contributors[i].itemId == itemIdOrName) {
                    return contributors[i];
                } else if (contributors[i].name == itemIdOrName) {
					return contributors[i];
				}
            }
			return null;
		},
		
        _showActions: function() {
            dojox.data.dom.removeChildren(this._actionsNode);
			var actions = this._getActions();
			for (var i = 0; i < actions.length; i++) {
			 	this._addAction(actions[i]);   
			}            
        },		
		
		_setupHover: function() {
			this.domNode.onmouseover = dojo.hitch(this, function() {
				dojo.removeClass(this._actionsNode, "hidden")
			});
            this.domNode.onmouseout = dojo.hitch(this, function() {
				dojo.addClass(this._actionsNode, "hidden");
			});
		},
		
		_getActions: function() {
            var actions = [];   
			if (this.category.newCategory) {
				return actions;
			}   
			if (this.parent.hasCategoryPermission(PermissionsClient.ACTION_ARCHIVE)) {
				if (!this.category.archived) {
					actions.push({
						label: Constants.getMessage("labelArchiveCategory", this.parent.getCategoryType().name),
						actionHandler: dojo.hitch(this, "_doArchive"),
						enabledIcon: ImageCache.ARCHIVE_ICON_ENABLED,
						disabledIcon: ImageCache.ARCHIVE_ICON_DISABLED,
						disabledOnReadOnly: true
					});
				}
				else {
					actions.push({
						label: Constants.getMessage("labelUnarchiveCategory", this.parent.getCategoryType().name),
						actionHandler: dojo.hitch(this, "_doUnarchive"),
						enabledIcon: ImageCache.RESTORE_ICON_ENABLED,
						disabledIcon: ImageCache.RESTORE_ICON_DISABLED,
						disabledOnReadOnly: true
					});
				}
			}
			return actions;	
		},
		
		_addAction: function(action) {
            var hover = action.disabledIcon && action.enabledIcon;
			var button = document.createElement("img");
			dojo.addClass(button, "action-button");
			button.src = hover ? action.disabledIcon : action.enabledIcon;
			button.title = action.label;
            button.onclick = function(event) {
                dojo.stopEvent(event);
                action.actionHandler();
            }
			if (hover) {
				button.onmouseover = function() { button.src = action.enabledIcon; }
				button.onmouseout = function() { button.src = action.disabledIcon; }
			}			
			this._actionsNode.appendChild(button);			
		},
		
        _handleRowClick: function(event) {
            dojo.stopEvent(event);
			if (!this.inEditMode) {
				this._toggleEditMode();
			}			
		},
		
		_toggleEditMode: function() {
			this.inEditMode = !this.inEditMode;
			this._showInfo();
			this.setInitialFocus();
		},
		
		_doEditKeypress: function(e) {
            if (e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.TAB) {
			    //perform the edit on ENTER or TAB pressed
			    this._doEdit();
            } else if (e.keyCode == dojo.keys.ESCAPE) {
                //on ESCAPE - cancel out if existing
                if (!this.category.newCategory) {
                    this._toggleEditMode();
                } else {
                    //'un-create' if new 
                    this.parent.removeEditor(this);
                }				
				dojo.stopEvent(e); //prevent Dialog from closing
			}
		},
		
		_doEdit: function() {
			if (this._name.value && dojo.trim(this._name.value).length > 0) {
				if (!this._validate()) {
					//revert if invalid for edit
					this._name.value = this.category.name;
                    //this._toggleEditMode();
					return;
                }
                this.category.name = this._name.value;
				this._save();				
			} else {
                //don't allow empty name - reset name
                this._name.value = this.category.name;
                this._name.focus(); 				
			}
		},	
		
		_doEditOwner: function() {
			this.editOwnerFinish = true;
			this._doEdit();
		},
		
		_clickOwner: function() {
			this.clickingOwner = true;
		},
		
		_validate: function() {	
            /*
            if (!this.category.newCategory && this._name.value.toUpperCase() == this.category.name.toUpperCase()) {
                return false;
            }
            */		
            //validate the name
			var existing = this.parent.getCategory(this._name.value);
            if (this._name.value == this.category.name && this._name.value.toUpperCase() != this.category.name.toUpperCase()) {
                //allow renaming of same category with change to case only
                return true; 
            } else if (existing && existing.itemId && existing.itemId != this.category.itemId) {
                //disallow naming/renaming of category to already existing category
                alert(Constants.getMessage("msgCategoryExists", this.parent.getCategoryType().name, this._name.value));
                return false;               
            }
			return true;
		},
		
		_setSaving: function() {
			this._isSaving = true;
			dojox.data.dom.removeChildren(this._nameText);
			this._nameText.appendChild(document.createTextNode(Constants.getMessage("labelSavingArtifact")));
		},
		
		_endSaving: function() {
			this._isSaving = false;
		},
		
        _doArchive: function() {
            this.category.archived = true;
			this._save();
        },
        
        _doUnarchive: function() {
            this.category.archived = false;
            this._save();
        },		
		
        _save: function() {
            if (this._isSaving) {
                return false;
            } else {
                this._setSaving();
            }
            var postObj = {
                newCategory: this.category.newCategory ? true : false,              
                name: this.category.name,
                typeItemId: this.parent.getCategoryType().itemId,
				archive: this.category.archived ? true : false                  
            }
            if (this.category.itemId) {
                postObj.itemId = this.category.itemId;
            }
			if (this._owner.value != null && dojo.trim(this._owner.value).length > 0) {
				var contributor = this._getContributor(dojo.trim(this._owner.value));
				var unassignedContributor = this._getContributor("Unassigned");
				if (contributor) {
				    var hasOwner = (this.category.ownerItemId != null);
					if ((this.isNew() || !hasOwner) && contributor.itemId == unassignedContributor.itemId) {
						//do not specify Unassigned owner for new categories or for categories that don't 
						//already have an owner.  In other words, force user to explicitly specifiy an owner. 
					} else {
						postObj.ownerItemId = contributor.itemId;
					}
				}				
			}
            var srh = new ServiceResponseHandler(this, "_saved", "_error");
            CategoryClient.postCategory(srh, postObj);                      
        },		
		
		_saved: function(category) {
            if (this.isNew()) {
                this.parent.addCategory(category);
            }			
			this.category = category;
			this.inEditMode = false;
			if(!this.clickingOwner || this.editOwnerFinish)
				this._initialize();
            //hide this editor if its been archived
            if (category.archived && !this.parent.isShowArchived()) {
                dojo.addClass(this.domNode, "hidden");
            } else {
                dojo.removeClass(this.domNode, "hidden");
            }	
			this._endSaving();		
			
            var type = null;

			//refresh catched Test Case CategoryTypeSummaries		
			if(this.parent.parent.getArtifactType() == Constants.MODEL_TYPE_CASE) {
                type = Constants.TYPE_TC_CATEGORY;
			}
            else if(this.parent.parent.getArtifactType() == Constants.MODEL_TYPE_PLAN) {
                type = Constants.TYPE_TP_CATEGORY;
            }
            if(type) {
                dojo.publish(GlobalEvents.ArtifactsUpdated, [ type, this.parent.parent.categoryTypes]);
            }
		},		
		
		_error: function(e) {
			console.error(this.declaredClass + ": Error: " + e.message);
		}
		
    });		
})();


