/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.StateTransitionConstraintContentArea");

dojo.require("com.ibm.asq.common.web.client.internal.option");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.StateTransitionConstraintTableView");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea");

(function(){
	var Constants 				= com.ibm.rqm.planning.web.client.internal.Constants;
	var getCurrentProjectArea	= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
	var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var ARTIFACT_STATE_TRANSITION_CONSTRAINTS_CATEGORY = Constants.ARTIFACT_STATE_TRANSITION_CONSTRAINTS_CATEGORY;
	var EditorUtil				= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.StateTransitionConstraintContentArea", [com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyContentArea], {
        
        name: "",
        text: "",
        content: "",
        csHelpId: "rqm_admin_artifact_state.html",
        csHelpLabel: Constants.getMessage("labelArtifactStateTransitionConstraintsHelp"),
        
        setName: function(){
            this.name = Constants.getMessage("labelStateTransitionConstraint");
            dojox.data.dom.removeChildren(this._name);
            var name = document.createTextNode(this.name);
            this._name.appendChild(name);
        },
        
        setText: function(){
            this.text = Constants.getMessage("labelSetStateTransitionConstraint");
            dojox.data.dom.removeChildren(this._text);
            var text = document.createTextNode(this.text);
            this._text.appendChild(text);
        },
        
        setContent: function(){
			this._getTeamRoles();
        },
		
		_getTeamRoles: function(){
			var responsehandler = new ServiceResponseHandler(this, "_getConstrainsPreference", "_error");
			com.ibm.team.process.web.client.internal.ProcessClient.getProjectAreaByUUID({uuid: getCurrentProjectArea().itemId}, responsehandler);
		},
		
		_getConstrainsPreference: function(projectArea){
			this.roles = projectArea.processDTO.roles;
			var option = new com.ibm.asq.common.web.client.internal.option();
			var responsehandler = new ServiceResponseHandler(this, "_displayConstrainsPreference", "_error")
			option.getProjectPreferences(responsehandler, getCurrentProjectArea().itemId, ARTIFACT_STATE_TRANSITION_CONSTRAINTS_CATEGORY);
			
		},
		
		_displayConstrainsPreference: function(constrainsPreference){
			this.constrainsPreference = constrainsPreference;
			var thePreferences = null;
			var constrains = Constants.STATETRANSITION_PREFERENCE_DEFAULT;
			if(this.constrainsPreference == null)
			{
				thePreferences = constrains;
			}
			else
			{
				thePreferences = dojo.fromJson(this.constrainsPreference.value).constrains;
			}
			
			var actions = [];
			
//			actions.push({
//	            actionId: "save",
//				label: Constants.getMessage("labelSaveConstrains"),
//                actionHandler: dojo.hitch(this, "_handleSaveConstrains"),
//                iconClass: "save-icon-image"                 
//	        });
			
			actions.push({
	            label: Constants.getMessage("labelRefresh"),
	            actionHandler: dojo.hitch(this, "_getTeamRoles"),
	            iconClass: "refresh-icon-image",
	            disabledOnReadOnly: true                 
	        });
			
			var args = {
				content: thePreferences, 
				parent: this,
				actions: actions
			};
			this.tableView = new com.ibm.rqm.planning.web.ui.internal.view.administration.StateTransitionConstraintTableView(args);
			this.tableView.initialize();
            dojox.data.dom.removeChildren(this._content);
            this._content.appendChild(this.tableView.domNode);
			this.setDirty(false);
		},
		
		_handleSaveConstrains: function(){
			var option = new com.ibm.asq.common.web.client.internal.option();
			this.parent.addSaveParticipantNum();
			var self = this;
			var handler = {
				success: function(results){
					self.parent.saveFinished();
					self._saveFinished(results);
				},
				error: function(e){
					self.parent.saveFinished();
					self._error(e);
				}
			}
			var responsehandler = new ServiceResponseHandler(handler, "success", "error");
			var args = {
					constrains: this.tableView.tableViewer.getContent()
				}
			if(this.constrainsPreference){
				option.updateProjectPreferences(responsehandler, getCurrentProjectArea().itemId, this.constrainsPreference.itemId, dojo.toJson(args));
			}else{
				option.createProjectPreferences(responsehandler, getCurrentProjectArea().itemId, ARTIFACT_STATE_TRANSITION_CONSTRAINTS_CATEGORY, dojo.toJson(args));
			}
//			this.tableView.saveActionNode.setDisabled(true);
			this.setDirty(false);
		},
		
        //@Abstract: Override to refresh the table when we're in display.
        refresh: function(){
			if(this.tableView)
                this.tableView.refresh();
        },

		_saveFinished: function(constrainsPreference){
			this.constrainsPreference = constrainsPreference;
			//Reload the cache of state transitions maintained by EditorUtil
			EditorUtil.loadStateTransitionConstraintSettings(null, true);
		},
        
        getValues: function(){
            return [{
				key: ARTIFACT_STATE_TRANSITION_CONSTRAINTS_CATEGORY,
				value: {
					constrains: this.tableView.tableViewer.getContent()
				}
			}];
        },
        
        _addCSHelpID: function(){
            var csHelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp();
            csHelp.contextID = this.csHelpId;
            this._cshelp.appendChild(csHelp.domNode);
			csHelp.setButtonLabel(Constants.getMessage("labelArtifactStateTransitionConstraintsHelp"));
        },
		
		onSave: function(){
			this._handleSaveConstrains();
		},
		
		_error: function(error){
            var message = error && error.message ? error.message : Constants.getMessage("msgSysPropErrorNoDetails");
            this.setErrorMessage(Constants.getMessage("msgSysPropError"), message);
		}
    });
    
})();
