/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea");

dojo.require("dijit.TitlePane");
dojo.require("dijit.form.Button");
dojo.require("dijit.InlineEditBox");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea",
dijit.TitlePane,
{
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-web-ui-more-less-collapsible-area">' +
                    '  <div class="header" dojoAttachPoint="focusNode" class="header">' +
                    '    <div class="header-left" dojoAttachPoint="leftHeaderDiv">' +
                    '      <div class="header-right" dojoAttachPoint="rightHeaderDiv">' +
                    '        <div class="header-mid" dojoAttachPoint="midHeaderDiv">' +
                    '          <span dojoAttachPoint="arrowNodeInner" style="display:none"></span>' +
                    '          <div name="${openCloseLabel}" dojoAttachEvent="onclick:toggle,onkeypress:_onTitleKey" tabindex="0" waiRole="button"' +
                    '               dojoAttachPoint="arrowNode">' +
//                    '            <span dojoAttachPoint="arrowLabelNode" class="arrow-label"></span>' +
                    '          </div>' +
                    '          <div class="actions-container" dojoAttachPoint="actionsContainerNode">' +
                    '          </div>' +
                    '          <div class="title-container">' +
                    '            <span dojoAttachPoint="titleNode" class="title"></span>' +
					'            <span dojoAttachPoint="editableTitleNode" class="title" dojoType="dijit.InlineEditBox"></span>' +
                    '            <span dojoAttachPoint="titleHelpNode" class="title-help"></span>' +                    
                    '          </div>' +
                    '        </div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '  <div class="content-outer-container" dojoAttachPoint="hideNode">' +
                    '    <div class="content-inner-container" dojoAttachPoint="wipeNode">' +
                    '      <div class="content-area" dojoAttachPoint="containerNode,contentNode" waiRole="region" tabindex="-1"></div>' +
                    '      <div class="content-bottom-container">' +
                    '        <div class="content-bottom-left">' +
                    '          <div class="content-bottom-right">' +
                    '            <div class="content-bottom-mid">' +
                    '            </div>' +
                    '          </div>' +
                    '        </div>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    open: true,
    collapsedTitle: "",  // title to show when collapsed
    widgetsInTemplate: true,
	
	//Set this to true to allow the label of the panel to be editable.
	//see comments above regarding using this feature.
	editableLabel: false,
	
	isFirstExpanding: true,

    contextHelpId: null,
    csHelpProviderId: null,
    csHelpEngine: null,
	
	openCloseLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelToggle"),
    
    postCreate: function() {
        com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea.superclass.postCreate.apply(this, arguments);
		//If this is an editable lable then hide the plain title div area and set the
		//style for the header so that it stands out from plain panels
		if (this.editableLabel == true)
		{
			this.titleNode.style.display = "none";
			this.editableTitleNode.domNode.style.display = "inline";
		}
		else  //Not editable so hide the editable widget
			this.editableTitleNode.domNode.style.display = "none";
			
        this._updateCaret();
		this.isFirstExpanding = true; // reset the flag for each widget instance created.
		this.actionMap = {};
		
		if (this.open && this.isFirstExpanding) {
			this.OnFirstExpand(this);
			this.isFirstExpanding = false;
		}
        // load in the cshelp widget if an help id is set.
        if(this.contextHelpId)
            this.setCSHelpId(this.contextHelpId, this.csHelpProviderId, this.csHelpEngine);
    },
    
    setCSHelpId: function(helpId, csHelpProviderId, csHelpEngine){
        this.contextHelpId = helpId;
        this.csHelpProviderId = csHelpProviderId;
        this.csHelpEngine = csHelpEngine;
        if (this.titleHelpNode) {
            dojox.data.dom.removeChildren(this.titleHelpNode);
            if (this.contextHelpId) {
                var args = {
                    'contextID': helpId,
                    'csHelpEngine': this.csHelpEngine,
                    'csHelpProviderId': this.csHelpProviderId
                };
                var cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.titleHelpNode.appendChild(cshelp.domNode);
            }
        }
    },

    addAction: function(label, handler, callbackData, iconClass) {
        var arg = {
            label: label,
            showLabel: true,  // will not show label if icon is specified
            onClick: function( e ) {
                if (!this.parent.open) {
                    this.parent.toggle();
                }
                this.clientCallback(this.callbackData, e);
            }
        };
        
        if (iconClass) {
            arg["iconClass"] = iconClass;
            arg["showLabel"] = false;
        }
        var button = new dijit.form.Button(arg);
        dojo.addClass(button.domNode, "com-ibm-asq-common-web-ui-more-less-collapsible-area-action");
        
        // store the client callback and data into the button
        button.parent = this;
        button.clientCallback = handler;
        button.callbackData = callbackData;
        
        this.actionsContainerNode.appendChild(button.domNode);
        this.actionMap[ label ] = button;
        
        return button;
    },
	
	removeAction: function(buttonWidget) {
		try {
			this.actionsContainerNode.removeChild(buttonWidget.domNode);
						
		} catch(e) {
			//no such action, do nothing
		}
	},
    
    setTitle: function(title) {
        this.title = title;
        this._updateTitle();
    },
    
    setCollapsedTitle: function(title) {
        this.collapsedTitle = title;
        this._updateTitle();
    },
    
    toggle: function() {
        com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea.superclass.toggle.apply(this, arguments);
        this._updateCaret();
        
		if (this.isFirstExpanding) {
			this.OnFirstExpand(this);
			this.isFirstExpanding = false;
		}
		
        // notify the client if they override this method.
        this.onToggle(this);
    },
    
    _updateCaret: function() {
        if (this.open) {
//            this.arrowLabelNode.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_Less");
            dojo.addClass(this.arrowNode, "arrow-icon-up");
            dojo.removeClass(this.arrowNode, "arrow-icon-down");
        } else {
//            this.arrowLabelNode.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_More");
            dojo.addClass(this.arrowNode, "arrow-icon-down");
            dojo.removeClass(this.arrowNode, "arrow-icon-up");
        }
        this._updateTitle();
    },
    
    _updateTitle: function() {
		if (this.editableLabel) 
		{
			//Editable so use the dijit.InlineEditBox methods to set the data to display/edit
			this.editableTitleNode.value = this.title;
			this.editableTitleNode._setDisplayValue(this.title);
		}
		else 
		{
			if (!this.open) 
			{
				if ((this.title != "") && (this.collapsedTitle != "")) 
				{
					this.titleNode.innerHTML = "<strong>" + this.title + "</strong> | " + this.collapsedTitle;
				}
				else 
					if (this.title != "") 
					{
						this.titleNode.innerHTML = "<strong>" + this.title + "</strong>";
					}
					else 
					{
						this.titleNode.innerHTML = this.collapsedTitle;
					}
			}
			else 
			{
				if (this.title != "") 
				{
					this.titleNode.innerHTML = "<strong>" + this.title + "</strong>";
				}
			}
		}


    },
    
    _onTitleClick: function() {
        this.toggle();
    },
    
    onToggle: function(widget) {
        
    },
	
	setEditableTitle : function(editable)
	{
		this.editableLabel = editable;
		if (this.editableLabel == true)
		{
			this.titleNode.style.display = "none";
			this.editableTitleNode.domNode.style.display = "inline";
		}
		else  //Not editable so hide the editable widget
 		{
			this.titleNode.style.display = "inline";
			this.editableTitleNode.domNode.style.display = "none";
		}
			
		this._updateTitle();
		
	},
	
	/**
	 * Subclass to override if need to do specific things during the first expand. 
	 * @param {Object} widget
	 */
	OnFirstExpand: function(widget) {
		
	},
	
	getActionButton: function( label )
	{
		return( ( this.actionMap ) ? this.actionMap[ label ] : null );
	}
       
});    
