/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function() {

var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants               = com.ibm.rqm.planning.web.client.internal.Constants;
var TestCaseClient		    = com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
var TestScriptClient        = com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
var TestScriptTableView     = com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView;
var TestCaseTableView       = com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView;
var CSHelp                  = com.ibm.asq.common.web.ui.internal.util.CSHelp;


// cap max number of query results at 1000
var MAX						= 1000;

var RUNNING_SEARCH_MSG		= Constants.getMessage("labelRunningSearch");
var FETCH_ERR_MSG			= Constants.getMessage("msgErrorRunningQuery");

dojo.declare(
	"com.ibm.rqm.planning.web.ui.internal.view.common.SelectItemTableDialog",
	[dijit._Widget, dijit._Templated],
	{
		//templatePath: xojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/SelectItemTableDialog.html"),
        templateString: '<div class="com-ibm-rqm-test-common-select-item-table">' +
		                '    <div dojoAttachPoint="_msgArea" class="NormalStatus"></div>' +
						'    <div dojoAttachPoint="_itemListArea" class="ItemList"></div>' +
                        '</div>',
		
        width: "800px",   //default dialog width
        
		/*
		invokeTimes: 0,
		returnedNum: 0,
        */
		
        constructor: function(params){
            this._items = [];
			//this._allSnapshots = [];  //all artifacts including snapshots
			dojo.mixin(this, params);			
            if (!this.itemType) {
                throw new Error("Missing parameter 'itemType' is REQUIRED!");
            }           
            if (!dojo.isFunction(this.onOk)) {
                throw new Error("Missing function for 'onOk' parameter");
            }       
            this.setExclusionList(this.exclusionList ? this.exclusionList : []);
			
			if(dojo.isIE) {
				//for defect 9494
				MAX = 50;
			}
        },		
		
		postCreate: function() {            
            //only enable snapshots option for test case item type
            this._enableSnapshots = (this.itemType == Constants.TYPE_CASE);
			//this._showSnapshots = false;
			this.exclusionMap= {};
			this._initItemList();
		},
		
        _showAsDialog: function() {
            var dialogArgs = {
                primaryTitle: this.title ? this.title : Constants.getMessage("labelTitle"),
                okCallback: dojo.hitch(this, "_onOK"),
                cancelCallback: dojo.hitch(this, "_onCancel"),
				_truncate: function() {}  //disable the Dialog's '_truncate'
            };          
                  
            if (this.showQueryFilters) {
                // Jazz uses super large z-index of 10000 which covers up the drop down menu
                // of the filters dojo widgets in the dialog which has z-index of 1000
                dialogArgs.zIndex = 998;
                dialogArgs.forceZIndex = true;
            }
            
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            if (this.itemType == Constants.TYPE_CASE) {
    	        var cshelp = new CSHelp({contextID: "rqm_add_test_cases.html"});
    	        cshelp.setButtonLabel(Constants.getMessage("labelAddExistingCSHelp"));
	            this._dialog.setSecondaryTitleNode(cshelp.domNode);
			} else if (this.itemType == Constants.TYPE_SCRIPT) {
	            var cshelp = new CSHelp({contextID: "rqm_add_test_scripts.html"});
	            cshelp.setButtonLabel(Constants.getMessage("labelAddExistingCSHelp"));
	            this._dialog.setSecondaryTitleNode(cshelp.domNode);
		    }	
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
			this._dialog.open(false);
			this._dialog.updatePosition();
			this._dialog.getOkButton().disabled
        },		
		
        repositionDialog: function() {
            if (this._dialog) {
                // set timeout to allow the browser to render dialog
                setTimeout(dojo.hitch(this._dialog, "updatePosition"), 100);
            }
        },
        
		_initItemList: function() {
			if (this.itemType == Constants.TYPE_SCRIPT) {
                var tableArgs = {
                    scripts: [], 
                    profile: Constants.TABLE_PROFILE_MINIMAL                
                }				
                tableArgs.showQueryFilters = this.showQueryFilters;
                tableArgs.queryFiltersCallback = dojo.hitch(this, "repositionDialog");
				tableArgs.postQueryFilterCallback = dojo.hitch(this, "_filterExcluded");
				tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");
				this._list = new TestScriptTableView(tableArgs);
			
			} else if (this.itemType == Constants.TYPE_CASE) {
				var tableArgs = {
					testCases: [],
                    profile: Constants.TABLE_PROFILE_MINIMAL
				}
                tableArgs.showQueryFilters = this.showQueryFilters;
                tableArgs.allowQuerySnapshots = this._enableSnapshots;				
                tableArgs.queryFiltersCallback = dojo.hitch(this, "repositionDialog");
                tableArgs.postQueryFilterCallback = dojo.hitch(this, "_filterExcluded");		
				tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");		
				this._list = new TestCaseTableView(tableArgs);
			} else {
				//HACK - allow other types; default to script view due to KeywordList using this widget improperly
				console.warn("Warning: Unsupported ItemType: '" + this.itemType + "'; Defaults to TestScriptTableView, but Widget may not function correctly!");
                var tableArgs = {
                    scripts: [], 
                    profile: Constants.TABLE_PROFILE_MINIMAL                
                }               
                tableArgs.showQueryFilters = this.showQueryFilters;
                tableArgs.queryFiltersCallback = dojo.hitch(this, "repositionDialog");
				tableArgs.rowCheckedListener = dojo.hitch(this, "_onTableRowsChecked");
                this._list = new TestScriptTableView(tableArgs);				
			}			
			
		},
		
        show: function(items){
			if (!this._dialog) {
                this._showAsDialog();
            }
            this.refresh(items);			            
        },		
        /*				
		reset: function() {
			this._clearStatusMessage();
			//this._retArr = [];
			//this._snapshotsToLoad = [];
			//this._showSnapshots = false;
			this._list = null;
			//this._showSnapshotsCheckbox.checked=false;
			this.exclusionMap = {};
		},
		*/
		
		refresh: function(items) {
			this._dialog.getOkButton().disabled = true;
			if (this._list == null) {
				this._initItemList();
			}
            if (this.showQueryFilters) {
                this._setItems([]);
            } else if (items) {
				this._setItems(items);
			} else {
                this._setStatusMessage(RUNNING_SEARCH_MSG);
                var srh = new ServiceResponseHandler(this, "_setItems", "_queryError");
                if (this.itemType == Constants.TYPE_CASE) {
                    // we need to fetch the summary amount of data. When we add the 
                    // items to the section table, it expects the modified, weight,
                    // and suspect to be available.
                    this._runningRequest = TestCaseClient.getTestCasesSummary(srh);
                } else if (this.itemType == Constants.TYPE_SCRIPT) {
                    this._runningRequest = TestScriptClient.getTestScriptsMinimal(srh);
                }
            }
		},
        
        setAllowsMultiple: function(allowsMultiple) {
            this.allowsMultiple = allowsMultiple;
        },
        
        setExclusionList: function(exclusionList) {
            this.exclusionMap= {};
            for (var i=0; i<exclusionList.length; i++) {
                var id= exclusionList[i];
                this.exclusionMap[id]= true;
            }
        },		
		
        _onCancel: function() {
            //this.reset();
        },
        
        _onOK: function() {
			if (dojo.isFunction(this.onOk)) {
				this.onOk(this._list.getSelectedItems());
			}
            /*
            for (var i=0; i<rows.length; i++) {
                // last element is the stored artifact or snapshot info
                var row = rows[i]; 
                //var element = row[row.length-1];
                if (!this._enableSnapshots || !row.revision || row.revision == "") { //row.versionableItemId != null) {
                    // an artifact
                    this._retArr.push(row);
                } else {
                    // a snapshot info
                    this._snapshotsToLoad.push(row);
                }
            }
            this._getSnapshotArtifacts();
            */
        },		
		
		_onTableRowsChecked: function() {
			//if any rows in the table are selected, make sure the Ok button is enabled
			var selectedRows = this._list.getSelectedItems();
            this._dialog.getOkButton().disabled = !(selectedRows && selectedRows.length > 0);
		},
		
		_setItems: function(items) {
			//filter the items 
			items = this._filterExcluded(items);
			
            //reset the content of the list with the non excluded items 
            this._list.refresh(items);          
            dojox.data.dom.removeChildren(this._itemListArea);
            this._itemListArea.appendChild(this._list.domNode);
            this._clearStatusMessage();
            this._dialog.getOkButton().disabled = true;
            this._dialog.updatePosition();			
		},
		
		_filterExcluded: function(items) {
            items = items || [];
			var filtered = [];
            for (var i = 0; i < items.length; i++) {
                // don't add an entry for the item if it is explicitly excluded
                if (!this.exclusionMap[items[i].itemId] && !this.exclusionMap[items[i].versionableItemId]) {
                    filtered.push(items[i]);
                }
            }
			return filtered;
		},
		
		/*
		_loadArtifacts: function(items) {		
            if (this._runningRequest) {
				this._runningRequest = null;
			}		      	
			items = items ? items : [];
			var artifacts = [];
			for (var i = 0; i < items.length; i++) {
				var artifact = items[i]; //{name: items[i].name, artifactId: items[i].itemId}
				artifact.artifactId = artifact.itemId;			
				
				// don't add an entry for the item if its already in the item list
				var itemId = this._enableSnapshots ? items[i].versionableItemId : items[i].itemId;
				if (!this.exclusionMap[items[i].itemId] && !this.exclusionMap[items[i].versionableItemId]) {
					artifacts.push(artifact);
				}	
			}
			
			//TODO - LOAD SNAPSHOTS ON DEMAND BY THE TABLE			
			if (this._enableSnapshots && this._showSnapshots) {
				this._snapshotArtifacts = artifacts;				
				
				//for defect 9494
				this.invokeTimes = Math.ceil(artifacts.length/MAX) ;
				for (var j = 0; j < this.invokeTimes; j++) {
					var params = {
						includeArchived: false,
						itemUUIDs: []
					};
					var startIndex = j * MAX;
					var tmpLen = (j==this.invokeTimes-1) ? (artifacts.length - startIndex): MAX;
					for (var i = 0; i < tmpLen; i++) {
						var id = artifacts[startIndex + i].itemId;
						params.itemUUIDs.push(id);
					}
					var srh = new ServiceResponseHandler(this, "_loadSnapshots", "_queryError");
					TestCaseClient.getSnapshotsMultipleItems(srh, params);
				}
				this._dialog.getOkButton().disabled = false;
			} else {			
				//reset the content of the list with the non excluded items 
				this._list.refresh(artifacts);			
				dojox.data.dom.removeChildren(this._itemListArea);
				this._itemListArea.appendChild(this._list.domNode);
				this._clearStatusMessage();
				this._dialog.getOkButton().disabled = false;
				this._dialog.updatePosition();
			}			
		},
		
        _loadSnapshots: function(snapshots) {
			this.returnedNum++;
			for (var j=0; j < (snapshots ? snapshots.length : 0); j++) {   
			    this._allSnapshots.push(snapshots[j]);                 
            }
			if (this.returnedNum == this.invokeTimes) {
				var allRows = [];
            	if (this._allSnapshots != null && this._allSnapshots.length > 0) {
                	for (var i = 0; i < this._snapshotArtifacts.length; i++) {
                    	var artifact = this._snapshotArtifacts[i];    
						artifact.snapshots = [];                
                    	for (var j=0; j < this._allSnapshots.length; j++) {                        
                        	var snapshot = this._allSnapshots[j];
                        	if (this._allSnapshots[j].artifactId == artifact.itemId) {
								artifact.snapshots.push(this._allSnapshots[j]);
                        	}
                    	}
						allRows.push(artifact);
                	}
                	this._list.refresh(allRows);
            	} else {
					this._list.refresh(this._snapshotArtifacts);
				}
			
				dojox.data.dom.removeChildren(this._itemListArea);
				this._itemListArea.appendChild(this._list.domNode);
				
				this._clearStatusMessage();
				this._dialog.getOkButton().disabled = false;
				this._dialog.updatePosition();
			}
        },
		*/
		
		_queryError: function(errorObj) {
			this._setStatusMessage(FETCH_ERR_MSG, true);
		},
		
        _clearStatusMessage: function() {
            this._msgArea.innerHTML= "";
        },
        
        _setStatusMessage: function(msg, isError) {
            this._msgArea.innerHTML= msg;
            this._msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
        }
		
        /*
        _toggleShowSnapshots: function(){
            if (this._enableSnapshots) {
                this._showSnapshots = !this._showSnapshots;
                // TODO - save current filter and selection from table to restore after refresh finishes.
                //  some enhancements to SortableFilterableTable are needed in order for this to be possible.
                this._initItemList();
                this.refresh();
            }
        },
		
		_getSnapshotArtifacts: function(artifacts) {
			if (artifacts != null && artifacts.length > 0) {
				var artifact = artifacts[0];
                var itemId = this._enableSnapshots ? artifact.versionableItemId : artifact.itemId;
				if (this.exclusionMap[itemId]!=true) {
					this._retArr.push(artifact);
				}
			}
			var nextToLoad = this._snapshotsToLoad.pop();
			if (nextToLoad != null) {
				var srh= new ServiceResponseHandler(this, "_getSnapshotArtifacts", "_queryError");
				if (this.itemType == Constants.TYPE_CASE) {
					//TestCaseClient.getTestCaseRevision(srh, nextToLoad.artifactId, nextToLoad.revision);
					TestCaseClient.getTestCaseRevision(srh, nextToLoad.itemId, nextToLoad.revision);
				}
			} else { // all snapshot artifacts are loaded	
				this.onOk(this._retArr, this.key);
			}
		}
		*/
		
	}
);

})();
