/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.TestPlanConfigSelector");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

(function() {
var Constants           = com.ibm.rqm.planning.web.client.internal.Constants;
var PlanClient          = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
var CategoryClient      = com.ibm.rqm.planning.web.client.internal.CategoryClient;
var SelectUtility       = dojo.getObject( "com.ibm.asq.common.web.ui.internal.util.SelectUtility" );
var CategorySelection   = com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection;
var EditorUtil          = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.TestPlanConfigSelector",
             [dijit._Widget, dijit._Templated], 
{
        
    templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/TestPlanConfigSelector.html"),                                  
                    

    height: "450px", 
    width: (dojo.isFF == 3) ? "430px" : "500px",
    
	labelowner: Constants.getMessage("labelowner"),
	
    postCreate: function(){
        this._categorySelect = new com.ibm.rqm.planning.web.ui.internal.view.construction.CategorySelection({
                 artifactType: Constants.MODEL_TYPE_PLAN            
        });             
        this._categoriesNode.appendChild(this._categorySelect.domNode);   
		
		EditorUtil.initContributors(this._owner, null);
    },
    
    initialize: function(args){
		this.selectedItems = args.selectedItems;
        this.getTPCategoryTypeSummaries(dojo.hitch(this, "_showAsDialog"));	 
    },
	
	getTPCategoryTypeSummaries: function(callBackFunc) {
        if (callBackFunc && typeof callBackFunc == "function") {
            
            var self = this;
            var serviceObj = {
                success: function(categoryTypeSummaries) {
                    var categories = dojo.isArray(categoryTypeSummaries) ? categoryTypeSummaries : [];
                    callBackFunc(categories);
                },
                failure: function(e) {
                    self._error(e);
                }
            };
            //get the category types (used instead of resolving them for each test case)
            var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");     
            CategoryClient.getCategoryTypeSummaries(responseHandler, Constants.MODEL_TYPE_PLAN);                
        }
    },
	
    _showAsDialog: function(categoryTypeSummaries)
    {
        this._categoryTypeSummaries = categoryTypeSummaries;
		
        if (!this._dialog) {
            var dialogArgs = {
                okCallback: dojo.hitch(this, "_okAction"),
                cancelCallback: dojo.hitch(this, "_cancelAction")
            };
            
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
			this._dialog.setTitle(Constants.getMessage("labelChangeTestPlanProperties"));
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
            this._dialog.getOkButton().disabled = false;                                
        }
        this._dialog.open(false);
    },  
	
	_okAction: function(){
		
		var args = {};
		
        args.uid = [];
        args.categories = [];
        for(var i=0; i<this.selectedItems.length; i++){
            args.uid.push(this.selectedItems[i].itemId);
            
			var categories = [];
			if (this.selectedItems[i].categories) {
				for (var k = 0; k < this.selectedItems[i].categories.length; k++) {
					categories.push(this.selectedItems[i].categories[k].itemId);
				}
			}
            categories = this._reCaculateCategories(categories);
            args.categories.push(categories.join());
        }
									
		var option = this._owner.options[this._owner.selectedIndex];
        if (option.text != Constants.getMessage("labelUnassigned")) {
			args.ownerId = option.value;
		}
		
        if (this.okCallback) {
            this.okCallback(args);
        }
    },
	
    _reCaculateCategories: function(categories) {
        var nowCategories = this._categorySelect.getSelectedItemIds();
        var oldCategories = categories;
        var newCategories = [];
        for(var i = 0; i < (this._categoryTypeSummaries ? this._categoryTypeSummaries.length:0); i++) {
            var type = this._categoryTypeSummaries[i];
            var newCategory = this._getCategoryByType(type, nowCategories);
            if(!newCategory)
                newCategory = this._getCategoryByType(type, oldCategories);
            if(newCategory)
                newCategories.push(newCategory);
        }
        return newCategories;
    },
  
    _getCategoryByType: function(type, categories) {
        var categoriesItemIds = type.categoryItemIds;
        for(var i = 0; i < (categoriesItemIds ? categoriesItemIds.length:0); i++) {
            for(var j = 0; j < (categories?categories.length:0); j++) {
                if(categoriesItemIds[i] == categories[j])
                    return categories[j];
            }
        }
        return null;
    },
		
	_cancelAction: function(){
        if (this.cancelCallback) {
            this.cancelCallback();
        }
    },
  
    _error: function(errorObj){
        console.error("Error in TestPlanConfigSelector " + errorObj.message);
    }
    
});
})();
    
