/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationListPanel");
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationListPanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.configuration.web.util.TestConfigurationUtil");
dojo.require("com.ibm.rqm.configuration.web.client.internal.configuration.WHITCHClient");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.table.ConfigurationTableView")

dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Configuration");
dojo.require("com.ibm.rational.test.lm.web.client.internal.config");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){ // module container
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var WHITCHClient = com.ibm.rqm.configuration.web.client.internal.configuration.WHITCHClient;
    var TestConfigurationUtil = com.ibm.rqm.configuration.web.util.TestConfigurationUtil;
    var ConfigurationTableView = com.ibm.rqm.configuration.web.ui.internal.view.table.ConfigurationTableView;
	var Constants  = com.ibm.rqm.planning.web.client.internal.Constants;
    
    dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.ConfigurationListPanel", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.configuration.web", "ui/internal/view/templates/configuration/TestConfigurationList.html"),
        
		labelStep1: Constants.getMessage("labelStep1"),
		labelStep2: Constants.getMessage("labelStep2"),
		labelStep3: Constants.getMessage("labelStep3"),
		labelNext: Constants.getMessage("labelNext"),
		labelPrevious: Constants.getMessage("labelPrevious"),
		labelLoading: ".  " + Constants.getMessage("labelLoading"),
		
        constructor: function(args){
            this.interactionLevel = (args.interactionLevel != null) ? args.interactionLevel : 1;
            this.coverageTarget = (args.coverageTarget != null) ? args.coverageTarget : 100;
            this.executionEnvTypes = (args.executionEnvTypes != null) ? args.executionEnvTypes : "";
            this.resourceInfomation = (args.resourceInfomation != null) ? args.resourceInfomation : "";
            this.weightsJsonString = (args.weightsJsonString != null) ? args.weightsJsonString : "";
            this.excludes = (args.excludes != null) ? args.excludes : "";
            this.includes = (args.includes != null) ? args.includes : "";
            this.types = (args.executionEnvTypes && dojo.isArray(args.executionEnvTypes)) ? args.executionEnvTypes : args.executionEnvTypes.split(",");
			this._newConfigsAdded = false;  //flag indicating if new configurations were added
			//the test plan UUID for applying added Configurations
			this.configurationAppliedTestPlanUUID = null;
        },
        
        postCreate: function(){
            this._initTitle();
			this._initTable();
            this._loadConfigurations();
        },
		
		//@Abstract
        _initTitle: function(){
            return;
        },
		
		_initTable: function(){
			dojox.data.dom.removeChildren(this._configurationList);
			this._displayLoadStatus(true);
			var args = {
				actions: this._getTableActions()
			};
            this._testConfigListWidget = new ConfigurationTableView(args);
            this._configurationList.appendChild(this._testConfigListWidget.domNode);
		},
		
		_getTableActions: function(){
			var actions = [];
			actions.push({
                label: Constants.getMessage("labelDeleteConfigurations"),
                actionHandler: dojo.hitch(this, "_handleDeleteConfigurationClicked"),
                iconClass: "remove-enabled-icon-image",
                disabledOnReadOnly: true
            });
			return actions;
		},
		
		_handleDeleteConfigurationClicked: function(){
			this._testConfigListWidget.removeSelectedConfigs();
		},
        
        _loadConfigurations: function(){
            var params = {
                interactionLevel: this.interactionLevel,
                coverageTarget: this.coverageTarget,
                executionEnvTypes: dojo.toJson(this.types),
                variablesJsonString: this.variablesJsonString,
                resourceInfomation: this.resourceInfomation,
                excludes: this.excludes,
                includes: this.includes,
                weightsJsonString: this.weightsJsonString
            };
            var responseHandler = new ServiceResponseHandler(this, "_addTestConfigToTable", "_error");
            WHITCHClient.postGenerateTestConfigurations(responseHandler, params);
            
        },
        
        _displayLoadStatus: function(status){
            this._nextButton.disabled = status;
            this._previousButton.disabled = status;
			this._progressIndicator.style.display = status ? "inline" : "none";
        },
        
        _addTestConfigToTable: function(configGroup){
            this._testConfigListWidget.appendConfigs(this._demarshal(configGroup));
			this._displayLoadStatus(false);
        },
        
        _demarshal: function(configGroup){
            var configs = [];
            for (var i = 0; i < ((configGroup.configurations) ? configGroup.configurations.length : 0); i++) {
                var attributeValues = new Array();
                var config = [];
                var executionEnvironmentEntries = configGroup.configurations[i].executionEnvironment.executionEnvironmentEntries;
                
                for (var j = 0; j < executionEnvironmentEntries.length; j++) {
                    var key = executionEnvironmentEntries[j].key;
                    var value = executionEnvironmentEntries[j].value;
                    attributeValues.push(value);
                    config.push({
                        key: key,
                        value: value
                    });
                }
                configs.push(config);
            }
            return configs;
        },
        
        _disableNext: function(){
            this._previousButton.disabled = true;
        },
        
        onPrevious: function(){
            this.controller.doPrevious();
        },
        
        onNext: function(){
			
        },
        
		//@Abstract
        doNext: function(){
            return;
        },
        
        _error: function(errorObj){
            confirm(errorObj.message);
        }
    });
    
})();
