/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.view.layout.PageLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.SideActionPanel");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.RelatedInfo");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");

(function() { // module container 

var SideActionPanel = com.ibm.asq.common.web.ui.internal.view.layout.SideActionPanel;
var RelatedInfo     = com.ibm.asq.common.web.ui.internal.view.common.RelatedInfo;
var Constants = com.ibm.asq.common.web.client.internal.Constants;

dojo.declare("com.ibm.asq.common.web.ui.internal.view.layout.PageLayout",
[dijit._Widget, dijit._Templated],
{
    defaultView: null,  // the default view to display in the content area
    hideSearchSection: false, // switch to hide or show the search action section at the top

    // Summary:
    //     Set the view to display in the content area. The specified view needs to be
    //     a widget. This method automatically caches the view into the page's cache, if
    //     the view has not been displayed, or the the replaceView flag is set to true,
    //     the cached view is replaced with the specified view.
    //
    setPageView: function(viewId, view, replaceView /* option to replace */) {
        if (this.getCachedView(viewId)) {
            // the view is already cached
            if (replaceView) {
                // update existing is set to replace
                this.cacheView(viewId, view);
            }
        } else {
            // cache view based on id
            this.cacheView(viewId, view);
        }

        if (this._currentView) {
            // remove the current displayed view if there is one
            this._viewContainer.removeChild(this._currentView.domNode);
        }
        // update the current display view
        this._currentView = this.getCachedView(viewId);
        this._viewContainer.appendChild(this._currentView.domNode);
    },

    // Summary:
    //     Set the specified button to be the primary action button in the side action
    //     panel.  The button should be an instance of the PrimaryActionButton.
    //
    setPrimaryAction: function(actionButton) {
        this._leftNavigator.getPrimaryActionSection().appendChild(actionButton);
    },
    
    // Summary:
    //     Set the specified button to be the secondary dropdown actions button
    //     in the side action panel.  The button should be an instance of the
    //     SecondaryActionsDropdownButton.
    //
    setSecondaryActions: function(actionsDropdownButton) {
        this._leftNavigator.getSecondaryActionsSection().appendChild(actionsDropdownButton);
    },
    
    // Summary:
    //     Return the side action panel.
    //
    getSideActionPanel: function() {
        return this._leftNavigator;
    },
    
    // Summary:
    //     Returns the view widget if it's cached by the specified id.
    // 
    getCachedView: function(id)
    {
        if(this._viewCache[id])
            return this._viewCache[id];
        return null;
    },
    
    // Summary:
    //     Set the specified view widget into the page's cache. Consumer of the page
    //     widget shouldn't need to explicitly cache a view in the page. When displaying
    //     a view in the page's content area using the page's setPageView() method, the
    //     specified view is automatically cached.
    //
    cacheView: function(id, viewWidget)
    {
        this._viewCache[id] = viewWidget;
    },

    // Summary:
    //     Removes a cached view from the page. A view may become "stale" or invalid. Removing
    ///    it from the page cache allows a new view to be generated. 
    // 
    removeCachedView: function(id)
    {
        if(this.getCachedView(id)) {
            this._viewCache[id] = null;
        }
    },
    
    templateString: '<div id="${id}" dojoAttachPoint="domNode"></div>',

    _leftNavigator: null,   // left side action panel
    _viewContainer: null,   // right side content container
    _viewCache: {},         // array of views based on its id
    _currentView: null,     // the view that is currently displayed in the right side content container
    _relatedInfoPanel: null,
        
    postCreate: function() {
        com.ibm.asq.common.web.ui.internal.view.layout.PageLayout.superclass.postCreate.apply(this, arguments);
        
        this._viewCache = {};
        var arg = this.hideSearchSection ? {hideSearchSection: true} : null;
        this._leftNavigator = new SideActionPanel(arg);
        this.domNode.appendChild(this._leftNavigator.domNode);
        
        this._viewContainer = document.createElement("div");
        this._leftNavigator.setMainContent(this._viewContainer);
        
        this._relatedInfoPanel = new RelatedInfo();
        this._viewContainer.appendChild(this._relatedInfoPanel.domNode);

        if (this.defaultView) {
            this.setPageView("default", this.defaultView);
        }
    },
    
    relatedInfoPanel: function() {
        return this._relatedInfoPanel;
    },
    
    addSampleRelatedInfo: function() {
        this._relatedInfoPanel.addLink(Constants.getMessage("labelRelatedSites"), Constants.getMessage("labelIBM"), "http://www.ibm.com");
        this._relatedInfoPanel.addLink(Constants.getMessage("labelInterestingSites"), Constants.getMessage("labelGoogle"), "http://www.google.com", "save-icon-image");
    }
});

})();