/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.planning.PlanResourceSection");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanResourceSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox")

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.ResourceTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function(){
	var ServiceResponseHandler 			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var Constants 						= com.ibm.rqm.planning.web.client.internal.Constants;
    var AbstractArtifactEditorSection 	= com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
	var ResourceTableView 				= com.ibm.rqm.planning.web.ui.internal.view.table.ResourceTableView;
	var EditorUtil 						= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var PreferenceClient				= com.ibm.rqm.planning.web.client.internal.PreferenceClient;
	var getCurrentProjectArea			= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var SelectUtility                   = dojo.getObject("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
    var CSHelp                          = com.ibm.asq.common.web.ui.internal.util.CSHelp;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanResourceSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div>'+
							'<div dojoAttachPoint="_resourceSection"></div>'+
						'</div>',
		
        constructor: function(){
			this._items = [];
            this._itemType = "Resource";
            this._itemsAdded = [];
            this._itemsRemoved = [];
        },
        
        postCreate: function(args){
            this.inherited('postCreate', arguments);
            //set up workitem support for this section
            this.addWorkItemSupport();
            this._d
        },
        
		//@Overriden from AbstractArtifactEditorSection - invoked on first expand
        initialize: function(isEditable){
			var content = [];
			if (this.getArtifact()) {
				content = this.getSection().content ? this.getSection().content : ""
			}
			this.setItems(content != "" ? dojo.fromJson(content):[]);
			this._initResources();		
			this.initialized = true;
        },
		
        //@Overriden from AbstractArtifactEditorSection
        getActions: function(){
            return null;
        },

		//@Overridden from AbstractArtifactEditorSection
        getSectionInfo: function() {      
            this.inherited("getSectionInfo", arguments);
            if (!this.initialized) {
				return null;
			} else {
				this.getSection().attributes = [];
				this.getSection().attributes.push({
					type: "richtext",
					name: "resources",
					value: dojo.toJson(this._items)
				});
				return this.getSection();
			}	
        },
		
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.initialize();
            this.addWorkItemSupport();
        },		
        
		_initResources: function() {           
            if(!this._resourceTableViewer) {
                dojox.data.dom.removeChildren(this._resourceSection);                
                this._resourceTableViewer = new ResourceTableView({
                    editable: this.isEditable(),
					printable: this.isPrintView(),
                    enableRowSelection: true,
					parentEditor: this.getParentEditor(),
                    resources: this.getItems(),
                    actions: this.getTableActions()
                });
                this._resourceSection.appendChild(this._resourceTableViewer.domNode);
            }
            else {
                this._resourceTableViewer.refresh(this.getItems());
            }
        },
        
        getItems: function() {
            return this._items;            
        },
        
        setItems: function(items) {
            this._items = items;            
        },
		
		//actions to be displayed in the table header
        getTableActions: function(){
            var actions = [];
			
            actions.push({
                label: Constants.getMessage("labelAddResourceLocation"),
                actionHandler: dojo.hitch(this, "_handleAddClick"),
                iconClass: "add-icon-image",
                disabledOnReadOnly: true                 
            });
            
            actions.push({
                label: Constants.getMessage("labelRemoveResourceLocation"),
                actionHandler: dojo.hitch(this, "_handleRemoveClick"),
                iconClass: "remove-enabled-icon-image",
                disabled: true                 
            });
                             
            return actions;
        },
		
        _handleAddClick: function() {
			var args = {
				onOk: dojo.hitch(this, "_addItems"),
				onCancel: dojo.hitch(this, function(){})
			}
			this.resourcesDialog = new com.ibm.rqm.planning.web.ui.internal.view.planning.PlanResourceSection.Dialog(args);	
        },
		
        _handleRemoveClick: function() {
			var items = this._resourceTableViewer.getSelectedItems();            
			if (items && items.length != 0) {
				this._removeItems(items); 
				this.setDirty();
			}  
        },      

		_removeItems: function(items) {            
            var displayItemType = Constants.getExtensionDisplayMessage(this._itemType);
            if ( !items || items.length == 0) {
				var msg = Constants.getMessage("msgDeleteTypeRequired", displayItemType);
				this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, msg);
                return;
            }
            var confirmMsg = "";
            if (items.length > 1) {
				confirmMsg = Constants.getMessage("msgRemoveItem2", items.length, displayItemType);
            } else {
                confirmMsg = Constants.getMessage("msgRemoveItem", displayItemType, Constants.getExtensionDisplayMessage(items[0].type));
            }
			
			var self = this;
			var	args = {
				message: confirmMsg,
				yesCallback: function(){
					for (var i = 0; i < (items ? items.length : 0); i++) {
                		self._removeItem(items[i]);
            		}
            		self._initResources();
					return true;          
				},
				noCallback: function() {
					return;
				}
			}
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        	dialog.setSecondaryTitle(Constants.getMessage("labelConfirmation"));
			dialog.open();
        },
		
        _removeItem: function(item) {       
            var updatedItems = [];        
            var items = this.getItems();  
            for (var i = 0; i < items.length; i++) {
				if(items[i].type == item.type){
					this._items.splice(i, 1);
					break;
				}	
            }
        },
		
		_addItems: function(scriptType, sharedLocation, resolvedPath) {
            //this._replaceAppendResources(this._scriptTypes.value, this._sharedLocations.value, this._resolvedPathValue.innerHTML);
			this._replaceAppendResources(scriptType, sharedLocation, resolvedPath);
			this._resourceTableViewer.refresh(this._items);
            this.setDirty(true);
		},
		
        _replaceAppendResources: function(type, location, resPath){
            var found = false;
            for(var i=0;i<this._items.length;i++){
                if(this._items[i].type == type){
                    found = true;
                    this._items[i] = {type: type, location: location, resolvedPath: resPath};
                    break;
                }   
            }
            if(!found){
                this._items.push({type: type, location: location, resolvedPath: resPath});
            }
        },  		

        _error: function(e){
            console.error("An error occurred: [" + e.message + "] @ PlanResourceSection#_error");
        }
    });
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.planning.PlanResourceSection.Dialog", [dijit._Widget, dijit._Templated], {

        templateString:     '<div dojoAttachPoint="_container">' +
	                        '    <div dojoAttachPoint="_message"></div>' +
	                        '    <table style="width=450px;">' +
							'        <tr style="width: 100%;">' +
							'            <td style="width: 50%"><label dojoAttachPoint="_selectTypeLabel"></label></td>' +
							'            <td style="width: 50%"><select style="width: 100%" dojoAttachPoint="_scriptTypes"></select><td>'+
							'        </tr>'+
	                        '        <tr style="width: 100%">' +
							'            <td style="width: 50%"><label dojoAttachPoint="_sharedLocationsLable"></label></td>' +
							'            <td style="width: 50%"><select style="width: 100%" dojoAttachPoint="_sharedLocations" dojoAttachEvent="onchange: _updateResolvedPath"></select><td>' +
							'        </tr>'+
	                        '        <tr style="width: 100%">' +
							'            <td style="width: 50%"><label dojoAttachPoint="_usePlanLable"></label></td>' +
							'            <td style="width: 50%">&nbsp;<input dojoType=\"dijit.form.CheckBox\" dojoAttachPoint="_usePlanValue"></input><td>' +
							'        </tr>'+
	                        '        <tr style="width: 100%">' +
							'            <td style="width: 50%"><label dojoAttachPoint="_usePhaseLable"></label></td>' +
							'            <td style="width: 50%">&nbsp;<input dojoType=\"dijit.form.CheckBox\" dojoAttachPoint="_usePhaseValue"></input><td>' +
							'        </tr>'+
	                        '    </table>'+
                            '    <div><label dojoAttachPoint="_resolvedPathLable"></label>&nbsp;&nbsp;<label style="width: 100%" dojoAttachPoint="_resolvedPathValue"></label></div>'+
	                        '</div>',

        width: "500px",
		title: Constants.getMessage("labelAddResourceLocation"),
		labelAddResource: Constants.getMessage("labelAddResourceLocation"),
		widgetsInTemplate: true,
		
        constructor: function(args){
            this.onOk = null;
			this.onCancel = null;
        },
        
        postCreate: function() {
            this._selectTypeLabel.innerHTML = Constants.getMessage("labelSelectScriptType");
            this._sharedLocationsLable.innerHTML = Constants.getMessage("labelSelectSharedLocation");
            this._resolvedPathLable.innerHTML = Constants.getMessage("labelResolevedPath");
            this._usePlanLable.innerHTML = Constants.getMessage("labelIncludePlan");
            this._usePhaseLable.innerHTML = Constants.getMessage("labelIncludeIteration");
			
			dojo.connect(this._usePlanValue, "onChange", this, "_planCheckBoxClicked");
			dojo.connect(this._usePhaseValue, "onChange", this, "_updateResolvedPath");
			
            EditorUtil.initScriptTypes(this._scriptTypes)
            this._getSharedLocations();          
			          
            var dialogArgs = {
				primaryTitle: this.title,
                okCallback: dojo.hitch(this, "_onOkClicked"),
                cancelCallback: dojo.hitch(this, "_onCancelClicked"),
				_truncate: function() {}  //disable the Dialog's '_truncate'
            };
						          
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
			var cshelp = new CSHelp({contextID: "rqm_add_resources.html"});
			cshelp.setButtonLabel(Constants.getMessage("labelAddResourcesCSHelp"));
			this._dialog.setSecondaryTitleNode(cshelp.domNode);
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
			this._dialog.getOkButton().disabled = false;
            this._dialog.open(false);
			
        },
		
        _getSharedLocations: function(){
            var responseHandler = new ServiceResponseHandler(this, "_fillSharedLocations", "_error");   
            var option = new com.ibm.asq.common.web.client.internal.option();     
            option.getProjectPreferences(responseHandler, getCurrentProjectArea().itemId, Constants.SHARED_RESOURCES_LOCATION_CATEGORY);
        },
		
        _fillSharedLocations: function(prefEntry) {         
            var locations = prefEntry ? dojo.fromJson(prefEntry.value) : [];
            this.sharedLocations = locations;
            for ( var i = 0; i < locations.length; i++ ) {
                var option = new Option(locations[i], locations[i]);
                SelectUtility.add(this._sharedLocations, option);
            }
            if (this.sharedLocations.length == 0) {
                this._message.innerHTML = Constants.getMessage("labelNoSharedLocationFound");
            } else {
                this._message.innerHTML = "";
            }			
            this._updateResolvedPath();
        },	

        _planCheckBoxClicked: function(){
            this._updateResolvedPath();
        },

        _updateResolvedPath: function(){
             var resolvedPath = this._sharedLocations.value;
            if (this._usePlanValue.checked) {
				resolvedPath += ("\\" + Constants.INCLUDE_PLAN)
			}
            if (this._usePhaseValue.checked) {
				resolvedPath += ("\\" + Constants.INCLUDE_ITERATION)
			}
            this._resolvedPathValue.innerHTML = resolvedPath;
        },		
		
        _onOkClicked: function(){
            this.onOk(this._scriptTypes.value, this._sharedLocations.value, this._resolvedPathValue.innerHTML);
        },
        
        _onCancelClicked: function(){
        },		
        
        _error: function(e){
            console.error(this.declaredClass + ": error: " + (e ? e.message : ""));          
        }
    
    });	
})();
