/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.appscan.web.ui.internal.view.planning.PlanSecurity");
dojo.provide("com.ibm.rqm.appscan.web.ui.internal.view.planning.PlanSecurity");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rqm.appscan.web.client.internal.Constants");
dojo.require("com.ibm.rqm.appscan.web.client.internal.ScanClient");
dojo.require("com.ibm.rqm.appscan.web.client.internal.InfoClient");
dojo.require("com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");

(function(){
   
   var ScanClient						= com.ibm.rqm.appscan.web.client.internal.ScanClient;
   var InfoClient						= com.ibm.rqm.appscan.web.client.internal.InfoClient;
   var AbstractArtifactEditorSection    = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
   var Constants 						= com.ibm.rqm.appscan.web.client.internal.Constants;
   var ServiceResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
   
    /**
     *  Displays the list of available test policies
     */
    dojo.declare("com.ibm.rqm.appscan.web.ui.internal.view.planning.PlanSecurity", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.appscan.web", "ui/internal/view/templates/planning/PlanSecurity.html"),
        
        constructor: function(){
            this._workItem = null;
        },
        
        postCreate: function(args){
			this.inherited('postCreate', arguments);
			this._initLabels();
        },
		
		/**
		 * Initialize the static labels on the widget
		 */
		_initLabels: function() {
			this._availableTestPoliciesLabel.innerHTML = Constants.getMessage("available_test_policies");
			this._loadingLabel.innerHTML = Constants.getMessage("loading");
		},
         
		initialize: function(isEditable){
            this._plan = this.getArtifact();
            this._checkVersion();
        },
		
		//@Overriden from AbstractArtifactEditorSection
        getSummaryContent: function(isEditable) {
			return Constants.getMessage("plan_security_summary");
        },
        
        _eval: function(json){
			return eval( "(" + json + ")" );
		},
        
        _initialize: function(){
			this._initTestPolicyList();
        },
		
		_checkVersion: function(){
			var responseHandler = new ServiceResponseHandler(this, "_checkVersionSuccess", "_checkVersionError");
			InfoClient.getVersion(responseHandler);
		},
        
        _initTestPolicyList: function(){
            var responseHandler = new ServiceResponseHandler(this, "_loadTestPoliciesSuccess", "_error");
			ScanClient.getTestPolicies(responseHandler);
        },
		
		_checkVersionSuccess: function(result) {
			
			try{
				// Check the version of ASE.  If its valid, show the available test policies
				var version = this._eval(result);
				if (version.IsValidDatabaseVersion){
					this._initialize();
				}
				else {
					// ASE version not valid, show a message to the user
					dojox.data.dom.removeChildren(this._container);
					this._container.innerHTML = Constants.getMessage("ase_wrong_version", version.StoredProcedureVersion, version.DatabaseVersion, version.CheckDatabaseVersion);
				}
			} catch(e) {
				dojox.data.dom.removeChildren(this._container);
				this._container.innerHTML = result;
			}
		},
        
        _loadTestPoliciesSuccess: function(result){
        	var testPolicies = this._eval(result).results;
        	
        	// Build Table
        	dojox.data.dom.removeChildren(this._testPolicyTable);
            
			var body = document.createElement("tbody");
        	var head = document.createElement("thead");
        		
        	var tr = document.createElement("tr");
			var td = document.createElement("th");		
			var label = document.createTextNode(Constants.getMessage("name"));
			td.appendChild(label);
			tr.appendChild(td);
			td = document.createElement("th");		
			label = document.createTextNode(Constants.getMessage("description"));
			td.appendChild(label);
			tr.appendChild(td);
			head.appendChild(tr);
        	this._testPolicyTable.appendChild(head);
        
        	for (var i = 0; i < testPolicies.length; i++)
			{
				tr = document.createElement("tr");
				
				td = document.createElement("td");		
				
				var path = "Admin/TestPoliciesEditor.aspx?testPolicyId=" + testPolicies[i].TestPolicyId;
						 	
				var link = new com.ibm.rqm.appscan.web.ui.internal.view.common.ASELink();
				var name = this._lookupName(testPolicies[i].Name);
				
				link.initialize({ path : path, 
								  title :  Constants.getMessage("edit_item", name),
								  text: name });
					
				td.appendChild(link.domNode);
				tr.appendChild(td);
				
				td = document.createElement("td");		
			    label = document.createTextNode(this._lookupDescription(testPolicies[i].Description));
				td.appendChild(label);
				tr.appendChild(td);
				
				body.appendChild(tr);
			}
			this._testPolicyTable.appendChild(body);
        },
        
        _lookupName: function(name){
        	
        	// Perform lookups on wf defined objects
        	if (name.substring(3,0) == 'wf-'){
				var lookup = Constants.getMessage(name);
        		if (lookup != null)
        			return lookup;
        	}
        	return name;
        },
        
        _lookupDescription: function(desc){
        	// Perform lookups on wf defined objects
        	if (desc.substring(3,0) == 'IDS'){
				var lookup = Constants.getMessage(desc);
        		if (lookup != null)
        			return lookup;
        	}
        	return desc;
        },
        
		_checkVersionError: function(e){
            dojox.data.dom.removeChildren(this._container);
			this._container.innerHTML = Constants.getMessage("error_ase_not_configured");
        },
		
        _error: function(e){
            console.error("An error occurred: [" + e.message + "] @ PlanSecurity#_error");
        }
    });
})();
