/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.planning.web.ui.internal.view.planning.TestPlanEditor");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.planning.TestPlanEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

// Require the common widgets needed
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.Section");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.CollapseExpandArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");

//these aren't required since they're loaded dynamically, they're here for debug purposes
//xojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.TestCasesSection");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.AbstractArtifactEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotEnabledEditor");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.RelatedInfo");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.team.workitem.web.client.util"); 

(function(){
    var Constants                = com.ibm.rqm.planning.web.client.internal.Constants;
    var ServiceResponseHandler   = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var PlanningPageClient       = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
    var SaveParticipant          = com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
	var Utility                  = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var SizingClient             = com.ibm.rqm.planning.web.client.internal.planning.SizingClient;
    var AbstractArtifactEditor   = com.ibm.rqm.planning.web.ui.internal.view.construction.AbstractArtifactEditor;
    var SnapshotEnabledEditor    = com.ibm.rqm.planning.web.ui.internal.view.construction.SnapshotEnabledEditor;
    var ImageCache               = com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;    	
    var GlobalEvents             = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var ViewHeaderLayout 		 = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	var PrintViewBaseURI         = "action=com.ibm.rqm.planning.printview.testPlan&id=";
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.planning.TestPlanEditor", 
    			[dijit._Widget, 
    			 dijit._Templated, 
    			 SnapshotEnabledEditor, 
    			 AbstractArtifactEditor,
    			 com.ibm.asq.common.web.ui.internal.view.common.RelatedInfoMixin], {
		        
		templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/planning/TestPlanEditor.html"),
           
        // object initializer
        constructor: function() {},
        
		
		
        // populate the template with data, connect events, and show initial sub-tab
        postCreate: function() {
            this.inherited('postCreate', arguments);
            this.setArtifactType(Constants.TYPE_PLAN);
			this.setSnapshotClient(PlanningPageClient);			
			
			//internalize params
			if (this.model) {
				this.setArtifactModel(this.model);
				this.model = null;
			}
            if (this.printable) {
                this.setPrintView(true);
				this.setReadOnly(true);
                this.printable = null;
            }			
						
			//set new/copy/snapshot states		
            this.setCopy(this.copy === true);
			this.setNew(this.kwArgs == null || this.isCopy());			
            this.setPrintViewURI(PrintViewBaseURI + (this.kwArgs && this.kwArgs.id ? this.kwArgs.id : ""));
            this.setIsSnapshot(!this.isCopy() && this.kwArgs != null && this.kwArgs.revision != null);		
			
            //create layout
            this.createLayout();			
			this.setLoading(true); 
			
			if (this.kwArgs) {				      				         
                var responseHandler = new ServiceResponseHandler(this, "_loadPlan", "_loadError");
                PlanningPageClient.getTestPlansWithWorkItems(responseHandler, this.kwArgs);
			
			} else {
				this.initialize({
					artifact: this.plan ? this.plan : null
				});
			}
			this.staticRelatedInfo = new com.ibm.asq.common.web.ui.internal.view.common.RelatedInfoPanel();
			this.staticRelatedInfo.getInfoGroup(Constants.getMessage("labelTestPlanWorkitems"), true);
        },
		
		_loadPlan: function(plans) {			
            if (!dojo.isArray(plans) || plans.length != 1) {
                this._loadError(new Error(Constants.getMessage("errTestPlanNotFound")));
                return;
            }			
			this.initialize({
				artifact: plans[0]
			});
		},
        
		//initialize's param should be consistent with others
        initialize: function(args) {	
			var plan = 	(args == null)? null : args.artifact; 
            // clear the section removed list.
            this._sectionsRemoved = [];            
                 	 
            // specify the elements in the overview tab
            if (plan != null) {				
                if (this.isCopy()) {
                    // clear out the snapshot info
                    plan.snapshotInfo = null;					
					this._setupCopy(plan);				
                }					
				this.setArtifact(plan);
	        }
			
            // if this is a "new" test plan, don't set dirty from the start         
			this.setDirty(this.isCopy());
            
            //update header with title, etc               
			this.updateHeader();          
			
            //load sections      
			this.loadSections(this._sectionsNode, this.isPrintView() ? null : this._tableOfContentsNode);			

			dojox.data.dom.removeChildren(this.getHeader().getInfoAttributesNode());
			this.setOriginator(plan == null ? null : plan.originator.name);         
            this.setState(plan == null ? Constants.STATETRANSITION_STATE_DRAFT : plan.state); 
            if (this.isNew() && !this.isCopy()) {
                this.showTemplateSelect();              
                this._defaultName = this.getHeader().getTitle();
                this.setLastSaved(Constants.STATUS_NEW);
            }
            this._addRelatedInfo();
            this.setCSHelpId("rqm_test_plan_overview.html");
			this.setSCHelpButtonLabel(Constants.getMessage("labelTestPlanEditorCSHelp"));
			
			//complete loading
			this.setLoadingArtifactComplete();

            if (this.isNew() || this.isCopy()) {
                if (this.getHeader().viewEditableTitle) {
                    setTimeout(dojo.hitch(this.getHeader().viewEditableTitle, "focusWithPrompt"), 1);
                }
            }
        },
       
        createLayout: function() {
			if (this.isPrintView()) {
				this.createPrintViewLayout();
				return;
			}
			
            this._snapshotsNode = document.createElement("div");
            this._sectionsNode = document.createElement("div");
                       
            this.createHeader("testplan-icon-48", this._sectionsNode, this._snapshotsNode);
            dojox.data.dom.removeChildren(this._contentNode);
            this._contentNode.appendChild(this.getHeader().domNode);
			
			this.loadTableOfContents(this._tableOfContentsNode);
            
            //create snapshot pane
            this._snapshotsNode.appendChild(this.getSnapshotPane().domNode);
        },
		
		createPrintViewLayout: function() {
            //remove the table of contents node  
			this._layoutRow.deleteCell(0);
			this._sectionsNode = document.createElement("div");
                       
            this.createHeader("testplan-icon-48", this._sectionsNode, null);
            dojox.data.dom.removeChildren(this._contentNode);
            this._contentNode.appendChild(this.getHeader().domNode);
		},

        getStaticRelatedInfo: function() {
			if(this.staticRelatedInfo){
				return this.staticRelatedInfo;
			}else{
				return null;
			}
        },
            
        getStaticRelatedInfoId: function() {
			if(this.getArtifact() && this.getArtifact().id){
				return this.declaredClass + this.getArtifact().id;
			}else{
				var date = new Date();
				return this.declaredClass + date.toString();//"com.ibm.rqm.planning.web.ui.internal.view.planning.TestPlanEditor" + date string. This is the relatedInfomationId of new created Test Plan.
			}
        },
        
        _addRelatedInfo: function() {
            var plan = this.getArtifact();
            if (plan && plan.workItems != null && this.staticRelatedInfo) {
				var group = this.staticRelatedInfo.getInfoGroup(Constants.getMessage("labelTestPlanWorkitems"));
				dojox.data.dom.removeChildren(group._actionListNode);
                for( var iWork = 0; iWork < plan.workItems.length; iWork++ ) {
                    var workItem = plan.workItems[iWork];
					var href = com.ibm.team.workitem.web.client.util.getViewWorkItemURI(workItem.id);
//                    var http = window.location.protocol + 
//                               "//" + 
//                               window.location.host + 
//                               "/jazz/web/projects/" + 
//                               com.ibm.team.process.web.projectArea.name +
//                               window.location.search +
//                               href;
					var http = Utility.getViewWorkItemURI(workItem.id);

                    var heading = workItem.id + " " + workItem.summary;
                    var trimmedHeading = false;
                    if( heading.length > 32 ) {
                        heading = heading.substring( 0, 28 ) + "...";
                        trimmedHeading = true;
                    }
                                        
                    var wiLink = this.staticRelatedInfo.addLink(Constants.getMessage("labelTestPlanWorkitems"), heading, http );
                    if (trimmedHeading && wiLink.domNode) {
                        wiLink.domNode.title = workItem.summary;
                    }
                }               
            }            
        },
		
		_setupCopy: function(plan) {
            // create a save participant for each test case
            var testcases = plan ? plan.testCases : [];
            if (this.isCopy() && testcases != null && testcases.length > 0) {
                for (var i = 0; i < testcases.length; i++) {
                    var saveParticipant = new SaveParticipant();
                    saveParticipant.setDirty(false);
                    saveParticipant.setVersionableItemId(testcases[i].versionableItemId);
                    saveParticipant.setItemId(testcases[i].versionableItemId);
                    saveParticipant.setItemType(Constants.MODEL_TYPE_CASE);
                    this.getSaveContext().addParticipant(saveParticipant);
                    this.getSaveContext().createReference(this, "testCasesAdded", saveParticipant);

                }
            }			
		},
              
        setSaveState: function(){
            this._saveButton.disabled = false;
        },
		
        //@Override this method must be implemented to support locking
        lock: function(serviceResponseHandler) {
            var args = {
                lock: !this.getArtifact().locked,
                versionableItemId: this.getArtifact().versionableItemId
            }
            PlanningPageClient.postLockTestPlan(serviceResponseHandler, args);
        },
        
        _saveError: function(status){
            status.caughtAt = "TestPlanEditor#_saveError";
            this.setLastSaved(Constants.STATUS_FAILURE, status);
        },
        
        // @override for SaveParticipant
        getItemType: function(){
            return Constants.MODEL_TYPE_PLAN;
        },
        
        // @override for SaveParticipant
        validate: function(){
            var name = this.getHeader().getTitle();
            if (dojo.trim(name).length == 0 || (this._defaultName && name == this._defaultName)) {
				this.setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR,
					Constants.getMessage("msgRequiredField", Constants.getMessage("labelName"), Constants.getMessage("labelTestPlan")));
                return false;
            }            
            return true;
        },
        
        // @override for SaveParticipant
        getAttributes: function(){
			var args = this.inherited('getAttributes', arguments);         
			
			//new plan
            args.newTestPlan = this.isNew();
			args.name = this.getHeader().getTitle();
            args.state = this.getStateValue();
            //args.name = this._planName.value;
            //args.state = this._planStatus.value;			
            
            if (!this.isNew()) {
                args.id = this.getArtifact().id;				
            } else {
				args.templateItemId = this.getTemplate().itemId;
				args.templateStateId = this.getTemplate().stateId;
			}
           
            // snapshot properties
            if (this._createSnapshot == true) {
                args.createSnapshot = true;
                args.snapshotName = this._snapshotName;
                args.snapshotDescription = this._snapshotDescription;
            }            
            return args;
        },
		
		
        
		// @override for SaveParticipant
		saveFinished: function(savedItem) {

			//State Management 
			//Clear the state changed flag on each save.
			this.setStateChanged(false);            
            this.setPrintViewURI(PrintViewBaseURI + savedItem.id);

            this.inherited('saveFinished', arguments);
           
            //refresh snapshot info
            if (this._createSnapshot) {
                // refresh the snapshot list
                this.getSnapshotPane().refreshSnapshots();
				
			    /**
			     *  Temporary UI hack to propagate snapshots since server
			     *  is currently lacking this capability.
			     *  
			     *  THIS METHOD WILL BE REMOVED.
			     *  DO NOT USE THIS METHOD OR COPY THIS CODE ELSEWHERE
			     *  
			     *  TODO:  remove this workaround when snapshot propagation
			     *  is properly implemented
			     *  
			     *  @deprecated     DO NOT USE THIS METHOD.
			 	 */
				var internalHandler = {
					doCallback: function(returnVal, args) {},
			  		doErrorCallback: function(error){
						alert(error.message);
					}
				};
				
				var snapshotPropagationArgs = {
					testPlanCurrentVersionableUUID: savedItem.versionableItemId,
					testPlanSnapshotVersionableUUID: savedItem.createdSnapshotUUID
				};
				PlanningPageClient.HACK_DO_NOT_USE__postFollowupSnapshot(internalHandler, snapshotPropagationArgs);
				
                // unset the snapshot properties on the editor so next save doesn't create a snapshot
                this._createSnapshot = false;
                this._snapshotName = null;
                this._snapshotDescription = null;
            }
			this._addRelatedInfo();
        },
        
        getOwnerItemId: function(){
            return this.getArtifact() != null ? this.getArtifact().owner.itemId : null;
        },
        
        _error: function(e){
            console.error("An error occurred: [" + e.message + "] @ TestPlanEditor#_error");
        },

        // handler on artifacts changed
        onArtifactChanged: function(changeType, artifactType, artifactArray) {
            // handle test cases changed
            if (artifactType == Constants.TYPE_CASE) {
                // check if changed test cases are associated with the test plan
                var testCaseSection = this.getSection("com.ibm.rqm.planning.editor.section.planTestCases");
                // get the test case section to check
                if (testCaseSection) {
                    for (var i = 0; i < artifactArray.length; i++) {
                        if (testCaseSection._contains(artifactArray[i])) {
                            // changed test case is in the test case section of the test plan
                            if (changeType == GlobalEvents.ArtifactsDeleted) {
                                // mark editor with warning of associated test case is deleted
                                this.getHeader().setStatus(Constants.getMessage("msgEditorTestCaseInPlanDeleted"), ViewHeaderLayout.ERROR);
                            } else if (changeType == GlobalEvents.ArtifactsUpdated) {
                                // mark editor with warning of associated test case is changed
                                this.getHeader().setStatus(Constants.getMessage("msgEditorTestCaseInPlanChanged"), ViewHeaderLayout.INFO);
                            }
                            // done updating the editor as long as there is one change
                            return;
                        }
                    }
                }
            // handle test plans changed
            } else if (artifactType == Constants.TYPE_PLAN) {
                for (var i = 0; i < artifactArray.length; i++) {
                    if (this.getArtifact() && (this.getArtifact().id == artifactArray[i].id)) {
                        // it is in the deleted artifacts list, close itself
//                        this._publishSelfClosing();
                        // mark editor with warning of associated test case is changed
						if (!this.isSnapshot()) {
	                        this.getHeader().setStatus(Constants.getMessage("msgEditorTestPlanDeleted"), ViewHeaderLayout.ERROR);
							this.setArchived(true);
						}
                        // done updating the editor if it is deleted
                        return;
                    }
                }
            }
        }
    });
})();
