////TestScriptImportTableView
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptImportTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");


dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("TestScriptImportTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    constructor: function(parentView) {
        this.parentView = parentView;
    },

    rowCheckStateChanged: function(tableViewer, rowObject, state){
		var selectionCount = tableViewer.getCheckedRows().length;
        if (this.parentView.deleteActionNode)
            this.parentView.deleteActionNode.setDisabled(selectionCount == 0);
		if(this.parentView.rowCheckListener && dojo.isFunction(this.parentView.rowCheckListener)){
			this.parentView.rowCheckListener(selectionCount != 0);
		}
    },
	
	tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){
        if (callback == null && callbackData != null && dojo.isFunction(callbackData)) {
            callbackData();
        }
		if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    }
});

dojo.declare("TestScriptImportTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    getNumColumns: function(content){
        return 3;
    },

    getColumnHeading: function(content, index){
        var headings = [
            Constants.getMessage("labelName"),
			Constants.getMessage("labelLocation"),
			Constants.getMessage("labelType")
        ];
        return headings[index];
    },
    getColumnHeadingImage: function(content, atColumnIndex){
        return null;
    },
	 
    getColumnHeadingAlignment: function(atColumnIndex){
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex){
	 	  return false;
    },

    getIsColumnSortable: function(content, atColumnIndex){
        return true;     
    },
    
    getColumnWidth: function(content, atColumnIndex){
        switch(atColumnIndex) {
            case 0: 
                return 30;
            case 1: 
                return 50;
            case 2: 
                return 20;
        }
        return 0;
    },
    
    getContentObjects: function(content){
        return content;
    },
    
    getRowDataId: function(rowData){
        return "_table_row_for_testScript_" + rowData.id;
    },
    
    getCellDisplayType: function(rowData, atColumnIndex){
        var type = TableConstants.RENDER_TYPE_TEXT;
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex){
        return TableConstants.ALIGN_LEFT;
    },

    getCellDisplayData: function(rowData, atColumnIndex){
        var val = "data";
        switch(atColumnIndex) {
            case 0: 
                val = rowData.name;
                break;
            case 1: 
                val = rowData.dataStoreLocation;
                break;
            case 2: 
                val = Constants.getExtensionDisplayMessage(rowData.scriptTypeName);
                break;
        }
        return val;
    },

    getCellDisplayExtendedInfo: function(rowData, atColumnIndex){  
        var val ={};
        return val;
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptImportTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
	                 '   <div>' +
						  '      <span style=\"display: none;\" dojoAttachPoint="_queryTitle" class="query-title">${labelCurrentQuery}</span>' +
						  '      <span style=\"display: none;\" dojoAttachPoint="_queryName" class="query-name"></span>' +
						  '   </div>' +
						  '</div>',

    widgetsInTemplate: true,
	labelCurrentQuery: Constants.getMessage("labelCurrentQuery"),
	showFilterBox: true,

    constructor: function() {
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({showFilterControl: this.showFilterBox, sortByColumn: 0});        
        this._tableViewerNode.appendChild(tableViewer.domNode);
        var content = this.scripts ? this.scripts : [];
        tableViewer.setContentPageSize(20);
        tableViewer.setContent(content);
        tableViewer.setContentProvider(new TestScriptImportTableView.TableProvider());
        tableViewer.setTableListener(new TestScriptImportTableView.TableListener(this));
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.queryName));
		tableViewer.setTableRowsCheckable(true, /* isSingleSelect */(this.singleSelect ? this.singleSelect : false));
		 this.deleteActionNode = tableViewer.addAction("delete", // action id
                    Constants.getMessage("labelDelete"), // label
                    null, // callback data
                    "delete-icon-image", // icon class
                    dojo.hitch(this, "_onDeleteActionClick")); // callback
        this.deleteActionNode.setDisabled(true);
		tableViewer.show();
		this.tableViewer = tableViewer;		  
    },

    getScripts: function() {
        return this.tableViewer.getContent();
    },
	
	getSelectedRows: function(){
		return this.tableViewer.getCheckedRows();
	},
	
    refresh: function(scripts) {
		this.scripts = scripts ? scripts : [];
		this.tableViewer.setContent(this.scripts);
		this.tableViewer.show();
    },
	
	showLoadingProgress: function(){
		this.tableViewer.showLoadingProgress();
	},
	
	endLoadingProgress: function(){
		this.tableViewer.endLoadingProgress();
	},
	
	_onDeleteActionClick: function(){
		var selectedRows = this.tableViewer.getCheckedRows();
		var content = this.tableViewer.getContent();
		if(selectedRows && content && selectedRows.length > 0){
			for(var i=0;i<selectedRows.length;i++){
					var updatedContent = [];
					for(var j=0;j<content.length;j++){
						if(content[j].id == selectedRows[i].id){
							continue;
						}
						updatedContent.push(content[j]);
					}
					content = updatedContent;//swap updated content so that less traversal is needed.
				}
			this.refresh(content);
		}
	}
	 
});

})();
