/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.AutomationsTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.rational.test.lm.web.client.internal.inventory");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var column_types = [
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT ];
						
 	var OPERATIONS_HEADINGS = [
		  {title: RtlmUiUtil.getMessage("Name"), native_column: "jobName", isSortable: true, includeInFilter: true, width: 50}, 
		  {title: RtlmUiUtil.getMessage("Provider"), native_column: "providerName", isSortable: true, includeInFilter: true, width: 50  }
		];
	var ID_FIELD = "jobId";

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.AutomationsTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function()
	{
		this.setParameters( { "column_headings" : OPERATIONS_HEADINGS,
							  "column_types" : column_types,
							  "id_fields" : ID_FIELD } );
	}
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.AutomationsListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	constructor: function( params )
	{
		for( var param in params )
		{
			this[ param ] = params[ param ];
		}
	},
	
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
    	if( this.receiver )
    	{
    		this.receiver.update( rowObject, state );
    		if( this.machinestbl )
    		{
    			var jobs = tableViewer.getCheckedRows();
	    		this.checkedMachines = this.machinestbl.getCheckedRows();
	    		
    			if( ( jobs.length > 0 ) && !( this.receiver.refreshbutton.callbackData ) )
    			{   
    				//Save the states			
    				this.rowStates = this.machinestbl.rowCheckedHandler.rowStates;
	    			var automation = new com.ibm.rational.test.lm.web.client.internal.inventory();
	    			var handler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper( this, 
	    																							"_displayMachinesForProvider", 
	    																							"_getAvailableAssetsByProviderError" );
	    			automation.getAvailableAssetsByProvider( handler, jobs[ 0 ].providerName, jobs[ 0 ].providerInstance );
    			}
                else {
                    this.receiver._fetchMachinePermissionsAndCheckButtonState();
                }
    		}
     	}
    },
    
    _displayMachinesForProvider: function( machines )
    {
    	this.machinestbl.setContent( machines );
    	
    	var states = {};
        
    	for( var id in this.rowStates )
    	{
	    	for( var iMachine = 0; iMachine < machines.Data.length; iMachine++ )
	    	{
	    		if( machines.Data[ iMachine ][ 0 ] == id )
	    		{
	    			states[ id ] = this.rowStates[ id ];
	    			break;
	    		} 
	    	}
    	}
    	
    	this.machinestbl.rowCheckedHandler.rowStates = states;
    	this.machinestbl.show();
        
        // done re-fetching the machine list for the provider.
        // check perm to enable the run button.
        this.receiver._fetchMachinePermissionsAndCheckButtonState();
    },
    
    _getAvailableAssetsByProviderError: function( error )
    {
        // Error condition. Still call check perm to enable the run button.
        this.receiver._fetchMachinePermissionsAndCheckButtonState();
    }
});

})();
