// com.ibm.rational.test.lm.web.ui.internal.views.reservations.Reservations
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.reservations.Reservations");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dojo.parser");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");

dojo.require("com.ibm.rational.test.lm.web.client.internal.reservations");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.CSHelp");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.reservations.QueryReservationWidget");

(function() 
{
	var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var RequestUriUtil = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil;
	var QueryReservationWidget = com.ibm.rational.test.lm.web.ui.internal.views.reservations.QueryReservationWidget;
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	
	dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.reservations.Reservations", [dijit._Widget, dijit._Templated, com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener], 
	{
		//This line is NECESSARY fot the attachPoints in the PageLayout to get instantiated
		widgetsInTemplate: true,
		_comboValues: null,
		_itemCount: 0,
		
		//The layout of the page.
		templateString: 
		'<div id="${id}" dojoAttachPoint="domNode">' +
		'    <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout"' +
		'         noline="true" dojoAttachPoint="myPageLayout">' + 		
		'        <!-- The table of reservations -->  '+
		'        <div dojoAttachPoint="reservations_panel">'+
		'            <div dojoAttachPoint="QueryReservationsNode" />' +
		'       </div>' +
		'    </div>'+
		'</div>',

//===========================================================

		
	postCreate: function()
    {
		this.inherited("postCreate", arguments);
  
  		//Set the title and cs help id on the page
		this.myPageLayout.setTitle(RtlmUiUtil.getMessage("resReservationsPageTitle"));
		this.myPageLayout.setCSHelpId("rtlm_reservations_page.html", com.ibm.rational.test.lm.web.CSHelpID);
		this.myPageLayout.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", "", RtlmUiUtil.getMessage("resReservationsPageTitle")));
				
	    var params = {defaultQuery: QueryReservationWidget.ALL_RESERVATION_QUERY_ID, 
		              isMultiSelectTable: true,
                      loadQueryAtCreate: false,
					  runQueryCallback: dojo.hitch(this, "updateCancelButton"),
					  progressStartCallback: dojo.hitch(this, "_startProgress"),
					  progressEndCallback: dojo.hitch(this, "_endProgress")};
        this.reservationWidget = new QueryReservationWidget(params, this.QueryReservationsNode);
		
		this.resultTable = this.reservationWidget.resultTable;
		this.reservationWidget.setRowCheckStateListener(this);
        
        this.cancelResButton = this.resultTable.addAction("cancel", RtlmUiUtil.getMessage("resCancelSelected"), null, "unreserve-icon-image", dojo.hitch( this, this.cancelReservations ) );
        this.updateButtonStatus();              
        
        // don't let the reservation result widget load the query combo during 
        // widget instantiation time. It's possible that combo results is returned before the 
        // widget is completed creation and it would call into the updateButtonStatus() (thru
        // _endProgress()), and our resultTable attribute would not have been initialized.
        // Explicitly init the query combo after the widget has been created.
        this.reservationWidget.loadQueryCombo();
	},
	
	_onRefreshActionClick: function()
	{
		this.reservationWidget.runSelectedQuery();
	},
	
	_startProgress : function()
	{
		this.myPageLayout.setProgressStart(RtlmUiUtil.getMessage("resMsgRetrievingReservationsWorking"));
		this.updateButtonStatus();
	},
	
	_endProgress : function()
	{
		this.myPageLayout.setProgressEnd();
		this.updateButtonStatus();
	},
	
	//Return true if any rows are checked
	isAnyRowChecked: function()
    {
		var resultTable = this.resultTable;
		var checkedRows = resultTable.getCheckedRows();
		return checkedRows.length > 0;
    },
	
	getCheckedCount : function()
	{
		var resultTable = this.resultTable;
		var checkedRows = resultTable.getCheckedRows();
		return checkedRows.length;
		
	},
	
    _hasCancelPermissions: function()
    {
        var selectedRows = this.resultTable.getCheckedRows();
        
        for(var i = 0; i < selectedRows.length; i++) {
            if(selectedRows[i]["canCancel"] == false)
                return false;
        }
        return true;
    },
    
	//Enable the cancel button if any rows are checked
	updateButtonStatus: function()
    {
		var checkedRows = this.getCheckedCount();
        var disableCancel = true;
        var hasPerm = true;
         
        if(checkedRows > 0) {
            hasPerm = this._hasCancelPermissions();
            disableCancel = !hasPerm;
        }
        this.cancelResButton.setDisabled(disableCancel);
        
        // show a warning message if the user doesn't have perm to
        // cancel the reservation.
        if(!hasPerm) {
            this.myPageLayout.setStatus(RtlmUiUtil.getMessage("resNoPermToCancel"), 
                ViewHeaderLayout.WARNING, RtlmUiUtil.getMessage("resNoPermToCancelDetails"));
        }
    },
	
	//Callback when a query is run
	updateCancelButton : function()
	{
		this.updateButtonStatus();
	},
	
	cancelReservations : function()
	{
		//The callback handler for the cancel and release calls
		var callbackObj = 
		{
			parent:this,
			successFunc: function(response) 
			{
				//console.debug("Successful server response: " + response);
				
				//If we are at the last item then reload the table
				//_itemCount is set below by counting all the checked rows
				this.parent._itemCount--;
				if (this.parent._itemCount == 0) 
				{
					this.parent.myPageLayout.setProgressEnd();
					this.parent.reservationWidget.runSelectedQuery();
				}
	    	},
			errorFunc: function(errorType, errorMessage) 
			{
				//console.debug("Bad server response: " + errorType);

				//If we are at the last item then reload the table
				//_itemCount is set below by counting all the checked rows
				this.parent._itemCount--;
				if (this.parent._itemCount == 0) 
				{
					this.parent.myPageLayout.setProgressEnd();
					this.parent.reservationWidget.runSelectedQuery();
				}
                if(errorType.message && errorType.message.toLowerCase().indexOf('license') > -1) {
                    RtlmUiUtil.showServerErrorMessage(MessageUtil.getMessage(MessageBundle.COMMON, "serverLicenseErrMsg"));                  
                }                
			}
		};
		var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(callbackObj, "successFunc", "errorFunc");
		var reservations = new com.ibm.rational.test.lm.web.client.internal.reservations();
		
		var table = this.resultTable;
		var checkedRows = table.getCheckedRows();
		if (checkedRows.length == 0)
			return;
		//Let the user know we are cancelling......
		this.myPageLayout.setProgressStart(RtlmUiUtil.getMessage("resCanceling"));
		this._itemCount = checkedRows.length;
		for (var i = 0; i < checkedRows.length; i++)
		{
			var row = checkedRows[i];
			var ID = row.id;
			//Get the dates of the reservation to determine if we are to perform a Cancel or a Release
			var fromDate = RtlmUiUtil.convertTimeStampToDate(row.reserveFrom);
			var toDate = RtlmUiUtil.convertTimeStampToDate(row.reserveTo);
			var today = new Date();
			if (fromDate <= today && toDate >= today)  //Do Release
			{
				reservations.endReservation(responseHandler, ID);
				
			}
			else  //Do Cancel
			{
				reservations.cancelReservation(responseHandler, ID);
			}
		}
	},

 	//Callback from the table when a row is checked	
	rowCheckStateChanged : function(tableViewer, rowObject, state)
    {
		//Update the button accordingly
        this.myPageLayout.clearStatus();
        this.updateButtonStatus();
    },
	
	
	/**
	 * Show the context sensitive help content in a predefined help panel.
	 */
	showHelpContent: function(helpButton, contextId)
	{
		this.CSHelp.showHelpContent(helpButton.domNode, contextId);
	}, 

	init: function() 
	{
		console.debug("Reservations.init");
	}
});

})();
