/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// author: vasrivas
//
// Summary: This is a widget to upload attachments
//
//

console.debug("Loading: com.ibm.rqm.datapool.web.ui.internal.widgets.DatapoolAttachmentViewer");
dojo.provide("com.ibm.rqm.datapool.web.ui.internal.widgets.DatapoolAttachmentViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dojo.io.iframe");
dojo.require("dojo.parser");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() { // module container

var Constants = com.ibm.asq.common.web.client.internal.Constants;
var PlanningConstants = com.ibm.rqm.planning.web.client.internal.Constants;

var TeamServerClient            = com.ibm.team.repository.web.transport.TeamServerClient;
var getCurrentProjectArea       = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var isGuest                     = com.ibm.team.repository.web.client.session.isGuest;
var UPLOAD_SERVICE_URI          = "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"; //RQM Upload Service
var DOWNLOAD_SERVICE_URI        = "com.ibm.rqm.integration.service.IIntegrationService/"; //RQM Downaload Service
var ACTION_PROP                 = "__jazzActionProp";
var FILE_UPLOAD_RESPONSE_PREFIX = "File Upload Response:"; // Must match response prefix in AttachmentRestService.java 
var _BYTES                      = " bytes";
var _currentBindRequest         = null;
var _currentUploadFrameNumber   = 10000;


function displayBlock(node, isDisplayed) {
    node.style.display= isDisplayed ? "block" : "none";
}

dojo.declare("com.ibm.rqm.datapool.web.ui.internal.widgets.DatapoolAttachmentViewer", [dijit._Widget, dijit._Templated],
{
        templatePath: dojo.moduleUrl("com.ibm.rqm.datapool.web", "ui/internal/widgets/templates/DatapoolAttachmentViewer.html"),       
 
            
        labelToAddAnAttachment: Constants.getMessage("labelToAddAnAttachment"),
        labelNumber: Constants.getMessage("labelNumber"),
		labelAttachmentId: PlanningConstants.getMessage("labelIdWithColon"),
		labelAttachmentName: PlanningConstants.getMessage("labelFileNameWithColon"), 
		labelAttachmentSize: PlanningConstants.getMessage("labelSizeWithColon"),
        
        postCreate: function() 
        {   
            this.editable = true;
            
            if(this._uploadForm != null)
            {
                this._initializeUploadInput();
                this._uploadForm_onsubmit_handle = dojo.connect(this._uploadForm, "onsubmit", this, "_performUploadSubmit");
            }
            this.reset();   
        },
        
		/*
         * getAllAttachments()
         * This can be called to get all the attachments present in the attachment table
         * Eg. of Array objects
         * {fileName: 'test.txt',id: 12,size: 4096,path: "C://Test/Project1"uuid: _UxtmJ01203452};
         */
                
        getAllAttachments: function()
        {
            return this._attachmentArray;
        },
		
        /*
         *initialize()
         * First function to be called when the view is created.
         * isPreviewMode: boolean
         * isNonEditableMode: boolean
         * elements: Array of Attachments
         * Eg. of Array object
         * {
                fileName: 'test.txt',
                id: 12,
                size: 4096,
                path: "C://Test/Project1"
                uuid: _UxtmJ01203452
            }; 
         * downloadServiceUri: String   eg. "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"
         * uploadServiceUri: String     eg. "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"
         */
        
        initialize: function(args)
        {
            if(args != null)
            {
                this.parentSection = args.parentSection ? args.parentSection : null;
                this._attachmentArray = args.elements ? args.elements : []; 
                
                UPLOAD_SERVICE_URI = args.uploadServiceUri ? args.uploadServiceUri:  UPLOAD_SERVICE_URI;
                DOWNLOAD_SERVICE_URI = args.downloadServiceUri ? args.downloadServiceUri:  DOWNLOAD_SERVICE_URI;
                if(args.isNonEditableMode)
                {
                    this.editable = false;
                    dojox.data.dom.removeChildren(this._uploadForm);
                    dojox.data.dom.removeChildren(this._message);
                    dojox.data.dom.removeChildren(this._uploadDiv);
                }else //Editable Mode
                {
                    this.editable = true;
                }
            }else
                this._attachmentArray = [];

			
			this._rebindAttachments();
        },
         
        reset: function() {
            this._attachmentArray= [];
            displayBlock(this._uploadForm, !isGuest());
            this._initializeUploadInput(); // reset the form
        },
        
        _initializeUploadInput: function() {
            var div= this._uploadInputDiv;
            dojox.data.dom.removeChildren(div);
            var inputLabel = document.createElement("label");
            var input1= document.createElement("INPUT");
            input1.type= "file";
            input1.name= "uploadFileInput";
            input1.className="UploadControl";
            this._uploadFileInput = input1;
			this._uploadFileInput_onchange_handle = dojo.connect(this._uploadFileInput, "onchange", this, "_askReplace");
            inputLabel.appendChild(input1);
            div.appendChild(inputLabel);
        },

        _getFileNameAndId: function(/* String id:filename*/ fileName) {
                var result = { id: " ", fileName: fileName };
                var fileNameColonIndex = fileName.indexOf(":");
                if(fileNameColonIndex > -1) {
                    result.id = fileName.substring(0,fileNameColonIndex);
                    result.fileName = fileName.substring(fileNameColonIndex+1, fileName.length);
                }
                return result;
        },

        _rebindAttachments: function() {
            if (this._attachmentArray.length == 1){
				this._attachmentId.innerHTML = this._attachmentArray[0].id;
            	var href = TeamServerClient._getServicePrefix() + DOWNLOAD_SERVICE_URI + "attachment/urn:com.ibm.rqm:attachment:"+this._attachmentArray[0].id;
				this._attachmentName.innerHTML = "<a href='" + href + "'>" + this._attachmentArray[0].fileName + "</a>";
				this._attachmentSize.innerHTML = this._attachmentArray[0].size;
			}
        },
		
        _showFormHideMessage: function() {
            this._isUploadInProgress = false;
            this._initializeUploadInput(); // reset the form
        },
        
        isUploadInProgress: function() {
            return this._isUploadInProgress;            
        },
		
		_askReplace: function(e) {
			if (this._attachmentArray.length == 1) {
				var r = confirm(Constants.getMessage("labelReplaceTheCurrentFile"));
				if (r == true) {
					this._performUploadSubmit();
				}
				else {
					e.preventDefault(e);
					this._uploadFileInput.value = "";
				}
			}else{
				this._performUploadSubmit();
			}
	   },

        _performUploadSubmit: function(e, isRestart) {
            if(e) {
                e.preventDefault(); 
            }
            this._isUploadInProgress = true;
            dojo.disconnect(this._uploadFileInput_onchange_handle);
            
            var fullPath;
            // Check to see if the file is already loaded
            if(isRestart) {
                fullPath = this._priorFullPath;
            } else {
                fullPath = this._uploadFileInput.value;
            }
            if (fullPath.length === 0) {
                this._showFormHideMessage();
                return;
            }
            this._priorFullPath = fullPath;
            var parts= fullPath.split("\\");
            var fileName= parts[parts.length-1];
            
            for (var i=0; i<this._attachmentArray.length; i++) {
                if (this._attachmentArray[i].fileName == fileName) {
                    //fix defect 5940: check the fileName when uploading
                    var r = confirm(Constants.getMessage("confirmAttachmentHasBeenAdded", fullPath));
					if (r == false) {
						this._showFormHideMessage();
						return;
					}
                }           
            }
            var self= this;
            if(this._currentBindRequest != null) {
                this._currentBindRequest.cancel();
                this._currentBindRequest  = null;
                }
        /*
         * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the 
         * next one breaks. Resetting the iframe fixes the problem
         * Unfortunately this code reaches into iframe private variables.
         */
            if(dojo.io.iframe._frame) {
                document.body.removeChild(dojo.io.iframe._frame);               
            }
            var frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
            dojo.io.iframe._iframeName = frameName;
            dojo.io.iframe._frame = null;
            
            var url= TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + getCurrentProjectArea().itemId;
            
            this._currentBindRequest = dojo.io.iframe.send({
                form: this._uploadForm,
                url: TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + getCurrentProjectArea().itemId,
                content: {
                        mimetype: "text/html",
                        fileFields: "uploadFileInput"
                    },
                handleAs:  "html",
                handle: function(response, ioArgs) {
                    if(response instanceof Error){
                            if(response.dojoType && response.dojoType == "cancel") {
                                return; // this is a cancelled request, ignore it
                            }
                            self._currentBindRequest  = null; // no need to cancel it any more
                            alert(Constants.getMessage("alertErrorDuringFileUpload"));
                            self._showFormHideMessage();
                            console.debug("Error uploading file: " + response.message);
                            return;
                        }
                    var data = response;
                    // If the user presses cancel on Firefox or Stop, then we get an extra call on the next upload. Skip it
                    // The data URI of the bad call is for the page and not the service
                    if(data && data.documentURI && data.documentURI.indexOf("IAttachmentRestService") == -1) {
                        self._wasUploadCancelled = true;
                        self._performUploadSubmit(null, true);
                        return;                     
                    }                   
                    if(self._currentBindRequest && self._currentBindRequest.isCancelled) {
                        self._currentBindRequest = null; // no need to cancel it any more
                        return;
                        }
                    self._currentBindRequest  = null; // no need to cancel it any more
                    var uuidOrMessage = data.body.innerHTML; /* innerHTML OK */
                    if(uuidOrMessage.indexOf(FILE_UPLOAD_RESPONSE_PREFIX) != 0) {
                        alert(Constants.getMessage("alertFileUploadFailed", Constants.getMessage("alertFileUploadFailedReason")));
                        console.debug("Upload failed. Response is [" + uuidOrMessage + "]");
                        self._showFormHideMessage();
                        return;
                    }
                    uuidOrMessage = uuidOrMessage.substring(FILE_UPLOAD_RESPONSE_PREFIX.length, uuidOrMessage.length);
                    if (uuidOrMessage.substring(0,1) != "_") {
                        // If an upload was cancelled we get one of these anomolous calls
                        if(self._wasUploadCancelled) {
                            self._wasUploadCancelled = false;
                            return;
                        }
                        alert(Constants.getMessage("alertFileUploadFailed", uuidOrMessage));
                        self._showFormHideMessage();
                        return;
                    }

                    var message = uuidOrMessage.split(',');
                    var size=0;
                    if(message.length > 0) {
                        uuidOrMessage = message[0];
                        size = message[1];;
                        var id = message[2];
                        fileName = id +":"+fileName;
                    }
                    
                    if(size=="0") {
                        if(!confirm(Constants.getMessage("alertFileIsEmpty", fileName))) {
                            self._showFormHideMessage();
                            return;
                        }                       
                    }
                    
                    var fileNameAndId = self._getFileNameAndId(fileName);               
                    var attachment = {
                        id: fileNameAndId.id,
                        fileName: fileNameAndId.fileName,
                        size: size,
                        path: fullPath,
                        uuid: uuidOrMessage,
                        isNew: true
                    };
                    self._attachmentArray[0]= attachment ;
					self.parentSection.setDirty();
                    self._rebindAttachments();
                    
                    self._showFormHideMessage();
                    
                    var uri = TeamServerClient._getServicePrefix() + DOWNLOAD_SERVICE_URI + "attachment/urn:com.ibm.rqm:attachment:"+attachment.id;
                    uri = location.protocol + "//" + location.host + uri;
					var parentEditorSections = self.parentSection.getParentEditor().getSections();
					var dataSection = null;
                    for(var i = 0; i < (parentEditorSections ? parentEditorSections.length: 0); i++) {
						if(parentEditorSections[i].declaredClass == "com.ibm.rqm.datapool.web.ui.internal.view.extension.DatapoolDataSection"){
							dataSection = parentEditorSections[i];
							break;
						}
					}
					if(dataSection) {
						dataSection.updateDataTable(uri);
					}
                }
            });
        },
        
        refresh: function()
        {
            //Do nothing
        },
        
        _error: function(errorObj)
        {
            alert(Constants.getMessage("ErrorMsg", errorObj.message));
        }
});
}
)();
