/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
DatePicker = dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");

// convenient constant to specify today and yesterday
DatePicker.today = new Date();
DatePicker.yesterday = new Date().setDate(new Date().getDate() - 1);

dojo.require("dojo.date");
dojo.require("dijit.form.DateTextBox");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.DatePicker", dijit.form.DateTextBox, {

    templateString: "<table style=\"display: -moz-inline-stack;\" class=\"dijit dijitReset datePickerInline\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse\" waiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset datePickerInputField\" width=\"100%\"\r\n\t\t\t><input dojoAttachPoint='textbox,focusNode' dojoAttachEvent='onfocus,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress' autocomplete=\"off\"\r\n\t\t\ttype='${type}' name='${name}'\r\n\t\t/></td\r\n\t\t><td class=\"dijitReset dijitValidationIconField\" width=\"10%\"\r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div><div class='dijitValidationIconText'>&Chi;</div\r\n\t\t></td\r\n\t></tr\r\n></table>\r\n",
    nonExistDateMessage: "",
    minDate: null,
    maxDate: null,
	isInDialog: false,
    dateFormat: com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatLength(),
    imgSrc: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/elcl/calendar.gif"),
    imgSrcDisabled: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/dlcl/calendar.gif"),
    _calendarNode: null,
    _calendarImg: null,
    _calendarImgDisabled: null,
	popupCalendarOnFocus: false,
    
    constructor: function(){
    },
    
    // Override the compare to provide a date only compare.
    compare: function(date1, date2){
        return dojo.date.compare(date1, date2, "date");
    },
    
    postCreate: function(){
        this.inherited('postCreate', arguments);

        this.constraints = {selector: 'date', formatLength: this.dateFormat};
	
        if (this.minDate)
            this.constraints['min'] = this.minDate;
        if (this.maxDate) 
            this.constraints['max'] = this.maxDate;
        
        // on IE, the mozilla inline stack style is not honored.
        // make our dom node style inline to avoid line breaks before & after    
        if(dojo.isIE) {
            dojo.style(this.domNode, "display", "inline");
        }
        // Init the widget to create a calendar pop up button to the parent node.
        this.init();
    },
    
    // Summary:
    //    Init the widget to create a calendar pop up button to the parent node.
    //    Note that the parent node may not exist yet so the consumer will need
    //    to invoke the init() method after the widget is attached to a parent.
    init: function(){
        var parentNode = ((dojo.isIE) ? this.domNode.parentElement : this.domNode.parentNode);
        if (parentNode) {
            this._calendarNode = document.createElement("button");
            dojo.addClass(this._calendarNode, "icon-image-button");
            this._calendarNode.onclick = dojo.hitch(this, "_open");
            this._calendarNode.style.display = "inline";
			
			this._calendarNode.name=MessageUtil.getMessage(MessageBundle.COMMON, "labelDateSelect");
            
            this._calendarImgDisabled = document.createElement("img");
            this._calendarImgDisabled.src = this.imgSrcDisabled;
            this._calendarImg = document.createElement("img");
            this._calendarImg.src = this.imgSrc;
            
            if (this.disabled) 
                this._calendarNode.appendChild(this._calendarImgDisabled);
            else 
                this._calendarNode.appendChild(this._calendarImg);
            this._calendarNode.disabled = this.disabled;
            
            parentNode.appendChild(this._calendarNode);
        }
    },
    
    setDisabled: function(/* Boolean*/disabled){
        if (this._calendarNode) {
            if (disabled) {
                if (!this.disabled) {
                    this._calendarNode.removeChild(this._calendarImg);
                    this._calendarNode.appendChild(this._calendarImgDisabled);
                }
            }
            else {
                if (this.disabled) {
                    this._calendarNode.removeChild(this._calendarImgDisabled);
                    this._calendarNode.appendChild(this._calendarImg);
                }
            }
            this._calendarNode.disabled = disabled;
        }
        this.inherited('setDisabled', arguments);
        if (disabled) 
            this.validate();
    },

    
    _open: function(){
        if(this.disabled){return;}

        var self = this;

        if(!this._picker){
            var popupProto=dojo.getObject(this._popupClass, false);
            this._picker = new popupProto({
                onValueSelected: function(value){

                    self.focus(); // focus the textbox before the popup closes to avoid reopening the popup
                    setTimeout(dojo.hitch(self, "_close"), 1); // allow focus time to take

                    // this will cause InlineEditBox and other handlers to do stuff so make sure it's last
                    dijit.form.TimeTextBox.superclass.setValue.call(self, value, true);
                    self.onChange();
                    
                    // always tigger an onChange
                },
                lang: this.lang,
                constraints:this.constraints,
                isDisabledDate: function(/*Date*/ date){
                    // summary:
                    //  disables dates outside of the min/max of the TimeTextBox. Specify
                    //  this is a date compare only. This removes any time component for the
                    //  comparison. 
                    return self.constraints && (dojo.date.compare(self.constraints.min,date,"date") > 0 || dojo.date.compare(self.constraints.max,date,"date") < 0);
                }
            });
        }
        if(!this._opened){
            // if no value is in the date box, try to use the min date first
            // before resorting to the current date.
            var val = this.getValue() ? this.getValue() : this.constraints.min;
            this._picker.setValue(val || new Date());
            dijit.popup.open({
                parent: this,
                popup: this._picker,
                around: this.domNode,
                onCancel: dojo.hitch(this, this._close),
                onClose: function(){ self._opened=false; }
            });
            this._opened=true;
        }
        
        dojo.marginBox(this._picker.domNode,{ w:this.domNode.offsetWidth });

        dijit.focus(this.domNode);
		if(this.isInDialog)
            // our common dialogs are set to 10002 on FF 3, need to 
            // make it higher then our dialogs.
			this._picker.domNode.parentNode.style.zIndex = 10003;
    },
    
    // Summary:
    //    Override the parent's method to add additional test on non-existing dates.
    //
    isValid: function(/* Boolean*/isFocused){
        var isValid = this.inherited('isValid', arguments);
        
        // add one more check to ensure that the entered string is a valid date.
        // dates like 02/30/07 passes the format validation, but the date doesn't 
        // exist. Our checkDate ensure that we keep the text box in invalid mode if
        // such a date is entered.
        if (isValid) 
            return this._checkDate();
        return isValid;
    },
    
    // Summary:
    //    Check for invalid date that passes the format validation. If a date string
    //    is valid in format, the date text box doesn't flag it as invalid. However,
    //    its date value is null. Add this check to make the date text box stay in 
    //    invalid mode.
    // 
    _checkDate: function(){
        var date = this.getValue();
        
        if (!date && this.textbox.value && this.textbox.value.length > 0) {
            this.invalidMessage = this.nonExistDateMessage;
            return false;
        }
        return true;
    },
	
	_onFocus: function( evt )
	{
		//Don't popup the calendar when the control gets the focus unless required.
		if (this.popupCalendarOnFocus) {
			this.inherited( "_onFocus", arguments );
		}
	},
	
	displayMessage: function(/*String*/message){
		//Make sure that the class dijitTooptipContainerTop is added so that
		//the top is set correctly. 
		if( !dijit._masterTT ) { dijit._masterTT = new dijit._MasterTooltip(); }
		
		if( dijit._masterTT.containerNode )
		{
			var cls = dijit._masterTT.containerNode.className;
			if( cls.indexOf( "dijitTooltipContainerTop" ) == -1 )
			{
				dojo.addClass( dijit._masterTT.containerNode, "dijitTooltipContainerTop" );
			}
		}
		
		this.inherited( "displayMessage", arguments );
		
		if( dijit._masterTT )
		{
			//if the class is removed, add it back
			var cls = dijit._masterTT.containerNode.className;
			if( cls.indexOf( "dijitTooltipContainerTop" ) == -1 )
			{
				dojo.addClass( dijit._masterTT.containerNode, "dijitTooltipContainerTop" );
			}			
		}
	}
});
