/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.SimpleTimeSelector");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.SimpleTimeSelector",
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-asq-common-web-ui-simple-time-selector">' +
                    '  <label for="month-select"><span style="display:none">${monthSelect}</span></label>' +
					'  <select id="month-select" dojoAttachPoint="hourSelectNode" dojoAttachEvent="onchange:_valueSelected">' +
                    '    <option value="1">1' +
                    '    </option>' +
                    '    <option value="2">2' +
                    '    </option>' +
                    '    <option value="3">3' +
                    '    </option>' +
                    '    <option value="4">4' +
                    '    </option>' +
                    '    <option value="5">5' +
                    '    </option>' +
                    '    <option value="6">6' +
                    '    </option>' +
                    '    <option value="7">7' +
                    '    </option>' +
                    '    <option value="8">8' +
                    '    </option>' +
                    '    <option value="9">9' +
                    '    </option>' +
                    '    <option value="10">10' +
                    '    </option>' +
                    '    <option value="11">11' +
                    '    </option>' +
                    '    <option value="12">12' +
                    '    </option>' +
                    '  </select>' +
                    '  <label for="day-select"><span style="display:none">${daySelect}</span></label>'+
					'  <select id="day-select" dojoAttachPoint="minSelectNode" dojoAttachEvent="onchange:_valueSelected">' +
                    '    <option value="0">00' +
                    '    </option>' +
                    '    <option value="15">15' +
                    '    </option>' +
                    '    <option value="30">30' +
                    '    </option>' +
                    '    <option value="45">45' +
                    '    </option>' +
                    '    <option value="59">59' +
                    '    </option>' +
                    '  </select>' +
                    '  <label for="hour-select"><span style="display:none">${hourSelect}</span></label>'+
					'  <select id="hour-select" dojoAttachPoint="ampmSelectNode" dojoAttachEvent="onchange:_valueSelected">' +
                    '    <option value="0">AM' +
                    '    </option>' +
                    '    <option value="12">PM' +
                    '    </option>' +
                    '  </select>' +
                    '</div>',
    time: null,
	
	monthSelect: MessageUtil.getMessage(MessageBundle.COMMON, "labelMonthSelect"),
	daySelect: MessageUtil.getMessage(MessageBundle.COMMON, "labelDaySelect"),
	hourSelect: MessageUtil.getMessage(MessageBundle.COMMON, "labelHourSelect"),
    
    postCreate: function() 
    {
        if(!this.time) {
            this.time = new Date();
            this.time.setHours(0, 0, 0, 0);
        }
        this.setTime(this.time);
    },
    
    // Summary:
    //    Set the time value of the time picker. The time is a Date object and only
    //    the hour and minute component of the date object is used to seed the values
    //    in the picker.
    //
    setTime: function(time)
    {
        var hr = time.getHours();     
        var min = time.getMinutes();  
        var pm = false;
        
        if(hr == 0) {
            hr = 12;
        }
        else if(hr >= 12) {
            if(hr > 12)
                hr -= 12;  
            pm = true;
        }
        this.hourSelectNode.options[hr-1].selected = true;
        
        for(var i = 0; i < this.minSelectNode.options.length; i++) {
            var val = parseInt(this.minSelectNode.options[i].value);
            if(min <= val) {
                this.minSelectNode.options[i].selected = true;
                break;
            }
        }
        if(pm) {
            this.ampmSelectNode.options[1].selected = true;
        }
        else {
            this.ampmSelectNode.options[0].selected = true;
        }
        
    },
    
    // Summary:
    //    Returns the current time value in the time selector. A Date object is 
    //    returned. The client should only retrieve the hour/minute components of the
    //    returned Date object.
    // 
    getTime: function()
    {
        var am = this.ampmSelectNode.options[0].selected;
        var hr = this._getSelectValue(this.hourSelectNode);
        var min = this._getSelectValue(this.minSelectNode);
        
        if(am) {
            if(hr == 12) {
                hr = 0;
            }
        }
        else {
            if(hr < 12)
                hr += 12;
        }
        this.time.setHours(hr, min, 0, 0);
        return this.time;
    },
        
    _valueSelected: function()
    {
        this.onChange();
    },
    
    onChange: function()
    {
        
    },

    _getSelectValue: function(node)
    {
        var val = 0;
        for(var i = 0; i < node.options.length; i++) {
            if(node.options[i].selected) {
                val = parseInt(node.options[i].value);
                break;
            }
        }
        return val;
    }
    
});

})();
