/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// author: Bianca Jiang
//
// Summary: This is an extension to the dojo TimeTextBox to have an icon next to the text box
// that is clickable for the popup time picker instead of click on the text box.  A calendar
// button will be created next to the text box in the parent node so if the parent node is not
// specified in the constructor, the consumer must invoke the init() method to explicitly create
// the button.
//
// New attribute:  
// imgSrc - the path to a custom img file for the icon. Default is a clock-look icon.
// imgSrcDisabled - the path to a custom img file for the disabled icon.  Default is a faded clock icon.
//
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.TimePicker");
dojo.require("dijit.form.TimeTextBox");
dojo.require("dijit.form.Button");

dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.TimePicker",
    dijit.form.TimeTextBox,
{
    imgSrc: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/elcl/clock.gif"),
    imgSrcDisabled: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/dlcl/clock.gif"),
    _clockNode: null,
    _clockImg: null,
    _clockImgDisabled: null,
	
	postCreate: function(){
		this.inherited('postCreate', arguments);
		
        // Init the widget to create a clock pop up button to the parent node.
        this.init();
    },
    
    // Summary:
    //    Init the widget to create a clock pop up button to the parent node.
    //    Note that the parent node may not exist yet so the consumer will need
    //    to invoke the init() method after the widget is attached to a parent.
    init: function() {
        var parentNode = this.domNode.parentNode;
        if (parentNode) {   
//            this._clockNode = document.createElement("button");
//            dojo.addClass(this._clockNode, "icon-image-button");   
//            this._clockNode.onclick = dojo.hitch(this, "_open");
//            this._clockNode.style.display = "inline";
//            
//            this._clockImgDisabled = document.createElement("img");
//            this._clockImgDisabled.src = this.imgSrcDisabled;
//            this._clockImg = document.createElement("img");
//            this._clockImg.src = this.imgSrc;
//            
//            if (this.disabled)
//                this._clockNode.appendChild(this._clockImgDisabled);
//            else
//                this._clockNode.appendChild(this._clockImg);
//            this._clockNode.disabled = this.disabled;
//            
//            parentNode.appendChild(this._clockNode);
			var arg = {
	            label: MessageUtil.getMessage(MessageBundle.COMMON, "timepicker_button"),
	            showLabel: false, // label only or both icon and label
	            iconClass: ( this.disabled ) ? "timepicker-button-image-disabled" : "timepicker-button-image",
	            'class': "icon-image-button",
				onClick: dojo.hitch(this, "_open")
        	};
			
			this._clockNode = new dijit.form.Button( arg );
			parentNode.appendChild( this._clockNode.domNode );
        }
	},
	
    setDisabled: function(/* Boolean*/ disabled) {
        if (this._clockNode) {
            if (disabled) {
                if (!this.disabled) {
//                    this._clockNode.removeChild(this._clockImg);
//                    this._clockNode.appendChild(this._clockImgDisabled);
					this._clockNode.iconNode.className = "timepicker-button-image-disabled";
                }
            } else {
                if (this.disabled) {
//                    this._clockNode.removeChild(this._clockImgDisabled);
//                    this._clockNode.appendChild(this._clockImg);
					this._clockNode.iconNode.className = "timepicker-button-image";
                }
            }
            this._clockNode.disabled = disabled;
        }
        this.inherited('setDisabled', arguments);
        if (disabled)
            this.validate();
    },
    
	_onFocus: function(/*Event*/ evt){
		//override the supper class behavior. intentionally do nothing.
	},
	
	_open: function() {
		this.inherited(arguments);
		dijit.focus(this.domNode);
	}
});
