/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.jrs.Uploader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.io.iframe");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ImportPageLayout");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.asq.common.web.client.internal.ProjectAliasClient");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");

(function(){

    var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
    var SERVICE_PREFIX = dojo.baseUrl + "../../service/";
    var SERVICE_NAME = "com.ibm.rqm.integration.service.IIntegrationService";
    var FILE_UPLOAD_RESPONSE = "File Upload Response:";
    var Constants = com.ibm.asq.common.web.client.internal.Constants;
    var ProjectAliasClient = com.ibm.asq.common.web.client.internal.ProjectAliasClient;
    
    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.jrs.Uploader", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/templates/Uploader.html"),
        widgetsInTemplate: true,
		_projectAlias: null,

        constructor: function() {
            // default title for the page header
            this.title = "";
            
            // default message prompt in the import section
            this.importPrompt = "";
            
            // cs help info for the page header
            this.contextHelpId = null;
            this.csHelpEngine = null;
            this.csHelpProviderId = null;
            this.cshelp = null;
            this.cshelpLabel = null;
            
            // cs help info for the import section message prompt
            this.importPromptCShelpId = null;
            this.importPromptCShelp = null;
            this.importPromptCShelpLabel = null;
        },

        postCreate: function(){
            this.inherited('postCreate', arguments);
            
            this._header.setTitle(this.title);
            this._header.onImport = dojo.hitch(this, "onUpload");
            this._header.showContentArea(false);
            
            if (this.contextHelpId)
                this._header.setCSHelpId(this.contextHelpId, this.csHelpProviderId, this.csHelpEngine);
            if (this.cshelpLabel)
                this._header.setCSHelpButtonLabel(this.cshelpLabel);
            
            this._uploadButton = this._header.getImportButton();
            dojo.connect(this.uploadFileInput, "onchange", this, "_enableUpload");
            
            this.fileLabel.innerHTML = Constants.getMessage("labelFile");
            if (this.importPrompt)
                this._mainImportSection.setTitle(this.importPrompt);
            if (this.importPromptCShelpId)
                this._mainImportSection.setCSHelpId(this.importPromptCShelpId, this.csHelpProviderId, this.csHelpEngine);
            if (this.importPromptCShelpLabel)
                this._mainImportSection.setCSHelpButtonLabel(this.importPromptCShelpLabel);
                
			// get the project area alias
			var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
	        var self = this;
	        var handler = {
	            success: function(alias) {
	            	self._setProjectAlias(alias);
	            },
	            error: function(e) {
	            	console.error(this.declaredClass + ": Error: " + e.message);
	            }
	        };
	        var responseHandler = new ServiceResponseHandler(handler, "success", "error");
	        ProjectAliasClient.getProjectAlias(responseHandler, {processAreaUUID : projectArea.itemId});
                
        },
        
        getViewHeader: function(){
            return this._header;
        },
        
        getUploadButton: function(){
            return this._uploadButton;
        },
        
        _enableUpload: function(){
            this._uploadButton.disabled = (this.uploadFileInput.value.length == 0);
        },
        
        _setProjectAlias: function(projectAlias) {
            this._projectAlias = projectAlias;
        },
        
        onUpload: function(){
            this._uploadButton.disabled = true;
            this._header.setProgressStart(Constants.getMessage("uploaderProgressUploading"));
            this._header.clearStatus();
            var deferred = dojo.io.iframe.send({
                url: SERVICE_PREFIX + SERVICE_NAME + "/resources/" + this._projectAlias + "/" + this.artifactType,
                handleAs: "html",
                uploadButton: this._uploadButton,
                uploadFileInput: this.uploadFileInput,
                header: this._header,
                form: this._uploadForm,
                content: {
                    mimetype: "text/html",
                    fileFields: "uploadFileInput"
                },
                handle: function(response, ioArgs){
                    this.header.setProgressEnd();
                    if (response instanceof Error) {
                        this.uploadButton.disabled = false;
                        if (response.dojoType && response.dojoType == "cancel") {
                            return; // this is a cancelled request, ignore it
                        }
                        console.debug("Error uploading file: " + response.message);
                    } else {
                        var responseMsg = response.body.innerHTML;
                        if (responseMsg.indexOf(FILE_UPLOAD_RESPONSE) == 0) {
                            var data = responseMsg.substring(FILE_UPLOAD_RESPONSE.length);
                            var responseArray = data.split(",");
                            if (responseArray[0] == "201") {
                                this.uploadButton.disabled = true;
                                this.uploadFileInput.value = "";
                                this.header.setStatus(responseArray[1], ViewHeaderLayout.SUCCESS);
                            } else {
                                this.uploadButton.disabled = false;
                                this.header.setStatus(responseArray[1], ViewHeaderLayout.ERROR);
                            }
                        } else {
                            this.uploadButton.disabled = false;
                            this.header.setStatus(Constants.getMessage("uploaderInvalidServerResponse"), ViewHeaderLayout.ERROR);
                        }
                    }
                }
            });
        }
    });
    
})();
