/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    Defines an interface for an object to receive form events. The consumer can register
//    an object with this interface to receive form events such as an attribute field value
//    changed by the user. The consumer must subclass from this interface class and overrides
//    all methods declared here to receive the appropriate messages.
//
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormListener");

dojo.declare(
  "com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormListener", null,
{
    // Summary:
    //    This message is invoked when an attribute field's value has been changed by the
    //    user. 
    //    @param attributeKey The attribute's key, the key of which the application used to create the
    //    attribute, 
    //    @param oldValue The previous value in the field.
    //    @param newValue The new value
    //
    //    @return Optionally, the application can return directives to the form to modify the
    //    the field's states as follows:
    //    {
    //       edited: <true, false>, 
    //       hasError: <true, false>,
    //       errorMessage: "string" if hasError is true, supplies the appropriate error message for the field.
    //    }
    //
    attributeValueChanged: function(attributeKey, oldValue, newValue)
    {
        // optionally returns these directives to affect the field's state (display)
        // return {hasError: true, errorMessage: "the value is no good!"};
    },
	
    attributeDeleteActionInvoked: function(attributeKey)
    {
    }
});

