/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.bf.utils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xml.type.XMLTypePackage;

import com.buildforge.services.client.dbo.Build;
import com.buildforge.services.client.dbo.Manifest;
import com.buildforge.services.client.dbo.Project;
import com.buildforge.services.common.dbo.ServerDBO;
import com.ibm.ccl.soa.deploy.core.CoreFactory;
import com.ibm.ccl.soa.deploy.core.DeployCoreRoot;
import com.ibm.ccl.soa.deploy.core.ExtendedAttribute;
import com.ibm.ccl.soa.deploy.core.HostingLink;
import com.ibm.ccl.soa.deploy.core.InstallState;
import com.ibm.ccl.soa.deploy.core.Topology;
import com.ibm.ccl.soa.deploy.core.Unit;
import com.ibm.ccl.soa.deploy.net.IPv6AddressNotationType;
import com.ibm.ccl.soa.deploy.net.IpInterface;
import com.ibm.ccl.soa.deploy.net.IpInterfaceUnit;
import com.ibm.ccl.soa.deploy.net.NetFactory;
import com.ibm.ccl.soa.deploy.operation.ExitStatus;
import com.ibm.ccl.soa.deploy.operation.Operation;
import com.ibm.ccl.soa.deploy.operation.OperationFactory;
import com.ibm.ccl.soa.deploy.operation.OperationUnit;
import com.ibm.ccl.soa.deploy.operation.RunState;
import com.ibm.ccl.soa.deploy.os.OperatingSystem;
import com.ibm.ccl.soa.deploy.os.OperatingSystemUnit;
import com.ibm.ccl.soa.deploy.os.OsFactory;
import com.ibm.ccl.soa.deploy.server.Server;
import com.ibm.ccl.soa.deploy.server.ServerFactory;
import com.ibm.ccl.soa.deploy.server.ServerUnit;
import com.ibm.rational.test.lm.spi.bf.AutomationService;
import com.ibm.rational.test.lm.spi.bf.common.DiscoveryMapping;



public class CreateTopology {

	private static final EDataType STRING_TYPE = XMLTypePackage.eINSTANCE
			.getString();
	private static final EDataType INTEGER_TYPE = XMLTypePackage.eINSTANCE
			.getInt();

	private static CreateTopology instance = null;

	private CreateTopology() {

	}

	public static CreateTopology getInstance() {
		if (instance == null)
			instance = new CreateTopology();
		return instance;
	}

	/**
	 * Create Server Unit
	 * 
	 * @param unitName
	 * @param serverName
	 * @param CPUArch
	 * @param CPUCount
	 * @param MemorySize
	 * @return
	 */
	private ServerUnit createServerUnit(String machineName, String machineId,
			Manifest manifest) {

		ServerUnit serverUnit = ServerFactory.eINSTANCE.createServerUnit();
		serverUnit.setInitInstallState(InstallState.UNKNOWN_LITERAL);

		Server server = ServerFactory.eINSTANCE.createServer();
		server.setName(machineName);
		server.setSystemId(machineId);

		// Set CPU Architecture
		String cpuArch = manifest.getPropertyValue(DiscoveryMapping.CPU_ARCH);
		server.setCpuArchitecture(cpuArch);
		// Set CPU Number
		String cpuCount = manifest.getPropertyValue(DiscoveryMapping.NUM_CPU);
		if ((cpuCount != null) && (!"".equals(cpuCount))) {//$NON-NLS-1$
			server.setCpuCount(new BigInteger(cpuCount));
		} else {
			server.setCpuCount(new BigInteger("-1"));//$NON-NLS-1$
		}
		// Set Total Memory
		String memorySize = manifest
				.getPropertyValue(DiscoveryMapping.MEM_TOTAL);
		if (memorySize != null && (!"".equals(memorySize))) {//$NON-NLS-1$
			server.setMemorySize(new BigInteger(memorySize));
		} else {
			server.setMemorySize(new BigInteger("-1"));//$NON-NLS-1$
		}
		// Set CPU Manufacture
		String cpuManufacture = manifest
				.getPropertyValue(DiscoveryMapping.CPU_MANUFACTURER);
		server.setCpuManufacturer(cpuManufacture);
		// Set CPU Speed
		String cpuSpeed = manifest.getPropertyValue(DiscoveryMapping.CPU_MHZ);
		if (cpuSpeed != null && (!"".equals(cpuSpeed))) {//$NON-NLS-1$
			server.setCpuSpeed(Integer.valueOf(cpuSpeed));
		} else {
			server.setCpuSpeed((-1));
		}
		// Set CPU Type
		String cpuType = manifest.getPropertyValue(DiscoveryMapping.CPU_MODEL);
		server.setCpuType(DiscoveryMapping.getRTLMCpuTypeForBfType(cpuType));
		serverUnit.getHostingOrAnyCapabilities().add(server);

		return serverUnit;
	}

	/**
	 * Create Windows OS Unit
	 * 
	 * @param unitName
	 * @param osName
	 * @param hostName
	 * @return
	 */
	private OperatingSystemUnit createOSUnit(Manifest currentManifest) {

		OperatingSystem os = OsFactory.eINSTANCE.createOperatingSystem();
		// Set OS host name
		os.setHostname(currentManifest
				.getPropertyValue(DiscoveryMapping.NET_HOSTNAME));
		// Set Os release as Distribution
		os.setOsDistribution(currentManifest
				.getPropertyValue(DiscoveryMapping.OS_RELEASE));
		// Set OS service pack
		os.setOsServicepack(currentManifest
				.getPropertyValue(DiscoveryMapping.WIN_SERVICEPACK));
		// Set OS type as os system name
		os.setOsType(currentManifest
				.getPropertyValue(DiscoveryMapping.OS_SYSNAME));
		os.setOsVersion(currentManifest
				.getPropertyValue(DiscoveryMapping.OS_VERSION));
		// Set OS fqdn
		os.setFqdn(currentManifest.getPropertyValue(DiscoveryMapping.NET_FQDN));
		OperatingSystemUnit osUnit = OsFactory.eINSTANCE
				.createOperatingSystemUnit();
		osUnit.setInitInstallState(InstallState.UNKNOWN_LITERAL);
		osUnit.getHostingOrAnyCapabilities().add(os);

		return osUnit;
	}

	/**
	 * Create hostLink between the host and hostee
	 * 
	 * @param host
	 * @param hostee
	 */
	private void createHostLink(Unit host, Unit hostee) {
		HostingLink hostingLink = CoreFactory.eINSTANCE.createHostingLink();
		hostingLink.setHosted(hostee);
		hostingLink.setName(host.getName() + "_hosting_" + hostee.getName());//$NON-NLS-1$
		host.getHostingLinks().add(hostingLink);
	}

	/**
	 * Create Server topology with build forge's manifest mapping The mapping's
	 * key is
	 * 
	 * @param assetMapping
	 * @return
	 */
	// TODO need move the function to spi.util package
	public String createServerTopology(HashMap<String, Manifest> assetMapping) {

		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Build Forge Server Topology");//$NON-NLS-1$
		Iterator<String> keyItr = assetMapping.keySet().iterator();
		while (keyItr.hasNext()) {
			String machineInfo = keyItr.next();
			String machineName = null;
			String machineId = null;
			String[] uriArray = machineInfo.toString().split(":"); //$NON-NLS-1$
			if (uriArray.length == 2) {
				machineName = uriArray[0];
				machineId = uriArray[1];
			}
			Manifest currentManifest = assetMapping.get(machineInfo);

			if (currentManifest != null) {
				// Add a server with OS
				ServerUnit server_unit = this.createServerUnit(machineName,
						machineId, currentManifest);
				top.getUnits().add(server_unit);

				// Add a OS
				OperatingSystemUnit os_unit = this
						.createOSUnit(currentManifest);
				top.getUnits().add(os_unit);

				// TODO Show creat IpInterface after resolving the net domain
				// stack
				// overflow problem

				IpInterfaceUnit ip_unit = this
						.createIpInterfaceUnit(currentManifest);
				top.getUnits().add(ip_unit);
				// Add a hostLink between os and ip
				this.createHostLink(os_unit, ip_unit);
				// Add a hostLink between server and OS
				this.createHostLink(server_unit, os_unit);
			}

		}

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");//$NON-NLS-1$
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}

	}

	/**
	 * Create IpInterface Unit
	 * 
	 * @param unitName
	 * @param ipAddress
	 * @return
	 */
	public static IpInterfaceUnit createIpInterfaceUnit(Manifest manifest) {
		IpInterfaceUnit ipInterfaceUnit = NetFactory.eINSTANCE
				.createIpInterfaceUnit();
		IpInterface ipInterface = NetFactory.eINSTANCE.createIpInterface();

		// Set IP version
		String net_version = manifest
				.getPropertyValue(DiscoveryMapping.NET_IPV);

		// Set the IpAddress -- differ IPV6 and IPV4
		if (DiscoveryMapping.IPV4.equals(net_version)) {
			ipInterface.setIpV4Address(manifest
					.getPropertyValue(DiscoveryMapping.NET_IPV + net_version));
		} else if (DiscoveryMapping.IPV6.equals(net_version)) {
			ipInterface.setIpV6Address(manifest
					.getPropertyValue(DiscoveryMapping.NET_IPV + net_version));
			// Set notation type to colon
			ipInterface
					.setIpv6NotationType(IPv6AddressNotationType.IPV6_COLON_NOTATION_LITERAL);
		}

		ipInterfaceUnit.getHostingOrAnyCapabilities().add(ipInterface);

		return ipInterfaceUnit;
	}

	// /**
	// * The serverName is like RTLM_hostname, we just need the host name
	// */
	// private String getHostname(String serverName) {
	// String hostname = "";//$NON-NLS-1$
	// String[] nameArray = serverName.toString().split("_");//$NON-NLS-1$
	// if (nameArray.length > 1)
	// hostname = nameArray[1];
	// return hostname;
	//
	// }

	/**
	 * Create server topology with server's id
	 * 
	 * @param serverId
	 * @return
	 */
	public String createServerTopology(List<String> serverIds) {
		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Build Forge Server Topology");//$NON-NLS-1$
		Iterator<String> iter = serverIds.iterator();
		while (iter.hasNext()) {
			String serverId = iter.next();
			ServerUnit serverUnit = ServerFactory.eINSTANCE.createServerUnit();
			serverUnit.setName("Server unit");//$NON-NLS-1$
			// serverUnit.setInitInstallState(InstallState.UNKNOWN_LITERAL);
			Server server = ServerFactory.eINSTANCE.createServer();
			server.setSystemId(serverId);
			serverUnit.getHostingOrAnyCapabilities().add(server);
			top.getUnits().add(serverUnit);
		}

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");//$NON-NLS-1$
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}

	}

	public String createServerStatusTopology(String serverId,  ServerDBO.Error err) {
		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Build Forge Server Topology");//$NON-NLS-1$

		ServerUnit serverUnit = ServerFactory.eINSTANCE.createServerUnit();
		serverUnit.setName("Server unit");//$NON-NLS-1$
		// serverUnit.setInitInstallState(InstallState.UNKNOWN_LITERAL);
		Server server = ServerFactory.eINSTANCE.createServer();
		server.setSystemId(serverId);
		ExtendedAttribute runState_EA = CoreFactory.eINSTANCE
				.createExtendedAttribute();
		runState_EA.setName("CurrentRunState");//$NON-NLS-1$
		
		if (err.equals(ServerDBO.Error.YES)) {
			runState_EA.setValue(RunState.RUNNING_LITERAL.getLiteral());
		} else if (err.equals(ServerDBO.Error.NO)){
			runState_EA.setValue(RunState.STOPPED_LITERAL.getLiteral());
		} else {
			runState_EA.setValue(RunState.WAITING_LITERAL.getLiteral());
		} 
		runState_EA.setValue(RunState.RUNNING_LITERAL.getLiteral());//$NON-NLS-1$
		serverUnit.getExtendedAttributes().add(runState_EA);
		serverUnit.getHostingOrAnyCapabilities().add(server);
		top.getUnits().add(serverUnit);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");//$NON-NLS-1$
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}
	}

	/**
	 * Convert topology to stream
	 * 
	 * @param m_topology
	 * @param os
	 */
	private void saveTopologyToStream(Topology m_topology, OutputStream os) {
		Resource res = m_topology.eResource();

		if (res == null) {
			// Topology isn't yet in a resource; make one for it. (If Topology
			// was in a resource it wouldn't be safe to add it to a new one
			// because that would
			// remove it from its current Resource).
			ResourceSet rs = new ResourceSetImpl();
			res = rs.createResource(URI.createURI("resource.extension"));//$NON-NLS-1$
			DeployCoreRoot root = CoreFactory.eINSTANCE.createDeployCoreRoot();
			root.setTopology(m_topology);
			res.getContents().add(root);
		}

		try {
			HashMap options = new HashMap();
			options.put(XMLResource.OPTION_EXTENDED_META_DATA, Boolean.TRUE);
			res.save(os, options);
		} catch (IOException ex) {
			// needs logging...
			ex.printStackTrace();
		} catch (IllegalArgumentException ex) {
			// TODO: CoreResourceImpl.doSave now also saves the
			// Markers. I suspect that if the topology file is not
			// in the workspace, i.e., accessible via a IFile object,
			// this exception is thrown. Workround to ignore the
			// exception for now. This does not affect the saving to
			// a file, or to a byte array.
		}
	}

	/**
	 * Create TDM Operation unit to send job definition information
	 * 
	 * @param project
	 * @return
	 */
	public static OperationUnit createJobDefUnit(Project project,
			HashMap<String, String> params) {

		EClassifier[] eclassifiers = new EClassifier[params.size()];
		String[] names = new String[params.size()];
		Iterator<String> paramNameItr = params.keySet().iterator();
		int i = 0;
		while (paramNameItr.hasNext()) {
			names[i] = paramNameItr.next();
			eclassifiers[i++] = STRING_TYPE;
		}

		OperationUnit jobDefUnit = OperationFactory.eINSTANCE
				.createOperationUnit("Jobs", eclassifiers,//$NON-NLS-1$
						names, STRING_TYPE);//$NON-NLS-1$
		assert jobDefUnit != null;

		jobDefUnit.getOperation().setOperationName(project.getName());
		jobDefUnit.getOperation().setOperationId(
				String.valueOf(project.getProjectId()));

		// Set params
		for (String name : names) {
			String value = params.get(name);
			jobDefUnit.getOperation().setParameterValue(name, value);
		}

		return jobDefUnit;
	}

	/**
	 * Create TDM Operation unit to send job result information
	 * 
	 * @param build
	 * @param project
	 * @return
	 */
	public static OperationUnit createJobResultOperationUnit(Build build,
			Project project) {

		OperationUnit jobResultUnit = OperationFactory.eINSTANCE
				.createOperationUnit("Build Forge Jobs", new EClassifier[] {},//$NON-NLS-1$
						new String[] {}, null);//$NON-NLS-1$
		assert jobResultUnit != null;

		// TDM doesn't have attribute of currentRunState, so we use goalRunState
		// instead temporary.

		
		switch (build.getResult()) {
		case FAILED:
			jobResultUnit.setExitStatus(ExitStatus.FAILED_LITERAL);
			if (build.getCancellation()){
				jobResultUnit.setExitStatus(ExitStatus.CANCELED_LITERAL);
			}
			break;
		case PASSED:
			jobResultUnit.setExitStatus(ExitStatus.SUCCESS_LITERAL);
			break;
		case WARNING_FAILED_STEP:
		case WARNING_FILTER:
			jobResultUnit.setExitStatus(ExitStatus.UNKNOWN_LITERAL);
			break;
		case RUNNING:
			jobResultUnit.setExitStatus(ExitStatus.WAITING_LITERAL);
			break;
		default:
			jobResultUnit.setExitStatus(ExitStatus.UNKNOWN_LITERAL);
		}
		

		Operation jobResultOp = jobResultUnit.getOperation();

		jobResultOp.setName(project.getName());
		jobResultOp.setOperationId(String.valueOf(project.getProjectId()));

		// Set start Time and duration Time
		Long startTime = build.getFired() * 1000L;
		Long duration = build.getDuration() * 1000L;
		String buildId = String.valueOf(build.getBuildId());

		ExtendedAttribute startTime_EA = CoreFactory.eINSTANCE
				.createExtendedAttribute();
		startTime_EA.setName("StartTime");//$NON-NLS-1$
		startTime_EA.setValue(startTime);
		jobResultOp.getParent().getExtendedAttributes().add(startTime_EA);

		ExtendedAttribute durationTime_EA = CoreFactory.eINSTANCE
				.createExtendedAttribute();
		durationTime_EA.setName("DurationTime");//$NON-NLS-1$
		durationTime_EA.setValue(duration);
		jobResultOp.getParent().getExtendedAttributes().add(durationTime_EA);

		ExtendedAttribute jobId_EA = CoreFactory.eINSTANCE
				.createExtendedAttribute();
		jobId_EA.setName("JobId");//$NON-NLS-1$
		jobId_EA.setValue(buildId);
		jobResultOp.getParent().getExtendedAttributes().add(jobId_EA);

		ExtendedAttribute runState_EA = CoreFactory.eINSTANCE
				.createExtendedAttribute();
		runState_EA.setName("CurrentRunState");//$NON-NLS-1$
		// Set current Run State by external attribute temporary
		switch (build.getState()) {
		case WAITING:
			runState_EA.setValue(RunState.WAITING);
			break;
		case PENDING:
			runState_EA.setValue(RunState.UNKNOWN);
			break;
		case RUNNING:
			runState_EA.setValue(RunState.RUNNING);//$NON-NLS-1$
			break;
		case COMPLETED:
			runState_EA.setValue(RunState.STOPPED);//$NON-NLS-1$
			break;
		case LOCKED:
		case ARCHIVED:
			runState_EA.setValue(RunState.UNKNOWN);//$NON-NLS-1$
			break;
		default:
			runState_EA.setValue(RunState.UNKNOWN);//$NON-NLS-1$

		}

		jobResultUnit.getExtendedAttributes().add(runState_EA);
		return jobResultUnit;
	}

	/**
	 * Create TDM operation unit to send job log information
	 * 
	 * @param job
	 * @return
	 */
	public static OperationUnit createJobLogOperationUnit(Build job) {
		OperationUnit jobLogUnit = OperationFactory.eINSTANCE
				.createOperationUnit(
						"Build Forge Job Log", new EClassifier[] {},//$NON-NLS-1$
						new String[] {}, null);//$NON-NLS-1$
		assert jobLogUnit != null;
		AutomationService autoService = AutomationService.INSTANCE;
		jobLogUnit.setOperationLog(autoService.getJobLog(job, new String[]{"EXEC", "ERROR"}));
		return jobLogUnit;
	}

	/**
	 * Create TDM topology
	 * 
	 * @param job
	 * @return
	 */
	public String createJobLogTopology(Build job) {
		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Job Definition Topology");//$NON-NLS-1$
		OperationUnit jobLogUnit = this.createJobLogOperationUnit(job);
		top.getUnits().add(jobLogUnit);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");//$NON-NLS-1$
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}
	}

	/**
	 * Create TDM topology
	 * 
	 * @param jobs
	 * @return
	 */
	public String createJobDefinitionTopology(
			HashMap<Project, HashMap<String, String>> projectWithParams) {

		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Job Definition Topology");//$NON-NLS-1$

		Iterator<Project> Itr = projectWithParams.keySet().iterator();

		while (Itr.hasNext()) {
			Project currentJob = Itr.next();
			HashMap<String, String> currentParams = projectWithParams
					.get(currentJob);

			OperationUnit jobDefUnit = this.createJobDefUnit(currentJob,
					currentParams);

			top.getUnits().add(jobDefUnit);
		}
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");//$NON-NLS-1$
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}

	}

	/**
	 * 
	 * @param builds
	 * @param projects
	 * @return
	 */
	public String createJobResultTopology(Build[] builds, Project[] projects) {

		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Build Forge Job Result Topology");//$NON-NLS-1$

		int i = 0;
		for (Build currentBuild : builds) {
			Project currentProj = projects[i++];
			OperationUnit jobResultUnit = this.createJobResultOperationUnit(
					currentBuild, currentProj);
			top.getUnits().add(jobResultUnit);
		}

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}
	}
	
	@SuppressWarnings("unchecked")
	public String createDiscoveryOperationTopology(String operationId) {
		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Build Forge Discovery Operation Topology");//$NON-NLS-1$
		
		OperationUnit discoveryOperationUnit = OperationFactory.eINSTANCE
				.createOperationUnit(
						"Build Forge Discovery Operation", new EClassifier[] {},//$NON-NLS-1$
						new String[] {}, null);//$NON-NLS-1$
		
		Operation operation = discoveryOperationUnit.getOperation();
		operation.setOperationId(operationId);
		
		top.getUnits().add(discoveryOperationUnit);
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}
	}
	
	@SuppressWarnings("unchecked")
	public String createDiscoveryOperationWithRunState(Integer runState, String[] externalIds) {
		Topology top = CoreFactory.eINSTANCE.createTopology();
		top.setDisplayName("Build Forge Discovery Operation Topology");//$NON-NLS-1$
		
		OperationUnit discoveryOperationUnit = OperationFactory.eINSTANCE
				.createOperationUnit(
						"Build Forge Discovery Operation", new EClassifier[] {},//$NON-NLS-1$
						new String[] {}, null);//$NON-NLS-1$
		// run state : stopped, running and suspended
		ExtendedAttribute runState_EA = CoreFactory.eINSTANCE
				.createExtendedAttribute();
		runState_EA.setName("CurrentRunState");//$NON-NLS-1$
		runState_EA.setValue(runState);
		discoveryOperationUnit.getExtendedAttributes().add(runState_EA);
		top.getUnits().add(discoveryOperationUnit);
		
		if (externalIds != null && externalIds.length > 0) {
			ServerUnit serverUnit = ServerFactory.eINSTANCE.createServerUnit();
			serverUnit.setName("Build Forge Discovery Operation Server Unit");
			for (String externalId : externalIds) {
				Server server = ServerFactory.eINSTANCE.createServer();
				server.setSystemId(externalId);
				serverUnit.getHostingOrAnyCapabilities().add(server);
			}
			top.getUnits().add(serverUnit);

			createHostLink(discoveryOperationUnit, serverUnit);
		}
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		saveTopologyToStream(top, baos);
		String xml = null;
		try {
			xml = baos.toString("UTF-8");
			return xml;
		} catch (UnsupportedEncodingException e) {
			return null;
		}
	}

}
